@@ -25,12 +25,13 @@ from senlin.api.openstack.v1 import policy_types
 from senlin.api.openstack.v1 import profile_types
 from senlin.api.openstack.v1 import profiles
 from senlin.api.openstack.v1 import receivers
+from senlin.api.openstack.v1 import services
 from senlin.api.openstack.v1 import version
 from senlin.api.openstack.v1 import webhooks
 
 
 class API(wsgi.Router):
-    '''WSGI router for Cluster v1 REST API requests.'''
+    """WSGI router for Cluster v1 REST API requests."""
 
     def __init__(self, conf, **local_conf):
         self.conf = conf
@@ -57,6 +58,10 @@ class API(wsgi.Router):
                                "/profile-types/{type_name}",
                                action="get",
                                conditions={'method': 'GET'})
+            sub_mapper.connect("profile_type_ops",
+                               "/profile-types/{type_name}/ops",
+                               action="ops",
+                               conditions={'method': 'GET'})
 
         # Profiles
         res = wsgi.Resource(profiles.ProfileController(conf))
@@ -167,6 +172,11 @@ class API(wsgi.Router):
                                action="delete",
                                conditions={'method': 'DELETE'},
                                success=202)
+            sub_mapper.connect("cluster_operation",
+                               "/clusters/{cluster_id}/ops",
+                               action="operation",
+                               conditions={'method': 'POST'},
+                               success=202)
 
         # Nodes
         res = wsgi.Resource(nodes.NodeController(conf))
@@ -181,6 +191,14 @@ class API(wsgi.Router):
                                action="create",
                                conditions={'method': 'POST'},
                                success=202)
+            sub_mapper.connect("node_adopt",
+                               "/nodes/adopt",
+                               action="adopt",
+                               conditions={'method': 'POST'})
+            sub_mapper.connect("node_adopt_preview",
+                               "/nodes/adopt-preview",
+                               action="adopt_preview",
+                               conditions={'method': 'POST'})
             sub_mapper.connect("node_get",
                                "/nodes/{node_id}",
                                action="get",
@@ -200,6 +218,11 @@ class API(wsgi.Router):
                                action="delete",
                                conditions={'method': 'DELETE'},
                                success=202)
+            sub_mapper.connect("node_operation",
+                               "/nodes/{node_id}/ops",
+                               action="operation",
+                               conditions={'method': 'POST'},
+                               success=202)
 
         # Cluster Policies
         res = wsgi.Resource(cluster_policies.ClusterPolicyController(conf))
@@ -250,6 +273,10 @@ class API(wsgi.Router):
                                "/receivers/{receiver_id}",
                                action="get",
                                conditions={'method': 'GET'})
+            sub_mapper.connect("receiver_update",
+                               "/receivers/{receiver_id}",
+                               action="update",
+                               conditions={'method': 'PATCH'})
             sub_mapper.connect("receiver_delete",
                                "/receivers/{receiver_id}",
                                action="delete",
@@ -292,3 +319,14 @@ class API(wsgi.Router):
                                conditions={'method': 'GET'})
 
         super(API, self).__init__(mapper)
+
+        # Services
+        res = wsgi.Resource(services.ServiceController(conf))
+        with mapper.submapper(controller=res) as sub_mapper:
+
+            sub_mapper.connect("service_index",
+                               "/services",
+                               action="index",
+                               conditions={'method': 'GET'})
+
+        super(API, self).__init__(mapper)
