@@ -30,18 +30,29 @@ CONF = cfg.CONF
 
 
 def do_db_version():
-    '''Print database's current migration level.'''
+    """Print database's current migration level."""
     print(api.db_version(api.get_engine()))
 
 
 def do_db_sync():
-    '''Place a database under migration control and upgrade.
+    """Place a database under migration control and upgrade.
 
     DB is created first if necessary.
-    '''
+    """
     api.db_sync(api.get_engine(), CONF.command.version)
 
 
+def do_event_purge():
+    """Purge the specified event records in senlin's database."""
+    if CONF.command.age < 0:
+        print(_("age must be a positive integer."))
+        return
+    api.event_purge(api.get_engine(),
+                    CONF.command.project_id,
+                    CONF.command.granularity,
+                    CONF.command.age)
+
+
 class ServiceManageCommand(object):
     def __init__(self):
         self.ctx = context.get_admin_context()
@@ -94,7 +105,7 @@ class ServiceManageCommand(object):
             svc = self._format_service(service)
             if svc['status'] == 'down':
                 print(_('Dead service %s is removed.') % svc['service_id'])
-                service_obj.Service.delete(self.ctx, svc['service_id'])
+                service_obj.Service.delete(svc['service_id'])
 
     @staticmethod
     def add_service_parsers(subparsers):
@@ -117,9 +128,39 @@ def add_command_parsers(subparsers):
     parser.add_argument('version', nargs='?')
     parser.add_argument('current_version', nargs='?')
 
+    parser = subparsers.add_parser('event_purge')
+    parser.set_defaults(func=do_event_purge)
+    parser.add_argument('-p',
+                        '--project-id',
+                        nargs='?',
+                        metavar='<project1;project2...>',
+                        help=_("Purge event records with specified project. "
+                               "This can be specified multiple times, or once "
+                               "with parameters separated by semicolon."),
+                        action='append')
+    parser.add_argument('-g',
+                        '--granularity',
+                        default='days',
+                        choices=['days', 'hours', 'minutes', 'seconds'],
+                        help=_("Purge event records which were created in the "
+                               "specified time period. The time is specified "
+                               "by age and granularity, whose value must be "
+                               "one of 'days', 'hours', 'minutes' or "
+                               "'seconds' (default)."))
+    parser.add_argument('age',
+                        type=int,
+                        default=30,
+                        help=_("Purge event records which were created in the "
+                               "specified time period. The time is specified "
+                               "by age and granularity. For example, "
+                               "granularity=hours and age=2 means purging "
+                               "events created two hours ago. Defaults to "
+                               "30."))
+
+
 command_opt = cfg.SubCommandOpt('command',
                                 title='Commands',
-                                help='Show available commands.',
+                                help=_('Show available commands.'),
                                 handler=add_command_parsers)
 
 
