@@ -10,13 +10,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-'''
+"""
 Utilities for scaling actions and related policies.
-'''
+"""
 
 import math
 import random
 
+from oslo_config import cfg
 from oslo_log import log as logging
 
 from senlin.common import consts
@@ -67,24 +68,24 @@ def truncate_desired(cluster, desired, min_size, max_size):
 
     if min_size is not None and desired < min_size:
         desired = min_size
-        LOG.debug(_("Truncating shrinkage to specified min_size (%s).")
-                  % desired)
+        LOG.debug("Truncating shrinkage to specified min_size (%s).",
+                  desired)
 
     if min_size is None and desired < cluster.min_size:
         desired = cluster.min_size
-        LOG.debug(_("Truncating shrinkage to cluster's min_size (%s).")
-                  % desired)
+        LOG.debug("Truncating shrinkage to cluster's min_size (%s).",
+                  desired)
 
     if max_size is not None and max_size > 0 and desired > max_size:
         desired = max_size
-        LOG.debug(_("Truncating growth to specified max_size (%s).")
-                  % desired)
+        LOG.debug("Truncating growth to specified max_size (%s).",
+                  desired)
 
     if (max_size is None and desired > cluster.max_size and
             cluster.max_size > 0):
         desired = cluster.max_size
-        LOG.debug(_("Truncating growth to cluster's max_size (%s).")
-                  % desired)
+        LOG.debug("Truncating growth to cluster's max_size (%s).",
+                  desired)
 
     return desired
 
@@ -109,8 +110,14 @@ def check_size_params(cluster=None, desired=None, min_size=None, max_size=None,
         the checking.
     """
 
+    max_nodes_per_cluster = cfg.CONF.max_nodes_per_cluster
     if desired is not None:
         # recalculate/validate desired based on strict setting
+        if desired > max_nodes_per_cluster:
+            v = {'d': desired, 'm': max_nodes_per_cluster}
+            return _("The target capacity (%(d)s) is greater than the "
+                     "maximum number of nodes allowed per cluster "
+                     "(%(m)s).") % v
         if (min_size is not None and desired < min_size):
             v = {'d': desired, 'm': min_size}
             return _("The target capacity (%(d)s) is less than "
@@ -154,6 +161,11 @@ def check_size_params(cluster=None, desired=None, min_size=None, max_size=None,
                      "current desired_capacity (%(d)s) of the cluster.") % v
 
     if max_size is not None:
+        if max_size > max_nodes_per_cluster:
+            v = {'m': max_size, 'mc': max_nodes_per_cluster}
+            return _("The specified max_size (%(m)s) is greater than the "
+                     "maximum number of nodes allowed per cluster "
+                     "(%(mc)s).") % v
         if (min_size is None and cluster is not None and
                 max_size >= 0 and max_size < cluster.min_size):
             v = {'m': max_size, 'n': cluster.min_size}
@@ -230,7 +242,6 @@ def filter_error_nodes(nodes):
     """Filter out ERROR nodes from the given node list.
 
     :param nodes: candidate nodes for filter.
-    :param count: maximum number of nodes for selection.
     :return: a tuple containing the chosen nodes' IDs and the undecided
              (good) nodes.
     """
@@ -238,7 +249,7 @@ def filter_error_nodes(nodes):
     bad = []
     not_created = []
     for n in nodes:
-        if n.status == 'ERROR':
+        if n.status == consts.NS_ERROR or n.status == consts.NS_WARNING:
             bad.append(n.id)
         elif n.created_at is None:
             not_created.append(n.id)
@@ -304,21 +315,13 @@ def nodes_by_profile_age(nodes, count):
     :param count: maximum number of nodes for selection.
     :return: a list of IDs for victim nodes.
     """
-    selected, nodes = filter_error_nodes(nodes)
+    selected, candidates = filter_error_nodes(nodes)
     if count <= len(selected):
         return selected[:count]
 
     count -= len(selected)
-    node_map = []
-    for node in nodes:
-        entry = {
-            'id': node.id,
-            'created_at': node.rt['profile'].created_at
-        }
-        node_map.append(entry)
-
-    sorted_map = sorted(node_map, key=lambda m: m['created_at'])
+    sorted_list = sorted(candidates, key=lambda n: n.profile_created_at)
     for i in range(count):
-        selected.append(sorted_map[i]['id'])
+        selected.append(sorted_list[i].id)
 
     return selected
