@@ -18,20 +18,21 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import strutils
 
-from senlin.common import exception
+from senlin.common import exception as exc
 from senlin.common.i18n import _
 
 LOG = logging.getLogger(__name__)
 
 
 class AnyIndexDict(collections.Mapping):
-    '''Convenience schema for a list.'''
+    """Convenience schema for a list."""
+
     def __init__(self, value):
         self.value = value
 
     def __getitem__(self, key):
         if key != '*' and not isinstance(key, six.integer_types):
-            raise KeyError(_('Invalid key %s') % str(key))
+            raise KeyError("Invalid key %s" % str(key))
 
         return self.value
 
@@ -64,9 +65,9 @@ class SchemaBase(collections.Mapping):
                  max_version=None):
         if schema is not None:
             if type(self) not in (List, Map, Operation):
-                msg = _('Schema valid only for List or Map, not '
-                        '"%s"') % self[self.TYPE]
-                raise exception.InvalidSchemaError(message=msg)
+                msg = _('Schema valid only for List or Map, not %s'
+                        ) % self[self.TYPE]
+                raise exc.ESchema(message=msg)
 
         if self[self.TYPE] == self.LIST:
             self.schema = AnyIndexDict(schema)
@@ -92,57 +93,38 @@ class SchemaBase(collections.Mapping):
             return
 
         try:
+            # NOTE: this is the subclass's version of 'validate'
             self.validate(self.default, context)
-        except (ValueError, TypeError) as exc:
-            raise exception.InvalidSchemaError(
-                message=_('Invalid default %(default)s (%(exc)s)') %
-                dict(default=self.default, exc=exc))
-
-    def validate(self, context=None):
-        """Validates the schema.
-
-        :param context: A RequestContext instance for deep validation.
-        """
-        self._validate_default(context)
-
-        # validated nested schema: List or Map
-        if self.schema:
-            if isinstance(self.schema, AnyIndexDict):
-                self.schema.value.validate(context)
-            else:
-                for nested_schema in self.schema.values():
-                    nested_schema.validate(context)
-
-    def validate_constraints(self, value, context=None, skipped=None):
-        if not skipped:
-            skipped = []
+        except (ValueError, TypeError) as ex:
+            msg = _('Invalid default %(default)s: %(exc)s'
+                    ) % dict(default=self.default, exc=ex)
+            raise exc.ESchema(message=msg)
 
+    def validate_constraints(self, value, schema=None, context=None):
         try:
             for constraint in self.constraints:
-                if type(constraint) not in skipped:
-                    constraint.validate(value, context)
+                constraint.validate(value, schema=schema, context=context)
         except ValueError as ex:
-            raise exception.SpecValidationFailed(message=six.text_type(ex))
+            raise exc.ESchema(message=six.text_type(ex))
 
-    def _validate_version(self, version, key):
+    def _validate_version(self, key, version):
         if self.min_version and self.min_version > version:
-            msg = _('%(key)s(min_version=%(min)s) is not supported by '
+            msg = _('%(key)s (min_version=%(min)s) is not supported by '
                     'spec version %(version)s.'
                     ) % {'key': key, 'min': self.min_version,
                          'version': version}
-            raise exception.SpecValidationFailed(message=msg)
+            raise exc.ESchema(message=msg)
         if self.max_version:
             if version > self.max_version:
-                msg = _('%(key)s(max_version=%(max)s) is not supported '
+                msg = _('%(key)s (max_version=%(max)s) is not supported '
                         'by spec version %(version)s.'
                         ) % {'version': version, 'max': self.max_version,
                              'key': key}
-                raise exception.SpecValidationFailed(message=msg)
+                raise exc.ESchema(message=msg)
             else:
-                msg = _('Warning: %(key)s will be deprecated after version '
-                        '%(version)s!') % {'key': key,
-                                           'version': self.max_version}
-                LOG.warning(msg)
+                LOG.warning('Warning: %(key)s will be deprecated after '
+                            'version %(version)s!',
+                            {'key': key, 'version': self.max_version})
 
     def __getitem__(self, key):
         if key == self.DESCRIPTION:
@@ -200,6 +182,8 @@ class PropertySchema(SchemaBase):
         self.updatable = updatable
 
     def __getitem__(self, key):
+        # NOTE: UPDATABLE is only applicable to some specs which may be
+        #       eligible for an update operation later
         if key == self.UPDATABLE:
             return self.updatable
 
@@ -207,20 +191,21 @@ class PropertySchema(SchemaBase):
 
 
 class Boolean(PropertySchema):
+
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.BOOLEAN
         return super(Boolean, self).__getitem__(key)
 
     def to_schema_type(self, value):
-        return strutils.bool_from_string(str(value), strict=True)
+        try:
+            return strutils.bool_from_string(str(value), strict=True)
+        except ValueError:
+            msg = _("The value '%s' is not a valid Boolean") % value
+            raise exc.ESchema(message=msg)
 
     def resolve(self, value):
-        if str(value).lower() not in ('true', 'false'):
-            msg = _('The value "%s" is not a valid Boolean') % value
-            raise exception.SpecValidationFailed(message=msg)
-
-        return strutils.bool_from_string(value, strict=True)
+        return self.to_schema_type(value)
 
     def validate(self, value, context=None):
         if isinstance(value, bool):
@@ -230,62 +215,66 @@ class Boolean(PropertySchema):
 
 
 class Integer(PropertySchema):
+
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.INTEGER
         return super(Integer, self).__getitem__(key)
 
     def to_schema_type(self, value):
+        if value is None:
+            return None
+
         if isinstance(value, six.integer_types):
             return value
+
         try:
             num = int(value)
         except ValueError:
-            raise ValueError(_('%s is not an integer.') % num)
+            msg = _("The value '%s' is not a valid Integer") % value
+            raise exc.ESchema(message=msg)
 
         return num
 
     def resolve(self, value):
-        try:
-            return int(value)
-        except (TypeError, ValueError):
-            msg = _('The value "%s" cannot be converted into an '
-                    'integer.') % value
-            raise exception.SpecValidationFailed(message=msg)
+        return self.to_schema_type(value)
 
     def validate(self, value, context=None):
         if not isinstance(value, six.integer_types):
             value = self.resolve(value)
 
-        self.validate_constraints(value, self, context)
+        self.validate_constraints(value, schema=self, context=context)
 
 
 class String(PropertySchema):
+
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.STRING
         return super(String, self).__getitem__(key)
 
     def to_schema_type(self, value):
-        return str(value)
+        try:
+            if isinstance(value, six.string_types):
+                return value
+            return str(value) if value is not None else None
+        except Exception:
+            raise
 
     def resolve(self, value):
-        try:
-            return str(value)
-        except (TypeError, ValueError):
-            raise
+        return self.to_schema_type(value)
 
     def validate(self, value, context=None):
         if not isinstance(value, six.string_types):
-            msg = _('The value "%s" cannot be converted into a '
-                    'string.') % value
-            raise exception.SpecValidationFailed(message=msg)
+            msg = _("The value '%s' is not a valid string.") % value
+            raise exc.ESchema(message=msg)
 
         self.resolve(value)
-        self.validate_constraints(value, self, context)
+        self.validate_constraints(value, schema=self, context=context)
 
 
 class Number(PropertySchema):
+
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.NUMBER
@@ -298,76 +287,61 @@ class Number(PropertySchema):
         try:
             return int(value)
         except ValueError:
-            return float(value)
+            try:
+                return float(value)
+            except ValueError:
+                msg = _("The value '%s' is not a valid number.") % value
+                raise exc.ESchema(message=msg)
 
     def resolve(self, value):
-        if isinstance(value, numbers.Number):
-            return value
-
-        try:
-            return int(value)
-        except ValueError:
-            return float(value)
+        return self.to_schema_type(value)
 
     def validate(self, value, context=None):
-        if isinstance(value, numbers.Number):
-            return
+        if not isinstance(value, numbers.Number):
+            value = self.resolve(value)
 
-        self.resolve(value)
-        self.resolve_constraints(value, self, context)
+        self.validate_constraints(value, schema=self, context=context)
 
 
 class List(PropertySchema):
+
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.LIST
         return super(List, self).__getitem__(key)
 
-    def _get_children(self, values, keys, context):
-        sub_schema = self.schema
-        if sub_schema is not None:
-            # We have a child schema specified for list elements
-            # Fake a dict of array elements, since we have only one schema
-            schema_arr = dict((k, sub_schema[k]) for k in keys)
-            subspec = Spec(schema_arr, dict(values))
-            subspec.validate()
-
-            return ((k, subspec[k]) for k in keys)
-        else:
-            return values
-
-    def get_default(self):
-        if not isinstance(self.default, collections.Sequence):
-            raise TypeError(_('"%s" is not a List') % self.default)
-
-        return self.default
+    def _get_children(self, values, context=None):
+        res = []
+        for i in range(len(values)):
+            res.append(self.schema[i].resolve(values[i]))
+        return res
 
     def resolve(self, value, context=None):
         if not isinstance(value, collections.Sequence):
             raise TypeError(_('"%s" is not a List') % value)
 
-        return [v[1] for v in self._get_children(enumerate(value),
-                                                 list(range(len(value))),
-                                                 context)]
+        return [v for v in self._get_children(value, context=context)]
 
     def validate(self, value, context=None):
-        if not isinstance(value, collections.Mapping):
-            raise TypeError(_('"%s" is not a Map') % value)
+        # if not isinstance(value, collections.Mapping):
+        if not isinstance(value, collections.Sequence):
+            msg = _("'%s' is not a List") % value
+            raise exc.ESchema(message=msg)
 
-        for key, child in self.schema.items():
-            item_value = value.get(key)
-            child.validate(item_value, context)
+        for v in value:
+            self.schema['*'].validate(v, context=context)
 
 
 class Map(PropertySchema):
+
     def __getitem__(self, key):
         if key == self.TYPE:
             return self.MAP
         return super(Map, self).__getitem__(key)
 
     def _get_children(self, values, context=None):
-        # There are cases where the Map is not specified to the very detailed
-        # levels, we treat them as valid specs as well.
+        # There are cases where the Map is not specified to the very
+        # detailed levels, we treat them as valid specs as well.
         if self.schema is None:
             return values
 
@@ -382,8 +356,12 @@ class Map(PropertySchema):
             return values
 
     def get_default(self):
+        if self.default is None:
+            return {}
+
         if not isinstance(self.default, collections.Mapping):
-            raise TypeError(_('"%s" is not a Map') % self.default)
+            msg = _("'%s' is not a Map") % self.default
+            raise exc.ESchema(message=msg)
 
         return self.default
 
@@ -392,16 +370,19 @@ class Map(PropertySchema):
             try:
                 value = jsonutils.loads(value)
             except (TypeError, ValueError):
-                pass
+                msg = _("'%s' is not a Map") % value
+                raise exc.ESchema(message=msg)
 
         if not isinstance(value, collections.Mapping):
-            raise TypeError(_('"%s" is not a Map') % value)
+            msg = _("'%s' is not a Map") % value
+            raise exc.ESchema(message=msg)
 
         return dict(self._get_children(value.items(), context))
 
     def validate(self, value, context=None):
         if not isinstance(value, collections.Mapping):
-            raise TypeError(_('"%s" is not a Map') % value)
+            msg = _("'%s' is not a Map") % value
+            raise exc.ESchema(message=msg)
 
         for key, child in self.schema.items():
             item_value = value.get(key)
@@ -409,45 +390,102 @@ class Map(PropertySchema):
 
 
 class StringParam(SchemaBase):
-    def _getitem__(self, key):
+
+    def __getitem__(self, key):
         if key == self.TYPE:
             return self.STRING
+        return super(StringParam, self).__getitem__(key)
+
+    def validate(self, value):
+        if not isinstance(value, six.string_types):
+            raise TypeError("value is not a string")
+
+        self.validate_constraints(value)
 
 
-class Operation(Map):
+class IntegerParam(SchemaBase):
+
+    def __getitem__(self, key):
+        if key == self.TYPE:
+            return self.INTEGER
+        return super(IntegerParam, self).__getitem__(key)
+
+    def validate(self, value):
+        try:
+            int(value)
+        except ValueError:
+            msg = _("The value '%s' is not a valid Integer") % value
+            raise ValueError(msg)
+
+        self.validate_constraints(value)
+
+
+class Operation(SchemaBase):
     """Class for specifying operations on profiles."""
-    pass
+
+    KEYS = (
+        DESCRIPTION, PARAMETERS,
+    ) = (
+        'description', 'parameters',
+    )
+
+    def __getitem__(self, key):
+        if key == self.DESCRIPTION:
+            return self.description or "Undocumented"
+        elif key == self.PARAMETERS:
+            if self.schema is None:
+                return {}
+            return dict((n, dict(s)) for n, s in self.schema.items())
+
+    def validate(self, data, version=None):
+        for k in data:
+            if k not in self.schema:
+                msg = _("Unrecognizable parameter '%s'") % k
+                raise exc.ESchema(message=msg)
+
+        for (k, s) in self.schema.items():
+            try:
+                if k in data:
+                    s.validate(data[k])
+                elif s.required:
+                    msg = _("Required parameter '%s' not provided") % k
+                    raise exc.ESchema(message=msg)
+
+                if version:
+                    s._validate_version(k, version)
+            except (TypeError, ValueError) as ex:
+                raise exc.ESchema(message=six.text_type(ex))
 
 
 class Spec(collections.Mapping):
-    '''A class that contains all spec items.'''
+    """A class that contains all spec items."""
+
     def __init__(self, schema, data, version=None):
         self._schema = schema
         self._data = data
         self._version = version
 
     def validate(self):
-        '''Validate the schema.'''
+        """Validate the schema."""
+
         for (k, s) in self._schema.items():
             try:
                 # Validate through resolve
                 self.resolve_value(k)
-                # Validate schema for versioned spec
+                # Validate schema for version
                 if self._version:
-                    self._schema[k]._validate_version(self._version, k)
+                    self._schema[k]._validate_version(k, self._version)
             except (TypeError, ValueError) as err:
-                msg = _('Spec validation error (%(key)s): %(err)s') % dict(
-                    key=k, err=six.text_type(err))
-                raise exception.SpecValidationFailed(message=msg)
+                raise exc.ESchema(message=six.text_type(err))
 
         for key in self._data:
             if key not in self._schema:
-                msg = _('Unrecognizable spec item "%s"') % key
-                raise exception.SpecValidationFailed(message=msg)
+                msg = _("Unrecognizable spec item '%s'") % key
+                raise exc.ESchema(message=msg)
 
     def resolve_value(self, key):
         if key not in self:
-            raise KeyError(_('Invalid spec item: "%s"') % key)
+            raise exc.ESchema(message="Invalid spec item: %s" % key)
 
         schema_item = self._schema[key]
         if key in self._data:
@@ -456,17 +494,18 @@ class Spec(collections.Mapping):
         elif schema_item.has_default():
             return schema_item.get_default()
         elif schema_item.required:
-            raise ValueError(_('Required spec item "%s" not assigned') % key)
+            msg = _("Required spec item '%s' not provided") % key
+            raise exc.ESchema(message=msg)
 
     def __getitem__(self, key):
-        '''Lazy evaluation for spec items.'''
+        """Lazy evaluation for spec items."""
         return self.resolve_value(key)
 
     def __len__(self):
-        '''Number of items in the spec.
+        """Number of items in the spec.
 
         A spec always contain all keys though some may be not specified.
-        '''
+        """
         return len(self._schema)
 
     def __contains__(self, key):
@@ -479,14 +518,14 @@ class Spec(collections.Mapping):
 def get_spec_version(spec):
     if not isinstance(spec, dict):
         msg = _('The provided spec is not a map.')
-        raise exception.SpecValidationFailed(message=msg)
+        raise exc.ESchema(message=msg)
 
     if 'type' not in spec:
         msg = _("The 'type' key is missing from the provided spec map.")
-        raise exception.SpecValidationFailed(message=msg)
+        raise exc.ESchema(message=msg)
 
     if 'version' not in spec:
         msg = _("The 'version' key is missing from the provided spec map.")
-        raise exception.SpecValidationFailed(message=msg)
+        raise exc.ESchema(message=msg)
 
     return (spec['type'], six.text_type(spec['version']))
