@@ -15,6 +15,7 @@ Common utilities module.
 """
 
 import random
+import re
 import string
 
 from jsonpath_rw import parse
@@ -29,7 +30,6 @@ from six.moves import urllib
 from senlin.common import consts
 from senlin.common import exception
 from senlin.common.i18n import _
-from senlin.common.i18n import _LI
 from senlin.objects import service as service_obj
 
 cfg.CONF.import_opt('max_response_size', 'senlin.common.config')
@@ -43,58 +43,18 @@ class URLFetchError(exception.Error, IOError):
     pass
 
 
-def parse_int_param(name, value, allow_zero=True, allow_negative=False,
-                    lower_limit=None, upper_limit=None):
-    if value is None:
-        return None
+def get_positive_int(v):
+    """Util function converting/checking a value of positive integer.
 
-    if value in ('0', 0):
-        if allow_zero:
-            return int(value)
-        raise exception.InvalidParameter(name=name, value=value)
-
-    try:
-        result = int(value)
-    except (TypeError, ValueError):
-        raise exception.InvalidParameter(name=name, value=value)
-    else:
-        if any([(allow_negative is False and result < 0),
-                (lower_limit and result < lower_limit),
-                (upper_limit and result > upper_limit)]):
-            raise exception.InvalidParameter(name=name, value=value)
-
-    return result
-
-
-def parse_bool_param(name, value):
-    if str(value).lower() not in ('true', 'false'):
-        raise exception.InvalidParameter(name=name, value=str(value))
-
-    return strutils.bool_from_string(value, strict=True)
-
-
-def validate_sort_param(value, whitelist):
-    """Validate a string value and see if it is a valid sort param.
-
-    :param value: A string as the input which should be one of the following
-                  formats:
-                  - 'key1,key2,key3'
-                  - 'key1:asc,key2,key3:desc'
-                  - 'key1:asc,key2:asc,key3:desc'
-    :param whitelist: A list of permitted sorting keys.
-    :return: None if validation succeeds or an exception of `InvalidParameter`
-             otherwise.
+    :param v: A value to be checked.
+    :returns: (b, v) where v is (converted) value if bool is True.
+              b is False if the value fails validation.
     """
-
-    if value is None:
-        return None
-
-    for s in value.split(','):
-        s_key, _sep, s_dir = s.partition(':')
-        if not s_key or s_key not in whitelist:
-            raise exception.InvalidParameter(name='sort key', value=s_key)
-        if s_dir and s_dir not in ('asc', 'desc'):
-            raise exception.InvalidParameter(name='sort dir', value=s_dir)
+    if strutils.is_int_like(v):
+        count = int(v)
+        if count > 0:
+            return True, count
+    return False, 0
 
 
 def parse_level_values(values):
@@ -117,15 +77,26 @@ def parse_level_values(values):
     return result
 
 
-def url_fetch(url, allowed_schemes=('http', 'https')):
-    '''Get the data at the specified URL.
+def level_from_number(value):
+    """Parse a given level value(from number to string).
+
+    :param value: event level number.
+    :return: A translated value.
+    """
+    n = int(value)
+    levels = {value: key for key, value in consts.EVENT_LEVELS.items()}
+    return levels.get(n, None)
+
+
+def url_fetch(url, allowed_schemes=('http', 'https'), verify=True):
+    """Get the data at the specified URL.
 
     The URL must use the http: or https: schemes.
     The file: scheme is also supported if you override
     the allowed_schemes argument.
     Raise an IOError if getting the data fails.
-    '''
-    LOG.info(_LI('Fetching data from %s'), url)
+    """
+    LOG.info('Fetching data from %s', url)
 
     components = urllib.parse.urlparse(url)
 
@@ -139,7 +110,7 @@ def url_fetch(url, allowed_schemes=('http', 'https')):
             raise URLFetchError(_('Failed to retrieve data: %s') % uex)
 
     try:
-        resp = requests.get(url, stream=True)
+        resp = requests.get(url, stream=True, verify=verify)
         resp.raise_for_status()
 
         # We cannot use resp.text here because it would download the entire
@@ -168,10 +139,54 @@ def random_name(length=8):
 
     lead = random.choice(string.ascii_letters)
     tail = ''.join(random.choice(string.ascii_letters + string.digits)
-                   for i in range(length - 1))
+                   for _ in range(length - 1))
     return lead + tail
 
 
+def format_node_name(fmt, cluster, index):
+    """Generates a node name using the given format.
+
+    :param fmt: A string containing format directives. Currently we only
+                support the following keys:
+                - "$nR": a random string with at most 'n' characters where
+                         'n' defaults to 8.
+                - "$nI": a string representation of the node index where 'n'
+                         instructs the number of digits generated with 0s
+                         padded to the left.
+    :param cluster: The DB object for the cluster to which the node belongs.
+                    This parameter is provided for future extension.
+    :param index: The index for the node in the target cluster.
+    :returns: A string containing the generated node name.
+    """
+    # for backward compatibility
+    if not fmt:
+        fmt = "node-$8R"
+
+    result = ""
+    last = 0
+    pattern = re.compile("(\$\d{0,8}[RI])")
+    for m in pattern.finditer(fmt):
+        group = m.group()
+        t = group[-1]
+        width = group[1:-1]
+        if t == "R":  # random string
+            if width != "":
+                sub = random_name(int(width))
+            else:
+                sub = random_name(8)
+        elif t == "I":  # node index
+            if width != "":
+                str_index = str(index)
+                sub = str_index.zfill(int(width))
+            else:
+                sub = str(index)
+        result += fmt[last:m.start()] + sub
+        last = m.end()
+    result += fmt[last:]
+
+    return result
+
+
 def isotime(at):
     """Stringify time in ISO 8601 format.
 
@@ -182,7 +197,7 @@ def isotime(at):
 
     st = at.strftime(_ISO8601_TIME_FORMAT)
     tz = at.tzinfo.tzname(None) if at.tzinfo else 'UTC'
-    st += ('Z' if tz == 'UTC' else tz)
+    st += ('Z' if tz == 'UTC' or tz == "UTC+00:00" else tz)
     return st
 
 
