@@ -10,11 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-'''
+"""
 Interface for database access.
 
 SQLAlchemy is currently the only supported backend.
-'''
+"""
 
 from oslo_config import cfg
 from oslo_db import api
@@ -31,10 +31,6 @@ def get_engine():
     return IMPL.get_engine()
 
 
-def get_session():
-    return IMPL.get_session()
-
-
 # Clusters
 def cluster_create(context, values):
     return IMPL.cluster_create(context, values)
@@ -102,11 +98,16 @@ def node_get_all(context, cluster_id=None, limit=None, marker=None, sort=None,
                              project_safe=project_safe)
 
 
-def node_get_all_by_cluster(context, cluster_id, project_safe=True):
-    return IMPL.node_get_all_by_cluster(context, cluster_id,
+def node_get_all_by_cluster(context, cluster_id, filters=None,
+                            project_safe=True):
+    return IMPL.node_get_all_by_cluster(context, cluster_id, filters=filters,
                                         project_safe=project_safe)
 
 
+def node_ids_by_cluster(context, cluster_id, filters=None):
+    return IMPL.node_ids_by_cluster(context, cluster_id, filters=None)
+
+
 def node_count_by_cluster(context, cluster_id, **kwargs):
     return IMPL.node_count_by_cluster(context, cluster_id, **kwargs)
 
@@ -190,11 +191,20 @@ def cluster_policy_get_all(context, cluster_id, filters=None, sort=None):
                                        sort=sort)
 
 
+def cluster_policy_ids_by_cluster(context, cluster_id):
+    return IMPL.cluster_policy_ids_by_cluster(context, cluster_id)
+
+
 def cluster_policy_get_by_type(context, cluster_id, policy_type, filters=None):
     return IMPL.cluster_policy_get_by_type(context, cluster_id, policy_type,
                                            filters=filters)
 
 
+def cluster_policy_get_by_name(context, cluster_id, policy_name, filters=None):
+    return IMPL.cluster_policy_get_by_name(context, cluster_id, policy_name,
+                                           filters=filters)
+
+
 def cluster_policy_attach(context, cluster_id, policy_id, values):
     return IMPL.cluster_policy_attach(context, cluster_id, policy_id, values)
 
@@ -335,8 +345,11 @@ def action_check_status(context, action_id, timestamp):
     return IMPL.action_check_status(context, action_id, timestamp)
 
 
-def action_delete_by_target(context, target, exceptions=None):
-    return IMPL.action_check_status(context, target, exceptions=exceptions)
+def action_delete_by_target(context, target, action=None,
+                            action_excluded=None, status=None):
+    return IMPL.action_delete_by_target(context, target, action=action,
+                                        action_excluded=action_excluded,
+                                        status=status)
 
 
 def dependency_add(context, depended, dependent):
@@ -355,6 +368,10 @@ def action_mark_succeeded(context, action_id, timestamp):
     return IMPL.action_mark_succeeded(context, action_id, timestamp)
 
 
+def action_mark_ready(context, action_id, timestamp):
+    return IMPL.action_mark_ready(context, action_id, timestamp)
+
+
 def action_mark_failed(context, action_id, timestamp, reason=None):
     return IMPL.action_mark_failed(context, action_id, timestamp, reason)
 
@@ -367,26 +384,30 @@ def action_acquire(context, action_id, owner, timestamp):
     return IMPL.action_acquire(context, action_id, owner, timestamp)
 
 
-def action_acquire_1st_ready(context, owner, timestamp):
-    return IMPL.action_acquire_1st_ready(context, owner, timestamp)
+def action_acquire_random_ready(context, owner, timestamp):
+    return IMPL.action_acquire_random_ready(context, owner, timestamp)
 
 
-def action_abandon(context, action_id):
-    return IMPL.action_abandon(context, action_id)
+def action_acquire_first_ready(context, owner, timestamp):
+    return IMPL.action_acquire_first_ready(context, owner, timestamp)
+
+
+def action_abandon(context, action_id, values=None):
+    return IMPL.action_abandon(context, action_id, values)
 
 
 def action_lock_check(context, action_id, owner=None):
-    '''Check whether an action has been locked(by a owner).'''
+    """Check whether an action has been locked(by an owner)."""
     return IMPL.action_lock_check(context, action_id, owner)
 
 
 def action_signal(context, action_id, value):
-    '''Send signal to an action via DB.'''
+    """Send signal to an action via DB."""
     return IMPL.action_signal(context, action_id, value)
 
 
 def action_signal_query(context, action_id):
-    '''Query signal status for the specified action.'''
+    """Query signal status for the specified action."""
     return IMPL.action_signal_query(context, action_id)
 
 
@@ -422,32 +443,43 @@ def receiver_delete(context, receiver_id):
     return IMPL.receiver_delete(context, receiver_id)
 
 
-def service_create(context, service_id, host=None, binary=None,
-                   topic=None):
-    return IMPL.service_create(context, service_id=service_id, host=host,
-                               binary=binary, topic=topic)
+def receiver_update(context, receiver_id, values):
+    return IMPL.receiver_update(context, receiver_id, values)
 
 
-def service_update(context, service_id, values=None):
-    return IMPL.service_update(context, service_id, values=values)
+def service_create(service_id, host=None, binary=None, topic=None):
+    return IMPL.service_create(service_id, host=host, binary=binary,
+                               topic=topic)
 
 
-def service_delete(context, service_id):
-    return IMPL.service_delete(context, service_id)
+def service_update(service_id, values=None):
+    return IMPL.service_update(service_id, values=values)
 
 
-def service_get(context, service_id):
-    return IMPL.service_get(context, service_id)
+def service_delete(service_id):
+    return IMPL.service_delete(service_id)
 
 
-def service_get_all(context):
-    return IMPL.service_get_all(context)
+def service_get(service_id):
+    return IMPL.service_get(service_id)
+
+
+def service_get_all():
+    return IMPL.service_get_all()
+
+
+def gc_by_engine(engine_id):
+    return IMPL.gc_by_engine(engine_id)
 
 
 def registry_create(context, cluster_id, check_type, interval, params,
-                    engine_id):
+                    engine_id, enabled=True):
     return IMPL.registry_create(context, cluster_id, check_type, interval,
-                                params, engine_id)
+                                params, engine_id, enabled=enabled)
+
+
+def registry_update(context, cluster_id, values):
+    return IMPL.registry_update(context, cluster_id, values)
 
 
 def registry_delete(context, cluster_id):
@@ -458,6 +490,14 @@ def registry_claim(context, engine_id):
     return IMPL.registry_claim(context, engine_id)
 
 
+def registry_get(context, cluster_id):
+    return IMPL.registry_get(context, cluster_id)
+
+
+def registry_get_by_param(context, params):
+    return IMPL.registry_get_by_param(context, params)
+
+
 def db_sync(engine, version=None):
     """Migrate the database to `version` or the most recent version."""
     return IMPL.db_sync(engine, version=version)
@@ -466,3 +506,8 @@ def db_sync(engine, version=None):
 def db_version(engine):
     """Display the current database version."""
     return IMPL.db_version(engine)
+
+
+def event_purge(engine, project, granularity, age):
+    """Purge the event records in database."""
+    return IMPL.event_purge(project, granularity, age)
