@@ -16,7 +16,7 @@ SQLAlchemy models for Senlin data.
 
 from oslo_db.sqlalchemy import models
 from oslo_utils import uuidutils
-from sqlalchemy import Boolean, Column, Float, ForeignKey, Integer
+from sqlalchemy import Boolean, Column, Numeric, ForeignKey, Integer
 from sqlalchemy import String, Text
 from sqlalchemy.ext import declarative
 from sqlalchemy.orm import backref
@@ -93,6 +93,7 @@ class Cluster(BASE, TimestampMixin, models.ModelBase):
     meta_data = Column(types.Dict)
     data = Column(types.Dict)
     dependents = Column(types.Dict)
+    config = Column(types.Dict)
 
 
 class Node(BASE, TimestampMixin, models.ModelBase):
@@ -119,6 +120,7 @@ class Node(BASE, TimestampMixin, models.ModelBase):
     meta_data = Column(types.Dict)
     data = Column(types.Dict)
     dependents = Column(types.Dict)
+    profile = relationship(Profile, backref=backref('nodes'))
 
 
 class ClusterLock(BASE, models.ModelBase):
@@ -167,6 +169,7 @@ class HealthRegistry(BASE, models.ModelBase):
     check_type = Column('check_type', String(255))
     interval = Column(Integer)
     params = Column(types.Dict)
+    enabled = Column(Boolean)
     engine_id = Column('engine_id', String(36))
 
 
@@ -225,9 +228,8 @@ class Action(BASE, TimestampMixin, models.ModelBase):
     cause = Column(String(255))
     owner = Column(String(36))
     interval = Column(Integer)
-    # FIXME: Don't specify fixed precision.
-    start_time = Column(Float(precision='24,8'))
-    end_time = Column(Float(precision='24,8'))
+    start_time = Column(Numeric(18, 6))
+    end_time = Column(Numeric(18, 6))
     timeout = Column(Integer)
     status = Column(String(255))
     status_reason = Column(Text)
