@@ -20,14 +20,14 @@ CONF = cfg.CONF
 
 
 class DriverBase(object):
-    '''Base class for all drivers.'''
+    """Base class for all drivers."""
 
     def __init__(self, params):
         self.conn_params = copy.deepcopy(params)
 
 
 class SenlinDriver(object):
-    '''Generic driver class'''
+    """Generic driver class"""
 
     def __init__(self, backend_name=None):
 
@@ -39,7 +39,9 @@ class SenlinDriver(object):
         self.compute = backend.compute
         self.loadbalancing = backend.loadbalancing
         self.network = backend.network
+        self.octavia = backend.octavia
         self.orchestration = backend.orchestration
-        self.telemetry = backend.telemetry
         self.identity = backend.identity
         self.message = backend.message
+        self.workflow = backend.workflow
+        self.block_storage = backend.block_storage
