@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import eventlet
 import six
 import time
 
@@ -17,36 +18,30 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import timeutils
 
+from senlin.common import consts
 from senlin.common import context as req_context
 from senlin.common import exception
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
 from senlin.common import utils
-from senlin.engine import cluster_policy as cp_mod
+from senlin.engine import dispatcher
 from senlin.engine import event as EVENT
 from senlin.objects import action as ao
+from senlin.objects import cluster_policy as cpo
 from senlin.objects import dependency as dobj
 from senlin.policies import base as policy_mod
 
 wallclock = time.time
 LOG = logging.getLogger(__name__)
 
-# Action causes
-CAUSES = (
-    CAUSE_RPC, CAUSE_DERIVED,
-) = (
-    'RPC Request',
-    'Derived Action',
-)
-
 
 class Action(object):
-    '''An action can be performed on a cluster or a node of a cluster.'''
+    """An action can be performed on a cluster or a node of a cluster."""
 
     RETURNS = (
         RES_OK, RES_ERROR, RES_RETRY, RES_CANCEL, RES_TIMEOUT,
+        RES_LIFECYCLE_COMPLETE, RES_LIFECYCLE_HOOK_TIMEOUT,
     ) = (
-        'OK', 'ERROR', 'RETRY', 'CANCEL', 'TIMEOUT',
+        'OK', 'ERROR', 'RETRY', 'CANCEL', 'TIMEOUT', 'LIFECYCLE_COMPLETE',
+        'LIFECYCLE_HOOK_TIMEOUT'
     )
 
     # Action status definitions:
@@ -59,10 +54,10 @@ class Action(object):
     #  CANCELLED: Action cancelled because worker thread was cancelled.
     STATUSES = (
         INIT, WAITING, READY, RUNNING, SUSPENDED,
-        SUCCEEDED, FAILED, CANCELLED
+        SUCCEEDED, FAILED, CANCELLED, WAITING_LIFECYCLE_COMPLETION
     ) = (
         'INIT', 'WAITING', 'READY', 'RUNNING', 'SUSPENDED',
-        'SUCCEEDED', 'FAILED', 'CANCELLED',
+        'SUCCEEDED', 'FAILED', 'CANCELLED', 'WAITING_LIFECYCLE_COMPLETION'
     )
 
     # Signal commands
@@ -72,7 +67,7 @@ class Action(object):
         'CANCEL', 'SUSPEND', 'RESUME',
     )
 
-    def __new__(cls, target, action, context, **kwargs):
+    def __new__(cls, target, action, ctx, **kwargs):
         if (cls != Action):
             return super(Action, cls).__new__(cls)
 
@@ -89,17 +84,17 @@ class Action(object):
 
         return super(Action, cls).__new__(ActionClass)
 
-    def __init__(self, target, action, context, **kwargs):
+    def __init__(self, target, action, ctx, **kwargs):
         # context will be persisted into database so that any worker thread
         # can pick the action up and execute it on behalf of the initiator
 
         self.id = kwargs.get('id', None)
         self.name = kwargs.get('name', '')
 
-        self.context = context
-        self.user = context.user
-        self.project = context.project
-        self.domain = context.domain
+        self.context = ctx
+        self.user = ctx.user_id
+        self.project = ctx.project_id
+        self.domain = ctx.domain_id
 
         self.action = action
         self.target = target
@@ -142,10 +137,10 @@ class Action(object):
 
         self.data = kwargs.get('data', {})
 
-    def store(self, context):
+    def store(self, ctx):
         """Store the action record into database table.
 
-        :param context: An instance of the request context.
+        :param ctx: An instance of the request context.
         :return: The ID of the stored object.
         """
 
@@ -177,11 +172,11 @@ class Action(object):
         if self.id:
             self.updated_at = timestamp
             values['updated_at'] = timestamp
-            ao.Action.update(context, self.id, values)
+            ao.Action.update(ctx, self.id, values)
         else:
             self.created_at = timestamp
             values['created_at'] = timestamp
-            action = ao.Action.create(context, values)
+            action = ao.Action.create(ctx, values)
             self.id = action.id
 
         return self.id
@@ -190,11 +185,10 @@ class Action(object):
     def _from_object(cls, obj):
         """Construct an action from database object.
 
-        :param context: the context used for DB operations;
         :param obj: a DB action object that contains all fields.
         :return: An `Action` object deserialized from the DB action object.
         """
-        context = req_context.RequestContext.from_dict(obj.context)
+        ctx = req_context.RequestContext.from_dict(obj.context)
         kwargs = {
             'id': obj.id,
             'name': obj.name,
@@ -213,115 +207,102 @@ class Action(object):
             'data': obj.data,
         }
 
-        return cls(obj.target, obj.action, context, **kwargs)
+        target_type = obj.action.split('_')[0]
+        if target_type == 'CLUSTER':
+            from senlin.engine.actions import cluster_action
+            ActionClass = cluster_action.ClusterAction
+        elif target_type == 'NODE':
+            from senlin.engine.actions import node_action
+            ActionClass = node_action.NodeAction
+        else:
+            from senlin.engine.actions import custom_action
+            ActionClass = custom_action.CustomAction
+
+        return ActionClass(obj.target, obj.action, ctx, **kwargs)
 
     @classmethod
-    def load(cls, context, action_id=None, db_action=None):
+    def load(cls, ctx, action_id=None, db_action=None, project_safe=True):
         """Retrieve an action from database.
 
-        :param context: Instance of request context.
+        :param ctx: Instance of request context.
         :param action_id: An UUID for the action to deserialize.
         :param db_action: An action object for the action to deserialize.
         :return: A `Action` object instance.
         """
         if db_action is None:
-            db_action = ao.Action.get(context, action_id)
+            db_action = ao.Action.get(ctx, action_id,
+                                      project_safe=project_safe)
             if db_action is None:
                 raise exception.ResourceNotFound(type='action', id=action_id)
 
         return cls._from_object(db_action)
 
     @classmethod
-    def load_all(cls, context, filters=None, limit=None, marker=None,
-                 sort=None, project_safe=True):
-        """Retrieve all actions of from database.
-
-        :param context: Instance of the request context.
-        :param filters: A dict of key-value pairs for filtering the resulted
-                        list of action objects.
-        :param limit: A number for restricting the number of records returned.
-        :param marker: The ID of the last seen action record. Only records
-                       that appear after this ID will be shown.
-        :param sort: A list of sorting keys (optionally with a sort dir),
-                     separated by commas.
-        :param project_safe: A boolean that indicates whether actions from
-                             other projects will be returned as well.
-        :return: A list of `Action` objects.
-        """
-
-        records = ao.Action.get_all(context, filters=filters, sort=sort,
-                                    limit=limit, marker=marker,
-                                    project_safe=project_safe)
-
-        for record in records:
-            yield cls._from_object(record)
-
-    @classmethod
-    def create(cls, context, target, action, **kwargs):
+    def create(cls, ctx, target, action, **kwargs):
         """Create an action object.
 
-        :param context: The requesting context.
+        :param ctx: The requesting context.
         :param target: The ID of the target cluster/node.
         :param action: Name of the action.
         :param dict kwargs: Other keyword arguments for the action.
         :return: ID of the action created.
         """
         params = {
-            'user': context.user,
-            'project': context.project,
-            'domain': context.domain,
-            'is_admin': context.is_admin,
-            'request_id': context.request_id,
-            'trusts': context.trusts,
+            'user_id': ctx.user_id,
+            'project_id': ctx.project_id,
+            'domain_id': ctx.domain_id,
+            'is_admin': ctx.is_admin,
+            'request_id': ctx.request_id,
+            'trusts': ctx.trusts,
         }
-        ctx = req_context.RequestContext.from_dict(params)
-        obj = cls(target, action, ctx, **kwargs)
-        return obj.store(context)
+        c = req_context.RequestContext.from_dict(params)
+        obj = cls(target, action, c, **kwargs)
+        return obj.store(ctx)
 
     @classmethod
-    def delete(cls, context, action_id):
+    def delete(cls, ctx, action_id):
         """Delete an action from database.
 
-        :param context: An instance of the request context.
+        :param ctx: An instance of the request context.
         :param action_id: The UUID of the target action to be deleted.
         :return: Nothing.
         """
-        ao.Action.delete(context, action_id)
+        ao.Action.delete(ctx, action_id)
 
     def signal(self, cmd):
-        '''Send a signal to the action.'''
+        """Send a signal to the action.
+
+        :param cmd: One of the command word defined in self.COMMANDS.
+        :returns: None
+        """
         if cmd not in self.COMMANDS:
             return
 
         if cmd == self.SIG_CANCEL:
-            expected_statuses = (self.INIT, self.WAITING, self.READY,
-                                 self.RUNNING)
+            expected = (self.INIT, self.WAITING, self.READY, self.RUNNING)
         elif cmd == self.SIG_SUSPEND:
-            expected_statuses = (self.RUNNING)
-        else:     # SIG_RESUME
-            expected_statuses = (self.SUSPENDED)
+            expected = (self.RUNNING)
+        else:  # SIG_RESUME
+            expected = (self.SUSPENDED)
 
-        if self.status not in expected_statuses:
-            reason = _("Action (%(action)s) is in unexpected status "
-                       "(%(actual)s) while expected status should be one of "
-                       "(%(expected)s).") % dict(action=self.id,
-                                                 expected=expected_statuses,
-                                                 actual=self.status)
-            EVENT.error(self.context, self, cmd, status_reason=reason)
+        if self.status not in expected:
+            LOG.error("Action (%(id)s) is in status (%(actual)s) while "
+                      "expected status must be one of (%(expected)s).",
+                      dict(id=self.id[:8], expected=expected,
+                           actual=self.status))
             return
 
-        # TODO(Yanyan Hu): use DB session here
         ao.Action.signal(self.context, self.id, cmd)
 
     def execute(self, **kwargs):
-        '''Execute the action.
+        """Execute the action.
 
         In theory, the action encapsulates all information needed for
         execution.  'kwargs' may specify additional parameters.
         :param kwargs: additional parameters that may override the default
                        properties stored in the action record.
-        '''
-        return NotImplemented
+        """
+        raise NotImplementedError
 
     def set_status(self, result, reason=None):
         """Set action status based on return value from execute."""
@@ -346,18 +327,36 @@ class Action(object):
             status = self.CANCELLED
             ao.Action.mark_cancelled(self.context, self.id, timestamp)
 
+        elif result == self.RES_LIFECYCLE_COMPLETE:
+            status = self.SUCCEEDED
+            ao.Action.mark_ready(self.context, self.id, timestamp)
+
         else:  # result == self.RES_RETRY:
-            status = self.READY
+            retries = self.data.get('retries', 0)
             # Action failed at the moment, but can be retried
-            # We abandon it and then notify other dispatchers to execute it
-            ao.Action.abandon(self.context, self.id)
+            # retries time is configurable
+            if retries < cfg.CONF.lock_retry_times:
+                status = self.READY
+                retries += 1
+
+                self.data.update({'retries': retries})
+                ao.Action.abandon(self.context, self.id, {'data': self.data})
+                # sleep for a while
+                eventlet.sleep(cfg.CONF.lock_retry_interval)
+                dispatcher.start_action(self.id)
+            else:
+                status = self.RES_ERROR
+                if not reason:
+                    reason = ('Exceeded maximum number of retries (%d)'
+                              '') % cfg.CONF.lock_retry_times
+                ao.Action.mark_failed(self.context, self.id, timestamp, reason)
 
         if status == self.SUCCEEDED:
-            EVENT.info(self.context, self, self.action, status, reason)
+            EVENT.info(self, consts.PHASE_END, reason or 'SUCCEEDED')
         elif status == self.READY:
-            EVENT.warning(self.context, self, self.action, status, reason)
+            EVENT.warning(self, consts.PHASE_ERROR, reason or 'RETRY')
         else:
-            EVENT.error(self.context, self, self.action, status, reason)
+            EVENT.error(self, consts.PHASE_ERROR, reason or 'ERROR')
 
         self.status = status
         self.status_reason = reason
@@ -368,14 +367,16 @@ class Action(object):
         self.status = status
         return status
 
-    def is_timeout(self):
-        time_lapse = wallclock() - self.start_time
-        return time_lapse > self.timeout
+    def is_timeout(self, timeout=None):
+        if timeout is None:
+            timeout = self.timeout
+        time_elapse = wallclock() - self.start_time
+        return time_elapse > timeout
 
     def _check_signal(self):
         # Check timeout first, if true, return timeout message
         if self.timeout is not None and self.is_timeout():
-            EVENT.debug(self.context, self, self.action, 'TIMEOUT')
+            EVENT.debug(self, consts.PHASE_ERROR, 'TIMEOUT')
             return self.RES_TIMEOUT
 
         result = ao.Action.signal_query(self.context, self.id)
@@ -391,29 +392,19 @@ class Action(object):
         return self._check_signal() == self.SIG_RESUME
 
     def _check_result(self, name):
-        """Check policy check status and generate event.
+        """Check policy status and generate event.
 
         :param name: Name of policy checked
         :return: True if the policy checking can be continued, or False if the
                  policy checking should be aborted.
         """
-        # Abort policy checking if failures found
-        status = 'CHECK ERROR'
-        reason = _("Failed policy '%(name)s': %(reason)s."
-                   ) % {'name': name, 'reason': self.data['reason']}
-        if self.data['status'] == policy_mod.CHECK_OK:
-            method = EVENT.debug
-            status = 'CHECK OK'
-            reason = self.data['reason']
-        else:
-            method = EVENT.error
-
-        method(self.context, self, self.action, status, reason)
-
+        reason = self.data['reason']
         if self.data['status'] == policy_mod.CHECK_OK:
             return True
-        else:
-            return False
+
+        self.data['reason'] = ("Failed policy '%(name)s': %(reason)s"
+                               ) % {'name': name, 'reason': reason}
+        return False
 
     def policy_check(self, cluster_id, target):
         """Check all policies attached to cluster and give result.
@@ -427,13 +418,12 @@ class Action(object):
         if target not in ['BEFORE', 'AFTER']:
             return
 
-        # TODO(Anyone): This could use the cluster's runtime data
-        bindings = cp_mod.ClusterPolicy.load_all(self.context, cluster_id,
-                                                 sort='priority',
-                                                 filters={'enabled': True})
+        bindings = cpo.ClusterPolicy.get_all(self.context, cluster_id,
+                                             sort='priority',
+                                             filters={'enabled': True})
         # default values
         self.data['status'] = policy_mod.CHECK_OK
-        self.data['reason'] = _('Completed policy checking.')
+        self.data['reason'] = 'Completed policy checking.'
 
         for pb in bindings:
             policy = policy_mod.Policy.load(self.context, pb.policy_id)
@@ -441,7 +431,10 @@ class Action(object):
             # cluster, no matter that policy is only interested in the
             # "BEFORE" or "AFTER" or both.
             if target == 'AFTER':
-                pb.record_last_op(self.context)
+                ts = timeutils.utcnow(True)
+                pb.last_op = ts
+                cpo.ClusterPolicy.update(self.context, pb.cluster_id,
+                                         pb.policy_id, {'last_op': ts})
 
             if not policy.need_check(target, self):
                 continue
@@ -454,9 +447,9 @@ class Action(object):
             if getattr(policy, 'cooldown', None):
                 if pb.cooldown_inprogress(policy.cooldown):
                     self.data['status'] = policy_mod.CHECK_ERROR
-                    self.data['reason'] = _('Policy %s cooldown is still '
-                                            'in progress.') % policy.id
-                return
+                    self.data['reason'] = ('Policy %s cooldown is still '
+                                           'in progress.') % policy.id
+                    return
 
             if method is not None:
                 method(cluster_id, self)
@@ -493,35 +486,37 @@ class Action(object):
             'created_at': utils.isotime(self.created_at),
             'updated_at': utils.isotime(self.updated_at),
             'data': self.data,
+            'user': self.user,
+            'project': self.project,
         }
         return action_dict
 
 
-# TODO(Yanyan Hu): Replace context parameter with session parameter
-def ActionProc(context, action_id):
-    '''Action process.'''
+def ActionProc(ctx, action_id):
+    """Action process."""
 
     # Step 1: materialize the action object
-    action = Action.load(context, action_id=action_id)
+    action = Action.load(ctx, action_id=action_id, project_safe=False)
     if action is None:
-        LOG.error(_LE('Action "%s" could not be found.'), action_id)
+        LOG.error('Action "%s" could not be found.', action_id)
         return False
 
-    # TODO(Anyone): Remove context usage in event module
-    EVENT.info(action.context, action, action.action, 'START')
+    EVENT.info(action, consts.PHASE_START, action_id[:8])
 
     reason = 'Action completed'
     success = True
     try:
         # Step 2: execute the action
         result, reason = action.execute()
+        if result == action.RES_RETRY:
+            success = False
     except Exception as ex:
         # We catch exception here to make sure the following logics are
         # executed.
         result = action.RES_ERROR
         reason = six.text_type(ex)
-        LOG.exception(_('Unexpected exception occurred during action '
-                        '%(action)s (%(id)s) execution: %(reason)s'),
+        LOG.exception('Unexpected exception occurred during action '
+                      '%(action)s (%(id)s) execution: %(reason)s',
                       {'action': action.action, 'id': action.id,
                        'reason': reason})
         success = False
