@@ -12,30 +12,26 @@
 
 import eventlet
 
-from senlin.common.i18n import _
+from oslo_log import log as logging
+from osprofiler import profiler
+
+from senlin.common import consts
 from senlin.common import scaleutils as su
 from senlin.engine.actions import base
 from senlin.engine import cluster as cm
 from senlin.engine import event as EVENT
 from senlin.engine import node as node_mod
 from senlin.engine import senlin_lock
+from senlin.objects import action as ao
 from senlin.objects import node as no
 from senlin.policies import base as pb
 
+LOG = logging.getLogger(__name__)
+
 
 class NodeAction(base.Action):
     """An action that can be performed on a cluster member (node)."""
 
-    ACTIONS = (
-        NODE_CREATE, NODE_DELETE, NODE_UPDATE,
-        NODE_JOIN, NODE_LEAVE,
-        NODE_CHECK, NODE_RECOVER
-    ) = (
-        'NODE_CREATE', 'NODE_DELETE', 'NODE_UPDATE',
-        'NODE_JOIN', 'NODE_LEAVE',
-        'NODE_CHECK', 'NODE_RECOVER'
-    )
-
     def __init__(self, target, action, context, **kwargs):
         """Constructor for a node action object.
 
@@ -49,48 +45,50 @@ class NodeAction(base.Action):
         super(NodeAction, self).__init__(target, action, context, **kwargs)
 
         try:
-            self.node = node_mod.Node.load(self.context, node_id=self.target)
+            self.entity = node_mod.Node.load(self.context, node_id=self.target)
         except Exception:
-            self.node = None
+            self.entity = None
 
+    @profiler.trace('NodeAction.do_create', hide_args=False)
     def do_create(self):
         """Handler for the NODE_CREATE action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        cluster_id = self.node.cluster_id
-        if cluster_id and self.cause == base.CAUSE_RPC:
+        cluster_id = self.entity.cluster_id
+        if cluster_id and self.cause == consts.CAUSE_RPC:
             # Check cluster size constraint if target cluster is specified
             cluster = cm.Cluster.load(self.context, cluster_id)
-            current = no.Node.count_by_cluster(self.context, cluster_id)
-            desired = current + 1
+            desired = no.Node.count_by_cluster(self.context, cluster_id)
             result = su.check_size_params(cluster, desired, None, None, True)
             if result:
                 # cannot place node into the cluster
-                no.Node.update(self.context, self.node.id, {'cluster_id': ''})
+                no.Node.update(self.context, self.entity.id,
+                               {'cluster_id': '', 'status': consts.NS_ERROR})
                 return self.RES_ERROR, result
 
-        res = self.node.do_create(self.context)
+        res = self.entity.do_create(self.context)
 
-        if cluster_id and self.cause == base.CAUSE_RPC:
+        if cluster_id and self.cause == consts.CAUSE_RPC:
             # Update cluster's desired_capacity and re-evaluate its status no
             # matter the creation is a success or not because the node object
-            # is # already treated as member of the cluster and the node
+            # is already treated as member of the cluster and the node
             # creation may have changed the cluster's status
-            cluster.eval_status(self.context, self.NODE_CREATE,
+            cluster.eval_status(self.context, consts.NODE_CREATE,
                                 desired_capacity=desired)
         if res:
-            return self.RES_OK, _('Node created successfully.')
+            return self.RES_OK, 'Node created successfully.'
         else:
-            return self.RES_ERROR, _('Node creation failed.')
+            return self.RES_ERROR, 'Node creation failed.'
 
+    @profiler.trace('NodeAction.do_delete', hide_args=False)
     def do_delete(self):
         """Handler for the NODE_DELETE action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        cluster_id = self.node.cluster_id
-        if cluster_id and self.cause == base.CAUSE_RPC:
+        cluster_id = self.entity.cluster_id
+        if cluster_id and self.cause == consts.CAUSE_RPC:
             # If node belongs to a cluster, check size constraint
             # before deleting it
             cluster = cm.Cluster.load(self.context, cluster_id)
@@ -107,9 +105,9 @@ class NodeAction(base.Action):
                 if grace_period:
                     eventlet.sleep(grace_period)
 
-        res = self.node.do_delete(self.context)
+        res = self.entity.do_delete(self.context)
 
-        if cluster_id and self.cause == base.CAUSE_RPC:
+        if cluster_id and self.cause == consts.CAUSE_RPC:
             # check if desired_capacity should be changed
             do_reduce = True
             params = {}
@@ -118,25 +116,44 @@ class NodeAction(base.Action):
                 do_reduce = pd.get('reduce_desired_capacity', True)
             if do_reduce and res:
                 params = {'desired_capacity': desired}
-            cluster.eval_status(self.context, self.NODE_DELETE, **params)
+            cluster.eval_status(self.context, consts.NODE_DELETE, **params)
 
-        if res:
-            return self.RES_OK, _('Node deleted successfully.')
-        else:
-            return self.RES_ERROR, _('Node deletion failed.')
+        if not res:
+            return self.RES_ERROR, 'Node deletion failed.'
 
+        # Remove all action records which target on deleted
+        # node except the on-going NODE_DELETE action from DB
+        try:
+            ao.Action.delete_by_target(
+                self.context, self.target,
+                action_excluded=[consts.NODE_DELETE],
+                status=[consts.ACTION_SUCCEEDED, consts.ACTION_FAILED])
+        except Exception as ex:
+            LOG.warning('Failed to clean node action records: %s',
+                        ex)
+        return self.RES_OK, 'Node deleted successfully.'
+
+    @profiler.trace('NodeAction.do_update', hide_args=False)
     def do_update(self):
         """Handler for the NODE_UPDATE action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
         params = self.inputs
-        res = self.node.do_update(self.context, params)
-        if res:
-            return self.RES_OK, _('Node updated successfully.')
-        else:
-            return self.RES_ERROR, _('Node update failed.')
+        new_profile_id = params.get('new_profile_id', None)
+        if new_profile_id and new_profile_id == self.entity.profile_id:
+            params.pop('new_profile_id')
 
+        if not params:
+            return self.RES_OK, 'No property to update.'
+
+        res = self.entity.do_update(self.context, params)
+        if res:
+            return self.RES_OK, 'Node updated successfully.'
+        else:
+            return self.RES_ERROR, 'Node update failed.'
+
+    @profiler.trace('NodeAction.do_join', hide_args=False)
     def do_join(self):
         """Handler for the NODE_JOIN action.
 
@@ -148,19 +165,13 @@ class NodeAction(base.Action):
         :returns: A tuple containing the result and the corresponding reason.
         """
         cluster_id = self.inputs.get('cluster_id')
-        # Check the size constraint of parent cluster
-        cluster = cm.Cluster.load(self.context, cluster_id)
-        current = no.Node.count_by_cluster(self.context, cluster_id)
-        result = su.check_size_params(cluster, current + 1, None, None, True)
+        result = self.entity.do_join(self.context, cluster_id)
         if result:
-            return self.RES_ERROR, result
-
-        result = self.node.do_join(self.context, cluster_id)
-        if result:
-            return self.RES_OK, _('Node successfully joined cluster.')
+            return self.RES_OK, 'Node successfully joined cluster.'
         else:
-            return self.RES_ERROR, _('Node failed in joining cluster.')
+            return self.RES_ERROR, 'Node failed in joining cluster.'
 
+    @profiler.trace('NodeAction.do_leave', hide_args=False)
     def do_leave(self):
         """Handler for the NODE_LEAVE action.
 
@@ -171,40 +182,48 @@ class NodeAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        # Check the size constraint of parent cluster
-        cluster = cm.Cluster.load(self.context, self.node.cluster_id)
-        current = no.Node.count_by_cluster(self.context, self.node.cluster_id)
-        result = su.check_size_params(cluster, current - 1, None, None, True)
-        if result:
-            return self.RES_ERROR, result
-
-        res = self.node.do_leave(self.context)
+        res = self.entity.do_leave(self.context)
         if res:
-            return self.RES_OK, _('Node successfully left cluster.')
+            return self.RES_OK, 'Node successfully left cluster.'
         else:
-            return self.RES_ERROR, _('Node failed in leaving cluster.')
+            return self.RES_ERROR, 'Node failed in leaving cluster.'
 
+    @profiler.trace('NodeAction.do_check', hide_args=False)
     def do_check(self):
         """Handler for the NODE_check action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        res = self.node.do_check(self.context)
+        res = self.entity.do_check(self.context)
         if res:
-            return self.RES_OK, _('Node status is ACTIVE.')
+            return self.RES_OK, 'Node check succeeded.'
         else:
-            return self.RES_ERROR, _('Node status is not ACTIVE.')
+            return self.RES_ERROR, 'Node check failed.'
 
+    @profiler.trace('NodeAction.do_recover', hide_args=False)
     def do_recover(self):
         """Handler for the NODE_RECOVER action.
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        res = self.node.do_recover(self.context, **self.inputs)
+        res = self.entity.do_recover(self.context, self)
         if res:
-            return self.RES_OK, _('Node recovered successfully.')
+            return self.RES_OK, 'Node recovered successfully.'
         else:
-            return self.RES_ERROR, _('Node recover failed.')
+            return self.RES_ERROR, 'Node recover failed.'
+
+    @profiler.trace('NodeAction.do_operation', hide_args=False)
+    def do_operation(self):
+        """Handler for the NODE_OPERATION action.
+
+        :returns: A tuple containing the result and the corresponding reason.
+        """
+        operation = self.inputs['operation']
+        res = self.entity.do_operation(self.context, **self.inputs)
+        if res:
+            return self.RES_OK, "Node operation '%s' succeeded." % operation
+        else:
+            return self.RES_ERROR, "Node operation '%s' failed." % operation
 
     def _execute(self):
         """Private function that finds out the handler and execute it."""
@@ -214,8 +233,8 @@ class NodeAction(base.Action):
         method = getattr(self, method_name, None)
 
         if method is None:
-            reason = _('Unsupported action: %s') % self.action
-            EVENT.error(self.context, self.node, self.action, 'Failed', reason)
+            reason = 'Unsupported action: %s' % self.action
+            EVENT.error(self, consts.PHASE_ERROR, reason)
             return self.RES_ERROR, reason
 
         return method()
@@ -228,33 +247,36 @@ class NodeAction(base.Action):
         """
         # Since node.cluster_id could be reset to '' during action execution,
         # we record it here for policy check and cluster lock release.
-        saved_cluster_id = self.node.cluster_id
-        if (saved_cluster_id and self.cause == base.CAUSE_RPC):
-            res = senlin_lock.cluster_lock_acquire(
-                self.context, self.node.cluster_id, self.id, self.owner,
-                senlin_lock.NODE_SCOPE, False)
+        saved_cluster_id = self.entity.cluster_id
+        if saved_cluster_id:
+            if self.cause == consts.CAUSE_RPC:
+                res = senlin_lock.cluster_lock_acquire(
+                    self.context, self.entity.cluster_id, self.id, self.owner,
+                    senlin_lock.NODE_SCOPE, False)
 
-            if not res:
-                return self.RES_RETRY, _('Failed in locking cluster')
+                if not res:
+                    return self.RES_RETRY, 'Failed in locking cluster'
 
-            self.policy_check(self.node.cluster_id, 'BEFORE')
-            if self.data['status'] != pb.CHECK_OK:
-                # Don't emit message since policy_check should have done it
-                senlin_lock.cluster_lock_release(saved_cluster_id, self.id,
-                                                 senlin_lock.NODE_SCOPE)
-                return self.RES_ERROR, 'Policy check: ' + self.data['reason']
+                self.policy_check(self.entity.cluster_id, 'BEFORE')
+                if self.data['status'] != pb.CHECK_OK:
+                    # Don't emit message since policy_check should have done it
+                    senlin_lock.cluster_lock_release(saved_cluster_id, self.id,
+                                                     senlin_lock.NODE_SCOPE)
+                    return self.RES_ERROR, ('Policy check: ' +
+                                            self.data['reason'])
+            elif self.cause == consts.CAUSE_DERIVED_LCH:
+                self.policy_check(self.entity.cluster_id, 'BEFORE')
 
-        reason = ''
         try:
-            res = senlin_lock.node_lock_acquire(self.context, self.node.id,
+            res = senlin_lock.node_lock_acquire(self.context, self.entity.id,
                                                 self.id, self.owner, False)
             if not res:
-                res = self.RES_ERROR
-                reason = _('Failed in locking node')
+                res = self.RES_RETRY
+                reason = 'Failed in locking node'
             else:
                 res, reason = self._execute()
                 if (res == self.RES_OK and saved_cluster_id and
-                        self.cause == base.CAUSE_RPC):
+                        self.cause == consts.CAUSE_RPC):
                     self.policy_check(saved_cluster_id, 'AFTER')
                     if self.data['status'] != pb.CHECK_OK:
                         res = self.RES_ERROR
@@ -262,8 +284,8 @@ class NodeAction(base.Action):
                     else:
                         res = self.RES_OK
         finally:
-            senlin_lock.node_lock_release(self.node.id, self.id)
-            if saved_cluster_id and self.cause == base.CAUSE_RPC:
+            senlin_lock.node_lock_release(self.entity.id, self.id)
+            if saved_cluster_id and self.cause == consts.CAUSE_RPC:
                 senlin_lock.cluster_lock_release(saved_cluster_id, self.id,
                                                  senlin_lock.NODE_SCOPE)
         return res, reason
