@@ -11,12 +11,12 @@
 # under the License.
 
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 import six
 
+from senlin.common import consts
 from senlin.common import exception as exc
-from senlin.common.i18n import _
-from senlin.common.i18n import _LE
 from senlin.common import utils
 from senlin.objects import node as no
 from senlin.profiles import base as pb
@@ -32,14 +32,6 @@ class Node(object):
     executed.
     """
 
-    statuses = (
-        INIT, ACTIVE, ERROR, WARNING, CREATING, UPDATING, DELETING,
-        RECOVERING
-    ) = (
-        'INIT', 'ACTIVE', 'ERROR', 'WARNING', 'CREATING', 'UPDATING',
-        'DELETING', 'RECOVERING'
-    )
-
     def __init__(self, name, profile_id, cluster_id=None, context=None,
                  **kwargs):
         self.id = kwargs.get('id', None)
@@ -66,7 +58,7 @@ class Node(object):
         self.created_at = kwargs.get('created_at', None)
         self.updated_at = kwargs.get('updated_at', None)
 
-        self.status = kwargs.get('status', self.INIT)
+        self.status = kwargs.get('status', consts.NS_INIT)
         self.status_reason = kwargs.get('status_reason', 'Initializing')
         self.data = kwargs.get('data', {})
         self.metadata = kwargs.get('metadata', {})
@@ -75,11 +67,11 @@ class Node(object):
 
         if context is not None:
             if self.user == '':
-                self.user = context.user
+                self.user = context.user_id
             if self.project == '':
-                self.project = context.project
+                self.project = context.project_id
             if self.domain == '':
-                self.domain = context.domain
+                self.domain = context.domain_id
             self._load_runtime_data(context)
 
     def _load_runtime_data(self, context):
@@ -88,7 +80,7 @@ class Node(object):
             profile = pb.Profile.load(context, profile_id=self.profile_id,
                                       project_safe=False)
         except exc.ResourceNotFound:
-            LOG.debug(_('Profile not found: %s'), self.profile_id)
+            LOG.debug('Profile not found: %s', self.profile_id)
 
         self.rt = {'profile': profile}
 
@@ -138,7 +130,7 @@ class Node(object):
         """Construct a node from node object.
 
         @param context: the context used for DB operations;
-        @param node: a node object that contains all fields;
+        @param obj: a node object that contains all fields;
         """
         kwargs = {
             'id': obj.id,
@@ -163,7 +155,7 @@ class Node(object):
 
     @classmethod
     def load(cls, context, node_id=None, db_node=None, project_safe=True):
-        '''Retrieve a node from database.'''
+        """Retrieve a node from database."""
         if db_node is None:
             db_node = no.Node.get(context, node_id, project_safe=project_safe)
             if db_node is None:
@@ -174,41 +166,15 @@ class Node(object):
     @classmethod
     def load_all(cls, context, cluster_id=None, limit=None, marker=None,
                  sort=None, filters=None, project_safe=True):
-        '''Retrieve all nodes of from database.'''
+        """Retrieve all nodes of from database."""
         objs = no.Node.get_all(context, cluster_id=cluster_id,
                                filters=filters, sort=sort,
                                limit=limit, marker=marker,
                                project_safe=project_safe)
 
-        return [cls._from_object(context, obj) for obj in objs]
-
-    def to_dict(self):
-        if self.rt['profile']:
-            profile_name = self.rt['profile'].name
-        else:
-            profile_name = 'Unknown'
-        node_dict = {
-            'id': self.id,
-            'name': self.name,
-            'cluster_id': self.cluster_id,
-            'physical_id': self.physical_id,
-            'profile_id': self.profile_id,
-            'user': self.user,
-            'project': self.project,
-            'domain': self.domain,
-            'index': self.index,
-            'role': self.role,
-            'init_at': utils.isotime(self.init_at),
-            'created_at': utils.isotime(self.created_at),
-            'updated_at': utils.isotime(self.updated_at),
-            'status': self.status,
-            'status_reason': self.status_reason,
-            'data': self.data,
-            'metadata': self.metadata,
-            'dependents': self.dependents,
-            'profile_name': profile_name,
-        }
-        return node_dict
+        for obj in objs:
+            node = cls._from_object(context, obj)
+            yield node
 
     def set_status(self, context, status, reason=None, **params):
         """Set status of the node.
@@ -221,10 +187,10 @@ class Node(object):
         """
         values = {}
         now = timeutils.utcnow(True)
-        if status == self.ACTIVE and self.status == self.CREATING:
+        if status == consts.NS_ACTIVE and self.status == consts.NS_CREATING:
             self.created_at = values['created_at'] = now
-        elif status == self.ACTIVE and self.status == self.UPDATING:
-            # TODO(Qiming): update_at should be changed unconditionally
+        if status not in [consts.NS_CREATING, consts.NS_UPDATING,
+                          consts.NS_RECOVERING, consts.NS_OPERATING]:
             self.updated_at = values['updated_at'] = now
 
         self.status = status
@@ -242,49 +208,37 @@ class Node(object):
             return {}
         return pb.Profile.get_details(context, self)
 
-    def update_dependents(self, context, dependents):
-        """Update dependency information of node's property.
-
-        :param context: An instance of request context.
-        :param dependents: The dependency information.
-        """
-
-        values = {'dependents': dependents}
-        no.Node.update(context, self.id, values)
-
     def do_create(self, context):
-        if self.status != self.INIT:
-            LOG.error(_LE('Node is in status "%s"'), self.status)
+        if self.status != consts.NS_INIT:
+            LOG.error('Node is in status "%s"', self.status)
             return False
 
-        self.set_status(context, self.CREATING, _('Creation in progress'))
+        self.set_status(context, consts.NS_CREATING, 'Creation in progress')
         try:
             physical_id = pb.Profile.create_object(context, self)
         except exc.EResourceCreation as ex:
-            self.set_status(context, self.ERROR, six.text_type(ex))
+            physical_id = ex.resource_id
+            self.set_status(context, consts.NS_ERROR, six.text_type(ex),
+                            physical_id=physical_id)
             return False
 
-        self.set_status(context, self.ACTIVE, _('Creation succeeded'),
+        self.set_status(context, consts.NS_ACTIVE, 'Creation succeeded',
                         physical_id=physical_id)
         return True
 
     def do_delete(self, context):
-        if not self.physical_id:
-            no.Node.delete(context, self.id)
-            return True
-
-        # TODO(Qiming): check if actions are working on it and can be canceled
-        self.set_status(context, self.DELETING, 'Deletion in progress')
+        self.set_status(context, consts.NS_DELETING, 'Deletion in progress')
         try:
             # The following operation always return True unless exception
             # is thrown
             pb.Profile.delete_object(context, self)
-            no.Node.delete(context, self.id)
-            return True
         except exc.EResourceDeletion as ex:
-            self.set_status(context, self.ERROR, six.text_type(ex))
+            self.set_status(context, consts.NS_ERROR, six.text_type(ex))
             return False
 
+        no.Node.delete(context, self.id)
+        return True
+
     def do_update(self, context, params):
         """Update a node's property.
 
@@ -295,7 +249,7 @@ class Node(object):
         if not self.physical_id:
             return False
 
-        self.set_status(context, self.UPDATING, 'Update in progress')
+        self.set_status(context, consts.NS_UPDATING, 'Update in progress')
 
         new_profile_id = params.pop('new_profile_id', None)
         res = True
@@ -304,7 +258,7 @@ class Node(object):
                 res = pb.Profile.update_object(context, self, new_profile_id,
                                                **params)
             except exc.EResourceUpdate as ex:
-                self.set_status(context, self.ERROR, six.text_type(ex))
+                self.set_status(context, consts.NS_ERROR, six.text_type(ex))
                 return False
 
         # update was not successful
@@ -318,7 +272,7 @@ class Node(object):
             self.rt['profile'] = pb.Profile.load(context,
                                                  profile_id=new_profile_id)
 
-        self.set_status(context, self.ACTIVE, 'Update succeeded', **props)
+        self.set_status(context, consts.NS_ACTIVE, 'Update succeeded', **props)
 
         return True
 
@@ -326,84 +280,171 @@ class Node(object):
         if self.cluster_id == cluster_id:
             return True
 
+        try:
+            res = pb.Profile.join_cluster(context, self, cluster_id)
+        except exc.EResourceUpdate as ex:
+            LOG.error('Node join cluster faild: %s.', ex)
+            return False
+
+        if not res:
+            return False
         timestamp = timeutils.utcnow(True)
         db_node = no.Node.migrate(context, self.id, cluster_id, timestamp)
         self.cluster_id = cluster_id
         self.updated_at = timestamp
         self.index = db_node.index
-
-        res = pb.Profile.join_cluster(context, self, cluster_id)
-        if res:
-            return True
-
-        # rollback
-        db_node = no.Node.migrate(context, self.id, None, timestamp)
-        self.cluster_id = ''
-        self.updated_at = timestamp
-        self.index = -1
-
-        return False
+        return True
 
     def do_leave(self, context):
         if self.cluster_id == '':
             return True
 
-        res = pb.Profile.leave_cluster(context, self)
-        if res:
-            timestamp = timeutils.utcnow(True)
-            no.Node.migrate(context, self.id, None, timestamp)
-            self.cluster_id = ''
-            self.updated_at = timestamp
-            self.index = -1
-            return True
-        else:
+        try:
+            res = pb.Profile.leave_cluster(context, self)
+        except exc.EResourceDeletion as ex:
+            LOG.error('Node leave cluster faild: %s.', ex)
             return False
 
+        if not res:
+            return False
+        timestamp = timeutils.utcnow(True)
+        no.Node.migrate(context, self.id, None, timestamp)
+        self.cluster_id = ''
+        self.updated_at = timestamp
+        self.index = -1
+        return True
+
     def do_check(self, context):
         if not self.physical_id:
             return False
 
-        res = True
         try:
             res = pb.Profile.check_object(context, self)
+        except exc.EServerNotFound as ex:
+            self.set_status(context, consts.NS_ERROR, six.text_type(ex),
+                            physical_id=None)
+            return True
         except exc.EResourceOperation as ex:
-            self.set_status(context, self.ERROR, six.text_type(ex))
+            self.set_status(context, consts.NS_ERROR, six.text_type(ex))
             return False
 
+        # Physical object is ACTIVE but for some reason the node status in
+        # senlin was WARNING. We only update the status_reason
         if res:
-            self.set_status(context, self.ACTIVE,
-                            _("Check: Node is ACTIVE."))
+            if self.status == consts.NS_WARNING:
+                msg = ("Check: Physical object is ACTIVE but the node status "
+                       "was WARNING. %s") % self.status_reason
+                self.set_status(context, consts.NS_WARNING, msg)
+                return True
+
+            self.set_status(context, consts.NS_ACTIVE,
+                            "Check: Node is ACTIVE.")
         else:
-            self.set_status(context, self.ERROR,
-                            _("Check: Node is not ACTIVE."))
+            self.set_status(context, consts.NS_ERROR,
+                            "Check: Node is not ACTIVE.")
 
-        return res
+        return True
 
-    def do_recover(self, context, **options):
+    def do_recover(self, context, action):
         """recover a node.
 
         This function is supposed to be invoked from a NODE_RECOVER action.
+        :param context: The request context of the action.
+        :param dict options: A map containing the recovery actions (with
+            parameters if any) and fencing settings.
         """
         if not self.physical_id:
             return False
 
-        self.set_status(context, self.RECOVERING,
-                        reason=_('Recover in progress'))
+        options = action.inputs
+        if options.get('check', False):
+            res = False
+            try:
+                res = pb.Profile.check_object(context, self)
+            except exc.EResourceOperation:
+                pass
+
+            if res:
+                self.set_status(context, consts.NS_ACTIVE,
+                                reason="Recover: Node is ACTIVE.")
+                return True
+
+        self.set_status(context, consts.NS_RECOVERING,
+                        reason='Recovery in progress')
 
         try:
-            physical_id = pb.Profile.recover_object(context, self, **options)
+            physical_id, status = pb.Profile.recover_object(context,
+                                                            self, **options)
         except exc.EResourceOperation as ex:
-            self.set_status(context, self.ERROR, reason=six.text_type(ex))
+            self.set_status(context, consts.NS_ERROR, reason=six.text_type(ex))
             return False
 
-        if not physical_id:
-            self.set_status(context, self.ERROR, reason=_('Recover failed'))
+        if not status:
+            self.set_status(context, consts.NS_ERROR, reason='Recovery failed')
             return False
 
         params = {}
-        if self.physical_id != physical_id:
+        if physical_id and self.physical_id != physical_id:
+            self.data['recovery'] = consts.RECOVER_RECREATE
+            params['data'] = self.data
             params['physical_id'] = physical_id
-        self.set_status(context, self.ACTIVE, reason=_('Recover succeeded'),
-                        **params)
+        self.set_status(context, consts.NS_ACTIVE,
+                        reason='Recovery succeeded', **params)
 
         return True
+
+    def do_operation(self, context, **inputs):
+        """Perform an operation on a node.
+
+        :param context: The request context.
+        :param dict inputs: The operation and parameters if any.
+        :returns: A boolean indicating whether the operation was a success.
+        """
+        if not self.physical_id:
+            return False
+
+        op = inputs['operation']
+        params = inputs.get('params', {})
+        self.set_status(context, consts.NS_OPERATING,
+                        reason="Operation '%s' in progress" % op)
+
+        try:
+            profile = self.rt['profile']
+            method = getattr(profile, 'handle_' + op)
+            method(self, **params)
+        except exc.EResourceOperation as ex:
+            self.set_status(context, consts.NS_ERROR, reason=six.text_type(ex))
+            return False
+
+        self.set_status(context, consts.NS_ACTIVE,
+                        reason="Operation '%s' succeeded" % op)
+        return True
+
+    def run_workflow(self, **options):
+        if not self.physical_id:
+            return False
+
+        workflow_name = options.pop('workflow_name')
+        inputs = options.pop('inputs')
+        definition = inputs.pop('definition', None)
+        params = {
+            'cluster_id': self.cluster_id,
+            'node_id': self.physical_id,
+        }
+        params.update(inputs)
+
+        try:
+            profile = self.rt['profile']
+            wfc = profile.workflow(self)
+            workflow = wfc.workflow_find(workflow_name)
+            if workflow is None:
+                wfc.workflow_create(definition, scope="private")
+            else:
+                definition = workflow.definition
+            inputs_str = jsonutils.dumps(params)
+            wfc.execution_create(workflow_name, str(inputs_str))
+        except exc.InternalError as ex:
+            raise exc.EResourceOperation(op='executing', type='workflow',
+                                         id=workflow_name,
+                                         message=six.text_type(ex))
+        return True
