@@ -13,8 +13,8 @@
 """
 Policy for load-balancing among nodes in a cluster.
 
-NOTE: For full documentation about how the deletion policy works, check:
-http://docs.openstack.org/developer/senlin/developer/policies/
+NOTE: For full documentation about how the load-balancing policy works,
+check: https://docs.openstack.org/senlin/latest/contributor/policies/
 load_balance_v1.html
 """
 
@@ -24,12 +24,10 @@ from oslo_log import log as logging
 from senlin.common import constraints
 from senlin.common import consts
 from senlin.common import exception as exc
-from senlin.common.i18n import _, _LW
+from senlin.common.i18n import _
 from senlin.common import scaleutils
 from senlin.common import schema
 from senlin.engine import cluster_policy
-from senlin.engine import node as nm
-from senlin.objects import cluster as co
 from senlin.objects import node as no
 from senlin.policies import base
 
@@ -45,6 +43,14 @@ class LoadBalancingPolicy(base.Policy):
     performed.
     """
     VERSION = '1.1'
+    VERSIONS = {
+        '1.0': [
+            {'status': consts.SUPPORTED, 'since': '2016.04'}
+        ],
+        '1.1': [
+            {'status': consts.SUPPORTED, 'since': '2018.01'}
+        ],
+    }
 
     PRIORITY = 500
 
@@ -52,6 +58,7 @@ class LoadBalancingPolicy(base.Policy):
         ('AFTER', consts.CLUSTER_ADD_NODES),
         ('AFTER', consts.CLUSTER_SCALE_OUT),
         ('AFTER', consts.CLUSTER_RESIZE),
+        ('AFTER', consts.NODE_RECOVER),
         ('AFTER', consts.NODE_CREATE),
         ('BEFORE', consts.CLUSTER_DEL_NODES),
         ('BEFORE', consts.CLUSTER_SCALE_IN),
@@ -64,17 +71,17 @@ class LoadBalancingPolicy(base.Policy):
     ]
 
     KEYS = (
-        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT
+        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT, LOADBALANCER
     ) = (
-        'pool', 'vip', 'health_monitor', 'lb_status_timeout'
+        'pool', 'vip', 'health_monitor', 'lb_status_timeout', 'loadbalancer'
     )
 
     _POOL_KEYS = (
         POOL_PROTOCOL, POOL_PROTOCOL_PORT, POOL_SUBNET,
-        POOL_LB_METHOD, POOL_ADMIN_STATE_UP, POOL_SESSION_PERSISTENCE,
+        POOL_LB_METHOD, POOL_ADMIN_STATE_UP, POOL_SESSION_PERSISTENCE, POOL_ID,
     ) = (
         'protocol', 'protocol_port', 'subnet',
-        'lb_method', 'admin_state_up', 'session_persistence',
+        'lb_method', 'admin_state_up', 'session_persistence', 'id',
     )
 
     PROTOCOLS = (
@@ -111,10 +118,10 @@ class LoadBalancingPolicy(base.Policy):
 
     HEALTH_MONITOR_KEYS = (
         HM_TYPE, HM_DELAY, HM_TIMEOUT, HM_MAX_RETRIES, HM_ADMIN_STATE_UP,
-        HM_HTTP_METHOD, HM_URL_PATH, HM_EXPECTED_CODES,
+        HM_HTTP_METHOD, HM_URL_PATH, HM_EXPECTED_CODES, HM_ID,
     ) = (
         'type', 'delay', 'timeout', 'max_retries', 'admin_state_up',
-        'http_method', 'url_path', 'expected_codes',
+        'http_method', 'url_path', 'expected_codes', 'id',
     )
 
     _SESSION_PERSISTENCE_KEYS = (
@@ -175,6 +182,11 @@ class LoadBalancingPolicy(base.Policy):
                     },
                     default={},
                 ),
+                POOL_ID: schema.String(
+                    _('ID of pool for the cluster on which nodes can '
+                      'be connected.'),
+                    default=None,
+                ),
             },
         ),
         VIP: schema.Map(
@@ -254,14 +266,21 @@ class LoadBalancingPolicy(base.Policy):
                 HM_EXPECTED_CODES: schema.String(
                     _('Expected HTTP codes for a passing HTTP(S) monitor.'),
                 ),
+                HM_ID: schema.String(
+                    _('ID of the health manager for the loadbalancer.'),
+                    default=None,
+                ),
             },
         ),
         LB_STATUS_TIMEOUT: schema.Integer(
-            _('Time in second to wait for loadbalancer to be ready'
-              '(provisioning_status is ACTIVE and operating_status is '
-              'ONLINE) before and after senlin requests lbaas V2 service '
-              'for lb operations. '),
-            default=600,
+            _('Time in second to wait for loadbalancer to become ready '
+              'after senlin requests LBaaS V2 service for operations.'),
+            default=300,
+        ),
+        LOADBALANCER: schema.String(
+            _('Name or ID of loadbalancer for the cluster on which nodes can '
+              'be connected.'),
+            default=None,
         )
     }
 
@@ -272,7 +291,7 @@ class LoadBalancingPolicy(base.Policy):
         self.vip_spec = self.properties.get(self.VIP, {})
         self.hm_spec = self.properties.get(self.HEALTH_MONITOR, None)
         self.lb_status_timeout = self.properties.get(self.LB_STATUS_TIMEOUT)
-        self.lb = None
+        self.lb = self.properties.get(self.LOADBALANCER, None)
 
     def validate(self, context, validate_props=False):
         super(LoadBalancingPolicy, self).validate(context, validate_props)
@@ -280,7 +299,8 @@ class LoadBalancingPolicy(base.Policy):
         if not validate_props:
             return True
 
-        nc = self.network(context.user, context.project)
+        nc = self.network(context.user_id, context.project_id)
+        oc = self.octavia(context.user_id, context.project_id)
 
         # validate pool subnet
         name_or_id = self.pool_spec.get(self.POOL_SUBNET)
@@ -300,10 +320,20 @@ class LoadBalancingPolicy(base.Policy):
                     ) % {'key': self.VIP_SUBNET, 'value': name_or_id}
             raise exc.InvalidSpec(message=msg)
 
-    def attach(self, cluster):
+        # validate loadbalancer
+        if self.lb:
+            try:
+                oc.loadbalancer_get(self.lb)
+            except exc.InternalError:
+                msg = _("The specified %(key)s '%(value)s' could not be found."
+                        ) % {'key': self.LOADBALANCER, 'value': self.lb}
+                raise exc.InvalidSpec(message=msg)
+
+    def attach(self, cluster, enabled=True):
         """Routine to be invoked when policy is to be attached to a cluster.
 
-        :param cluster: The target cluster to be attached to;
+        :param cluster: The cluster to which the policy is being attached to.
+        :param enabled: The attached cluster policy is enabled or disabled.
         :returns: When the operation was successful, returns a tuple (True,
                   message); otherwise, return a tuple (False, error).
         """
@@ -311,21 +341,30 @@ class LoadBalancingPolicy(base.Policy):
         if res is False:
             return False, data
 
-        nodes = nm.Node.load_all(oslo_context.get_current(),
-                                 cluster_id=cluster.id)
-
         lb_driver = self.lbaas(cluster.user, cluster.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
 
-        res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
-                                        self.hm_spec)
-        if res is False:
-            return False, data
+        # TODO(Anyone): Check if existing nodes has conflicts regarding the
+        # subnets. Each VM addresses detail has a key named to the network
+        # which can be used for validation.
+        if self.lb:
+            data = {}
+            data['preexisting'] = True
+            data['loadbalancer'] = self.lb
+            data['pool'] = self.pool_spec.get(self.POOL_ID, None)
+            data['vip_address'] = self.vip_spec.get(self.VIP_ADDRESS, None)
+            if self.hm_spec and self.hm_spec.get(self.HM_ID, None):
+                data['healthmonitor'] = self.hm_spec.get(self.HM_ID)
+        else:
+            res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
+                                            self.hm_spec)
+            if res is False:
+                return False, data
 
         port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
         subnet = self.pool_spec.get(self.POOL_SUBNET)
 
-        for node in nodes:
+        for node in cluster.nodes:
             member_id = lb_driver.member_add(node, data['loadbalancer'],
                                              data['pool'], port, subnet)
             if member_id is None:
@@ -333,11 +372,13 @@ class LoadBalancingPolicy(base.Policy):
                 # were created and return the failure reason.
                 # TODO(anyone): May need to "roll-back" changes caused by any
                 # successful member_add() calls.
-                lb_driver.lb_delete(**data)
+                if not self.lb:
+                    lb_driver.lb_delete(**data)
                 return False, 'Failed in adding node into lb pool'
 
             node.data.update({'lb_member': member_id})
-            node.store(oslo_context.get_current())
+            values = {'data': node.data}
+            no.Node.update(oslo_context.get_current(), node.id, values)
 
         cluster_data_lb = cluster.data.get('loadbalancers', {})
         cluster_data_lb[self.id] = {'vip_address': data.pop('vip_address')}
@@ -354,7 +395,7 @@ class LoadBalancingPolicy(base.Policy):
         :returns: When the operation was successful, returns a tuple of
             (True, data) where the data contains references to the resources
             created; otherwise returns a tuple of (False, err) where the err
-            contains a error message.
+            contains an error message.
         """
         reason = _('LB resources deletion succeeded.')
         lb_driver = self.lbaas(cluster.user, cluster.project)
@@ -367,16 +408,26 @@ class LoadBalancingPolicy(base.Policy):
         if policy_data is None:
             return True, reason
 
-        res, reason = lb_driver.lb_delete(**policy_data)
-        if res is False:
-            return False, reason
+        is_existed = policy_data.get('preexisting', False)
+        if not is_existed:
+            res, reason = lb_driver.lb_delete(**policy_data)
+            if res is False:
+                return False, reason
 
-        nodes = nm.Node.load_all(oslo_context.get_current(),
-                                 cluster_id=cluster.id, project_safe=False)
-        for node in nodes:
-            if 'lb_member' in node.data:
-                node.data.pop('lb_member')
-                node.store(oslo_context.get_current())
+            for node in cluster.nodes:
+                if 'lb_member' in node.data:
+                    node.data.pop('lb_member')
+                    values = {'data': node.data}
+                    no.Node.update(oslo_context.get_current(),
+                                   node.id, values)
+        else:
+            # the lb pool is existed, we need to remove servers from it
+            nodes = cluster.nodes
+            failed = self._remove_member(oslo_context.get_current(),
+                                         [node.id for node in nodes],
+                                         cp, lb_driver)
+            if failed:
+                return False, _('Failed to remove servers from existed LB.')
 
         lb_data = cluster.data.get('loadbalancers', {})
         if lb_data and isinstance(lb_data, dict):
@@ -395,7 +446,7 @@ class LoadBalancingPolicy(base.Policy):
         candidates = None
         if deletion is None:
             if action.action == consts.NODE_DELETE:
-                candidates = [action.node.id]
+                candidates = [action.entity.id]
                 count = 1
             elif action.action == consts.CLUSTER_DEL_NODES:
                 # Get candidates from action.input
@@ -403,9 +454,9 @@ class LoadBalancingPolicy(base.Policy):
                 count = len(candidates)
             elif action.action == consts.CLUSTER_RESIZE:
                 # Calculate deletion count based on action input
-                db_cluster = co.Cluster.get(action.context, cluster_id)
-                current = no.Node.count_by_cluster(action.context, cluster_id)
-                scaleutils.parse_resize_params(action, db_cluster, current)
+                cluster = action.entity
+                current = len(cluster.nodes)
+                scaleutils.parse_resize_params(action, cluster, current)
                 if 'deletion' not in action.data:
                     return []
                 else:
@@ -417,10 +468,11 @@ class LoadBalancingPolicy(base.Policy):
             candidates = deletion.get('candidates', None)
 
         # Still no candidates available, pick count of nodes randomly
+        # apply to CLUSTER_RESIZE/CLUSTER_SCALE_IN
         if candidates is None:
             if count == 0:
                 return []
-            nodes = no.Node.get_all_by_cluster(action.context, cluster_id)
+            nodes = action.entity.nodes
             if count > len(nodes):
                 count = len(nodes)
             candidates = scaleutils.nodes_by_random(nodes, count)
@@ -433,6 +485,106 @@ class LoadBalancingPolicy(base.Policy):
 
         return candidates
 
+    def _remove_member(self, context, candidates, policy, driver,
+                       handle_err=True):
+        # Load policy data
+        policy_data = self._extract_policy_data(policy.data)
+        lb_id = policy_data['loadbalancer']
+        pool_id = policy_data['pool']
+
+        failed_nodes = []
+        for node_id in candidates:
+            node = no.Node.get(context, node_id=node_id)
+            node_data = node.data or {}
+            member_id = node_data.get('lb_member', None)
+            if member_id is None:
+                LOG.warning('Node %(n)s not found in lb pool %(p)s.',
+                            {'n': node_id, 'p': pool_id})
+                continue
+
+            res = driver.member_remove(lb_id, pool_id, member_id)
+            values = {}
+            if res is not True and handle_err is True:
+                failed_nodes.append(node.id)
+                values['status'] = consts.NS_WARNING
+                values['status_reason'] = _(
+                    'Failed in removing node from lb pool.')
+            else:
+                node.data.pop('lb_member', None)
+                values['data'] = node.data
+            no.Node.update(context, node_id, values)
+
+        return failed_nodes
+
+    def _add_member(self, context, candidates, policy, driver):
+        # Load policy data
+        policy_data = self._extract_policy_data(policy.data)
+        lb_id = policy_data['loadbalancer']
+        pool_id = policy_data['pool']
+        port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
+        subnet = self.pool_spec.get(self.POOL_SUBNET)
+
+        failed_nodes = []
+        for node_id in candidates:
+            node = no.Node.get(context, node_id=node_id)
+            node_data = node.data or {}
+            member_id = node_data.get('lb_member', None)
+            if member_id:
+                LOG.warning('Node %(n)s already in lb pool %(p)s.',
+                            {'n': node_id, 'p': pool_id})
+                continue
+
+            member_id = driver.member_add(node, lb_id, pool_id, port, subnet)
+            values = {}
+            if member_id is None:
+                failed_nodes.append(node.id)
+                values['status'] = consts.NS_WARNING
+                values['status_reason'] = _(
+                    'Failed in adding node into lb pool.')
+            else:
+                node.data.update({'lb_member': member_id})
+                values['data'] = node.data
+            no.Node.update(context, node_id, values)
+
+        return failed_nodes
+
+    def _get_post_candidates(self, action):
+        # This method will parse action data passed from action layer
+
+        if (action.action == consts.NODE_CREATE or
+                action.action == consts.NODE_RECOVER):
+            candidates = [action.entity.id]
+        else:
+            creation = action.data.get('creation', None)
+            candidates = creation.get('nodes', []) if creation else []
+
+        return candidates
+
+    def _process_recovery(self, candidates, policy, driver, action):
+        # Process node recovery action
+        node = action.entity
+        data = node.data
+        lb_member = data.get('lb_member', None)
+        recovery = data.pop('recovery', None)
+        values = {}
+
+        # lb_member is None, need to add to lb pool
+        if not lb_member:
+            values['data'] = data
+            no.Node.update(action.context, values)
+            return candidates
+
+        # was a member of lb pool, check whether has been recreated
+        if recovery is not None and recovery == consts.RECOVER_RECREATE:
+            self._remove_member(action.context, candidates, policy, driver,
+                                handle_err=False)
+            data.pop('lb_member', None)
+            values['data'] = data
+            no.Node.update(action.context, values)
+            return candidates
+
+        return None
+
     def pre_op(self, cluster_id, action):
         """Routine to be called before an action has been executed.
 
@@ -449,30 +601,27 @@ class LoadBalancingPolicy(base.Policy):
         if len(candidates) == 0:
             return
 
-        db_cluster = co.Cluster.get(action.context, cluster_id)
-        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        hooks = action.data.get('hooks', {})
+        # if hooks properties are defined, defer removal of nodes from LB
+        # pool to the pre_op call during DEL_NODE action execution
+        if hooks:
+            return
+
+        obj = action.entity
+        lb_driver = self.lbaas(obj.user, obj.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
         cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
                                                self.id)
-        policy_data = self._extract_policy_data(cp.data)
-        lb_id = policy_data['loadbalancer']
-        pool_id = policy_data['pool']
 
         # Remove nodes that will be deleted from lb pool
-        for node_id in candidates:
-            node = nm.Node.load(action.context, node_id=node_id)
-            member_id = node.data.get('lb_member', None)
-            if member_id is None:
-                LOG.warning(_LW('Node %(n)s not found in lb pool %(p)s.'),
-                            {'n': node_id, 'p': pool_id})
-                continue
+        failed_nodes = self._remove_member(action.context, candidates,
+                                           cp, lb_driver)
 
-            res = lb_driver.member_remove(lb_id, pool_id, member_id)
-            if res is not True:
-                action.data['status'] = base.CHECK_ERROR
-                action.data['reason'] = _('Failed in removing deleted '
-                                          'node(s) from lb pool.')
-                return
+        if failed_nodes:
+            error = _('Failed in removing deleted node(s) from lb pool: %s'
+                      ) % failed_nodes
+            action.data['status'] = base.CHECK_ERROR
+            action.data['reason'] = error
 
         return
 
@@ -487,45 +636,30 @@ class LoadBalancingPolicy(base.Policy):
         :param action: The action object that triggered this operation.
         :returns: Nothing.
         """
+
         # TODO(Yanyanhu): Need special handling for cross-az scenario
         # which is supported by Neutron lbaas.
-        if action.action == consts.NODE_CREATE:
-            nodes_added = [action.node.id]
-        else:
-            creation = action.data.get('creation', None)
-            nodes_added = creation.get('nodes', []) if creation else []
-            if len(nodes_added) == 0:
-                return
+        candidates = self._get_post_candidates(action)
+        if len(candidates) == 0:
+            return
 
-        db_cluster = co.Cluster.get(action.context, cluster_id)
-        lb_driver = self.lbaas(db_cluster.user, db_cluster.project)
+        obj = action.entity
+        lb_driver = self.lbaas(obj.user, obj.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
         cp = cluster_policy.ClusterPolicy.load(action.context, cluster_id,
                                                self.id)
-        policy_data = self._extract_policy_data(cp.data)
-        lb_id = policy_data['loadbalancer']
-        pool_id = policy_data['pool']
-        port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
-        subnet = self.pool_spec.get(self.POOL_SUBNET)
-
-        # Add new nodes to lb pool
-        for node_id in nodes_added:
-            node = nm.Node.load(action.context, node_id=node_id)
-            member_id = node.data.get('lb_member', None)
-            if member_id:
-                LOG.warning(_LW('Node %(n)s already in lb pool %(p)s.'),
-                            {'n': node_id, 'p': pool_id})
-                continue
-
-            member_id = lb_driver.member_add(node, lb_id, pool_id, port,
-                                             subnet)
-            if member_id is None:
-                action.data['status'] = base.CHECK_ERROR
-                action.data['reason'] = _('Failed in adding new node(s) '
-                                          'into lb pool.')
+        if action.action == consts.NODE_RECOVER:
+            candidates = self._process_recovery(
+                candidates, cp, lb_driver, action)
+            if not candidates:
                 return
 
-            node.data.update({'lb_member': member_id})
-            node.store(action.context)
+        # Add new nodes to lb pool
+        failed_nodes = self._add_member(action.context, candidates,
+                                        cp, lb_driver)
+        if failed_nodes:
+            error = _('Failed in adding nodes into lb pool: %s') % failed_nodes
+            action.data['status'] = base.CHECK_ERROR
+            action.data['reason'] = error
 
         return
