@@ -10,18 +10,21 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo_config import cfg
+
 from senlin.common import constraints
 from senlin.common import consts
-from senlin.common import exception
+from senlin.common import exception as exc
 from senlin.common.i18n import _
 from senlin.common import scaleutils as su
 from senlin.common import schema
 from senlin.common import utils
-from senlin.objects import cluster as co
-from senlin.objects import node as no
 from senlin.policies import base
 
 
+CONF = cfg.CONF
+
+
 class ScalingPolicy(base.Policy):
     """Policy for changing the size of a cluster.
 
@@ -30,6 +33,11 @@ class ScalingPolicy(base.Policy):
     """
 
     VERSION = '1.0'
+    VERSIONS = {
+        '1.0': [
+            {'status': consts.SUPPORTED, 'since': '2016.04'}
+        ]
+    }
 
     PRIORITY = 100
 
@@ -112,8 +120,8 @@ class ScalingPolicy(base.Policy):
         :param name: Name for the policy object.
         :param spec: A dictionary containing the detailed specification for
                      the policy.
-        :param \*\*kwargs: Other optional parameters for policy object
-                           creation.
+        :param dict kwargs: Other optional parameters for policy object
+                            creation.
         :return: An object of `ScalingPolicy`.
         """
         super(ScalingPolicy, self).__init__(name, spec, **kwargs)
@@ -130,6 +138,21 @@ class ScalingPolicy(base.Policy):
         self.best_effort = adjustment[self.BEST_EFFORT]
         self.cooldown = adjustment[self.COOLDOWN]
 
+    def validate(self, context, validate_props=False):
+        super(ScalingPolicy, self).validate(context, validate_props)
+
+        if self.adjustment_number <= 0:
+            msg = _("the 'number' for 'adjustment' must be > 0")
+            raise exc.InvalidSpec(message=msg)
+
+        if self.adjustment_min_step < 0:
+            msg = _("the 'min_step' for 'adjustment' must be >= 0")
+            raise exc.InvalidSpec(message=msg)
+
+        if self.cooldown < 0:
+            msg = _("the 'cooldown' for 'adjustment' must be >= 0")
+            raise exc.InvalidSpec(message=msg)
+
     def _calculate_adjustment_count(self, current_size):
         """Calculate adjustment count based on current_size.
 
@@ -164,26 +187,29 @@ class ScalingPolicy(base.Policy):
         """
 
         # Use action input if count is provided
-        count = action.inputs.get('count', None)
-        current = no.Node.count_by_cluster(action.context, cluster_id)
-        if count is None:
+        count_value = action.inputs.get('count', None)
+        cluster = action.entity
+        current = len(cluster.nodes)
+
+        if count_value is None:
             # count not specified, calculate it
-            count = self._calculate_adjustment_count(current)
+            count_value = self._calculate_adjustment_count(current)
 
         # Count must be positive value
-        try:
-            count = utils.parse_int_param('count', count, allow_zero=False)
-        except exception.InvalidParameter:
+        success, count = utils.get_positive_int(count_value)
+        if not success:
             action.data.update({
                 'status': base.CHECK_ERROR,
                 'reason': _("Invalid count (%(c)s) for action '%(a)s'."
-                            ) % {'c': count, 'a': action.action}
+                            ) % {'c': count_value, 'a': action.action}
             })
             action.store(action.context)
             return
 
         # Check size constraints
-        cluster = co.Cluster.get(action.context, cluster_id)
+        max_size = cluster.max_size
+        if max_size == -1:
+            max_size = cfg.CONF.max_nodes_per_cluster
         if action.action == consts.CLUSTER_SCALE_IN:
             if self.best_effort:
                 count = min(count, current - cluster.min_size)
@@ -191,7 +217,7 @@ class ScalingPolicy(base.Policy):
                                           strict=not self.best_effort)
         else:
             if self.best_effort:
-                count = min(count, cluster.max_size - current)
+                count = min(count, max_size - current)
             result = su.check_size_params(cluster, current + count,
                                           strict=not self.best_effort)
 
