@@ -11,16 +11,18 @@
 # under the License.
 
 import copy
+import inspect
 
 from oslo_context import context as oslo_context
 from oslo_log import log as logging
 from oslo_utils import timeutils
+from osprofiler import profiler
 import six
 
 from senlin.common import consts
 from senlin.common import context
 from senlin.common import exception as exc
-from senlin.common.i18n import _, _LE, _LW
+from senlin.common.i18n import _
 from senlin.common import schema
 from senlin.common import utils
 from senlin.drivers import base as driver_base
@@ -32,7 +34,9 @@ LOG = logging.getLogger(__name__)
 
 
 class Profile(object):
-    '''Base class for profiles.'''
+    """Base class for profiles."""
+
+    VERSIONS = {}
 
     KEYS = (
         TYPE, VERSION, PROPERTIES,
@@ -56,7 +60,7 @@ class Profile(object):
     }
 
     properties_schema = {}
-    operations_schema = {}
+    OPERATIONS = {}
 
     def __new__(cls, name, spec, **kwargs):
         """Create a new profile of the appropriate class.
@@ -121,13 +125,15 @@ class Profile(object):
         self._computeclient = None
         self._networkclient = None
         self._orchestrationclient = None
+        self._workflowclient = None
+        self._block_storageclient = None
 
     @classmethod
-    def from_object(cls, profile):
-        '''Construct a profile from profile object.
+    def _from_object(cls, profile):
+        """Construct a profile from profile object.
 
         :param profile: a profile object that contains all required fields.
-        '''
+        """
         kwargs = {
             'id': profile.id,
             'type': profile.type,
@@ -144,33 +150,48 @@ class Profile(object):
 
     @classmethod
     def load(cls, ctx, profile=None, profile_id=None, project_safe=True):
-        '''Retrieve a profile object from database.'''
+        """Retrieve a profile object from database."""
         if profile is None:
             profile = po.Profile.get(ctx, profile_id,
                                      project_safe=project_safe)
             if profile is None:
                 raise exc.ResourceNotFound(type='profile', id=profile_id)
 
-        return cls.from_object(profile)
+        return cls._from_object(profile)
 
     @classmethod
-    def load_all(cls, ctx, limit=None, marker=None, sort=None, filters=None,
-                 project_safe=True):
-        """Retrieve all profiles from database."""
+    def create(cls, ctx, name, spec, metadata=None):
+        """Create a profile object and validate it.
 
-        records = po.Profile.get_all(ctx, limit=limit, marker=marker,
-                                     sort=sort, filters=filters,
-                                     project_safe=project_safe)
+        :param ctx: The requesting context.
+        :param name: The name for the profile object.
+        :param spec: A dict containing the detailed spec.
+        :param metadata: An optional dictionary specifying key-value pairs to
+                         be associated with the profile.
+        :returns: An instance of Profile.
+        """
+        if metadata is None:
+            metadata = {}
 
-        for record in records:
-            yield cls.from_object(record)
+        try:
+            profile = cls(name, spec, metadata=metadata, user=ctx.user_id,
+                          project=ctx.project_id)
+            profile.validate(True)
+        except (exc.ResourceNotFound, exc.ESchema) as ex:
+            error = _("Failed in creating profile %(name)s: %(error)s"
+                      ) % {"name": name, "error": six.text_type(ex)}
+            raise exc.InvalidSpec(message=error)
+
+        profile.store(ctx)
+
+        return profile
 
     @classmethod
     def delete(cls, ctx, profile_id):
         po.Profile.delete(ctx, profile_id)
 
     def store(self, ctx):
-        '''Store the profile into database and return its ID.'''
+        """Store the profile into database and return its ID."""
         timestamp = timeutils.utcnow(True)
 
         values = {
@@ -197,16 +218,39 @@ class Profile(object):
         return self.id
 
     @classmethod
+    @profiler.trace('Profile.create_object', hide_args=False)
     def create_object(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_create(obj)
 
     @classmethod
+    @profiler.trace('Profile.create_cluster_object', hide_args=False)
+    def create_cluster_object(cls, ctx, obj):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        try:
+            ret = profile.do_cluster_create(obj)
+        except NotImplementedError:
+            return None
+        return ret
+
+    @classmethod
+    @profiler.trace('Profile.delete_object', hide_args=False)
     def delete_object(cls, ctx, obj, **params):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_delete(obj, **params)
 
     @classmethod
+    @profiler.trace('Profile.delete_cluster_object', hide_args=False)
+    def delete_cluster_object(cls, ctx, obj, **params):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        try:
+            ret = profile.do_cluster_delete(obj, **params)
+        except NotImplementedError:
+            return None
+        return ret
+
+    @classmethod
+    @profiler.trace('Profile.update_object', hide_args=False)
     def update_object(cls, ctx, obj, new_profile_id=None, **params):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         new_profile = None
@@ -215,37 +259,72 @@ class Profile(object):
         return profile.do_update(obj, new_profile, **params)
 
     @classmethod
+    @profiler.trace('Profile.get_details', hide_args=False)
     def get_details(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_get_details(obj)
 
     @classmethod
+    @profiler.trace('Profile.adopt_node', hide_args=False)
+    def adopt_node(cls, ctx, obj, type_name, overrides=None, snapshot=False):
+        """Adopt a node.
+
+        :param ctx: Request context.
+        :param obj: A temporary node object.
+        :param overrides: An optional parameter that specifies the set of
+            properties to be overridden.
+        :param snapshot: A boolean flag indicating whether a snapshot should
+            be created before adopting the node.
+        :returns: A dictionary containing the profile spec created from the
+            specific node, or a dictionary containing error message.
+        """
+        parts = type_name.split("-")
+        tmpspec = {"type": parts[0], "version": parts[1]}
+        profile = cls("name", tmpspec)
+        return profile.do_adopt(obj, overrides=overrides, snapshot=snapshot)
+
+    @classmethod
+    @profiler.trace('Profile.join_cluster', hide_args=False)
     def join_cluster(cls, ctx, obj, cluster_id):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_join(obj, cluster_id)
 
     @classmethod
+    @profiler.trace('Profile.leave_cluster', hide_args=False)
     def leave_cluster(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_leave(obj)
 
     @classmethod
+    @profiler.trace('Profile.check_object', hide_args=False)
     def check_object(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
-        return profile.do_check(obj)
+        try:
+            return profile.do_check(obj)
+        except exc.InternalError as ex:
+            LOG.error(ex)
+            return False
 
     @classmethod
+    @profiler.trace('Profile.recover_object', hide_args=False)
     def recover_object(cls, ctx, obj, **options):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_recover(obj, **options)
 
     def validate(self, validate_props=False):
-        '''Validate the schema and the data provided.'''
+        """Validate the schema and the data provided."""
         # general validation
         self.spec_data.validate()
         self.properties.validate()
 
-        # TODO(Anyone): need to check the contents in self.CONTEXT
+        ctx_dict = self.properties.get('context', {})
+        if ctx_dict:
+            argspec = inspect.getargspec(context.RequestContext.__init__)
+            valid_keys = argspec.args
+            bad_keys = [k for k in ctx_dict if k not in valid_keys]
+            if bad_keys:
+                msg = _("Some keys in 'context' are invalid: %s") % bad_keys
+                raise exc.ESchema(message=msg)
 
         if validate_props:
             self.do_validate(obj=self)
@@ -255,12 +334,17 @@ class Profile(object):
         return dict((name, dict(schema))
                     for name, schema in cls.properties_schema.items())
 
+    @classmethod
+    def get_ops(cls):
+        return dict((name, dict(schema))
+                    for name, schema in cls.OPERATIONS.items())
+
     def _init_context(self):
         profile_context = {}
         if self.CONTEXT in self.properties:
             profile_context = self.properties[self.CONTEXT] or {}
 
-        ctx_dict = context.get_service_context(**profile_context)
+        ctx_dict = context.get_service_credentials(**profile_context)
 
         ctx_dict.pop('project_name', None)
         ctx_dict.pop('project_domain_name', None)
@@ -288,12 +372,12 @@ class Profile(object):
         return params
 
     def compute(self, obj):
-        '''Construct compute client based on object.
+        """Construct compute client based on object.
 
         :param obj: Object for which the client is created. It is expected to
                     be None when retrieving an existing client. When creating
                     a client, it contains the user and project to be used.
-        '''
+        """
 
         if self._computeclient is not None:
             return self._computeclient
@@ -328,75 +412,123 @@ class Profile(object):
         self._orchestrationclient = oc
         return oc
 
+    def workflow(self, obj):
+        if self._workflowclient is not None:
+            return self._workflowclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._workflowclient = driver_base.SenlinDriver().workflow(params)
+        return self._workflowclient
+
+    def block_storage(self, obj):
+        """Construct cinder client based on object.
+
+        :param obj: Object for which the client is created. It is expected to
+                    be None when retrieving an existing client. When creating
+                    a client, it contains the user and project to be used.
+        """
+        if self._block_storageclient is not None:
+            return self._block_storageclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._block_storageclient = driver_base.SenlinDriver().block_storage(
+            params)
+        return self._block_storageclient
+
     def do_create(self, obj):
         """For subclass to override."""
         raise NotImplementedError
 
+    def do_cluster_create(self, obj):
+        """For subclass to override."""
+        raise NotImplementedError
+
     def do_delete(self, obj, **params):
         """For subclass to override."""
         raise NotImplementedError
 
+    def do_cluster_delete(self, obj):
+        """For subclass to override."""
+        raise NotImplementedError
+
     def do_update(self, obj, new_profile, **params):
         """For subclass to override."""
-        LOG.warning(_LW("Update operation not supported."))
+        LOG.warning("Update operation not supported.")
         return True
 
     def do_check(self, obj):
         """For subclass to override."""
-        LOG.warning(_LW("Check operation not supported."))
+        LOG.warning("Check operation not supported.")
         return True
 
     def do_get_details(self, obj):
         """For subclass to override."""
-        LOG.warning(_LW("Get_details operation not supported."))
+        LOG.warning("Get_details operation not supported.")
+        return {}
+
+    def do_adopt(self, obj, overrides=None, snapshot=False):
+        """For subclass to override."""
+        LOG.warning("Adopt operation not supported.")
         return {}
 
     def do_join(self, obj, cluster_id):
         """For subclass to override to perform extra operations."""
-        LOG.warning(_LW("Join operation not specialized."))
+        LOG.warning("Join operation not specialized.")
         return True
 
     def do_leave(self, obj):
         """For subclass to override to perform extra operations."""
-        LOG.warning(_LW("Join operation not specialized."))
-        return True
-
-    def do_rebuild(self, obj):
-        """For subclass to override."""
-        LOG.warning(_LW("Rebuild operation not specialized."))
+        LOG.warning("Leave operation not specialized.")
         return True
 
     def do_recover(self, obj, **options):
         """Default recover operation.
 
+        This is provided as a fallback if a specific profile type does not
+        override this method.
+
         :param obj: The node object to operate on.
         :param options: Keyword arguments for the recover operation.
+        :return id: New id of the recovered resource or None if recovery
+            failed.
+        :return status: True indicates successful recovery, False indicates
+            failure.
         """
         operation = options.pop('operation', None)
-        # TODO(Qiming): The operation input could be a list of operations.
+        force_recreate = options.pop('force_recreate', None)
+        delete_timeout = options.pop('delete_timeout', None)
+
+        # The operation is a list of action names with optional parameters
         if operation and not isinstance(operation, six.string_types):
             operation = operation[0]
 
-        if operation and operation != consts.RECOVER_RECREATE:
-            LOG.error(_LE("Recover operation not supported: %s"), operation)
-            return False
+        if operation and operation['name'] != consts.RECOVER_RECREATE:
+            LOG.error("Recover operation not supported: %s", operation)
+            return None, False
 
+        extra_params = options.get('params', {})
+        fence_compute = extra_params.get('fence_compute', False)
         try:
-            self.do_delete(obj, **options)
+            self.do_delete(obj, force=fence_compute, timeout=delete_timeout,
+                           delete_ports_on_failure=force_recreate)
         except exc.EResourceDeletion as ex:
-            raise exc.EResourceOperation(op='recovering', type='node',
-                                         id=obj.id, message=six.text_type(ex))
+            if force_recreate:
+                # log error and continue on to creating the node
+                LOG.warning('Failed to delete node during recovery action: %s',
+                            ex)
+            else:
+                raise exc.EResourceOperation(op='recovering', type='node',
+                                             id=obj.id,
+                                             message=six.text_type(ex))
         res = None
         try:
             res = self.do_create(obj)
         except exc.EResourceCreation as ex:
             raise exc.EResourceOperation(op='recovering', type='node',
                                          id=obj.id, message=six.text_type(ex))
-        return res
+        return res, True
 
     def do_validate(self, obj):
         """For subclass to override."""
-        LOG.warning(_LW("Validate operation not supported."))
+        LOG.warning("Validate operation not supported.")
         return True
 
     def to_dict(self):
@@ -425,6 +557,5 @@ class Profile(object):
             return True
 
         msg = ", ".join(non_updatables)
-        LOG.error(_LE("The following properties are not updatable: %s."
-                      ) % msg)
+        LOG.error("The following properties are not updatable: %s.", msg)
         return False
