@@ -13,8 +13,9 @@
 from oslo_log import log as logging
 import six
 
+from senlin.common import consts
 from senlin.common import exception as exc
-from senlin.common.i18n import _, _LE
+from senlin.common.i18n import _
 from senlin.common import schema
 from senlin.common import utils
 from senlin.profiles import base
@@ -23,11 +24,13 @@ LOG = logging.getLogger(__name__)
 
 
 class StackProfile(base.Profile):
-    '''Profile for an OpenStack Heat stack.
+    """Profile for an OpenStack Heat stack."""
 
-    When this profile is used, the whole cluster is a collection of Heat
-    stacks.
-    '''
+    VERSIONS = {
+        '1.0': [
+            {'status': consts.SUPPORTED, 'since': '2016.04'}
+        ]
+    }
 
     KEYS = (
         CONTEXT, TEMPLATE, TEMPLATE_URL, PARAMETERS,
@@ -99,7 +102,7 @@ class StackProfile(base.Profile):
         self.stack_id = None
 
     def validate(self, validate_props=False):
-        '''Validate the schema and the data provided.'''
+        """Validate the schema and the data provided."""
         # general validation
         self.spec_data.validate()
         self.properties.validate()
@@ -144,6 +147,10 @@ class StackProfile(base.Profile):
         :param obj: The node object to operate on.
         :returns: The UUID of the heat stack created.
         """
+        tags = ["cluster_node_id=%s" % obj.id]
+        if obj.cluster_id:
+            tags.append('cluster_id=%s' % obj.cluster_id)
+            tags.append('cluster_node_index=%s' % obj.index)
         kwargs = {
             'stack_name': obj.name + '-' + utils.random_name(8),
             'template': self.properties[self.TEMPLATE],
@@ -153,13 +160,14 @@ class StackProfile(base.Profile):
             'parameters': self.properties[self.PARAMETERS],
             'files': self.properties[self.FILES],
             'environment': self.properties[self.ENVIRONMENT],
+            'tags': ",".join(tags)
         }
 
         try:
             stack = self.orchestration(obj).stack_create(**kwargs)
 
             # Timeout = None means we will use the 'default_action_timeout'
-            # It can be overridden by the TIMEOUT profile propertie
+            # It can be overridden by the TIMEOUT profile properties
             timeout = None
             if self.properties[self.TIMEOUT]:
                 timeout = self.properties[self.TIMEOUT] * 60
@@ -168,7 +176,8 @@ class StackProfile(base.Profile):
                                                    timeout=timeout)
             return stack.id
         except exc.InternalError as ex:
-            raise exc.EResourceCreation(type='stack', message=ex.message)
+            raise exc.EResourceCreation(type='stack',
+                                        message=six.text_type(ex))
 
     def do_delete(self, obj, **params):
         """Delete the physical stack behind the node object.
@@ -176,11 +185,13 @@ class StackProfile(base.Profile):
         :param obj: The node object to operate on.
         :param kwargs params: Optional keyword arguments for the delete
                               operation.
-        :returns: This operation always return True unless exception is
+        :returns: This operation always returns True unless exception is
                   caught.
         :raises: `EResourceDeletion` if interaction with heat fails.
         """
         stack_id = obj.physical_id
+        if not stack_id:
+            return True
 
         ignore_missing = params.get('ignore_missing', True)
         try:
@@ -237,7 +248,7 @@ class StackProfile(base.Profile):
         try:
             hc = self.orchestration(obj)
             # Timeout = None means we will use the 'default_action_timeout'
-            # It can be overridden by the TIMEOUT profile propertie
+            # It can be overridden by the TIMEOUT profile properties
             timeout = None
             if self.properties[self.TIMEOUT]:
                 timeout = self.properties[self.TIMEOUT] * 60
@@ -246,7 +257,7 @@ class StackProfile(base.Profile):
                               timeout=timeout)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='stack', id=self.stack_id,
-                                      message=ex.message)
+                                      message=six.text_type(ex))
 
         return True
 
@@ -254,7 +265,7 @@ class StackProfile(base.Profile):
         """Check stack status.
 
         :param obj: Node object to operate.
-        :returns: True if check succeedes, or False otherwise.
+        :returns: True if check succeeded, or False otherwise.
         """
         stack_id = obj.physical_id
         if stack_id is None:
@@ -263,15 +274,16 @@ class StackProfile(base.Profile):
         hc = self.orchestration(obj)
         try:
             # Timeout = None means we will use the 'default_action_timeout'
-            # It can be overridden by the TIMEOUT profile propertie
+            # It can be overridden by the TIMEOUT profile properties
             timeout = None
             if self.properties[self.TIMEOUT]:
                 timeout = self.properties[self.TIMEOUT] * 60
             hc.stack_check(stack_id)
             hc.wait_for_stack(stack_id, 'CHECK_COMPLETE', timeout=timeout)
         except exc.InternalError as ex:
-            LOG.error(_LE('Failed in checking stack: %s.'), ex)
-            return False
+            raise exc.EResourceOperation(op='checking', type='stack',
+                                         id=stack_id,
+                                         message=six.text_type(ex))
 
         return True
 
@@ -290,6 +302,112 @@ class StackProfile(base.Profile):
                 }
             }
 
+    def do_adopt(self, obj, overrides=None, snapshot=False):
+        """Adopt an existing stack node for management.
+
+        :param obj: A node object for this operation. It could be a puppet
+            node that provides only 'user', 'project' and 'physical_id'
+            properties when doing a preview. It can be a real Node object for
+            node adoption.
+        :param overrides: A dict containing the properties that will be
+            overridden when generating a profile for the stack.
+        :param snapshot: A boolean flag indicating whether the profile should
+            attempt a snapshot operation before adopting the stack. If set to
+            True, the ID of the snapshot will be used as the image ID.
+
+        :returns: A dict containing the spec created from the stack object or
+            a dict containing error information if failure occurred.
+        """
+        driver = self.orchestration(obj)
+
+        # TODO(Qiming): Add snapshot support
+        # snapshot = driver.snapshot_create(...)
+
+        try:
+            stack = driver.stack_get(obj.physical_id)
+            tmpl = driver.stack_get_template(obj.physical_id)
+            env = driver.stack_get_environment(obj.physical_id)
+            files = driver.stack_get_files(obj.physical_id)
+        except exc.InternalError as ex:
+            return {'Error': {'code': ex.code, 'message': six.text_type(ex)}}
+
+        spec = {
+            self.ENVIRONMENT: env.to_dict(),
+            self.FILES: files,
+            self.TEMPLATE: tmpl.to_dict(),
+            self.PARAMETERS: dict((k, v) for k, v in stack.parameters.items()
+                                  if k.find('OS::', 0) < 0),
+            self.TIMEOUT: stack.timeout_mins,
+            self.DISABLE_ROLLBACK: stack.is_rollback_disabled
+        }
+        if overrides:
+            spec.update(overrides)
+
+        return spec
+
+    def _refresh_tags(self, current, node, add=False):
+        """Refresh tag list.
+
+        :param current: Current list of tags.
+        :param node: The node object.
+        :param add: Flag indicating whether new tags are added.
+        :returns: (tags, updated) where tags contains a new list of tags and
+                  updated indicates whether new tag list differs from the old
+                  one.
+        """
+        tags = []
+        for tag in current:
+            if tag.find('cluster_id=') == 0:
+                continue
+            elif tag.find('cluster_node_id=') == 0:
+                continue
+            elif tag.find('cluster_node_index=') == 0:
+                continue
+            if tag.strip() != "":
+                tags.append(tag.strip())
+
+        if add:
+            tags.append('cluster_id=' + node.cluster_id)
+            tags.append('cluster_node_id=' + node.id)
+            tags.append('cluster_node_index=%s' % node.index)
+
+        tag_str = ",".join(tags)
+        return (tag_str, tags != current)
+
+    def do_join(self, obj, cluster_id):
+        if not obj.physical_id:
+            return False
+
+        hc = self.orchestration(obj)
+        try:
+            stack = hc.stack_get(obj.physical_id)
+            tags, updated = self._refresh_tags(stack.tags, obj, True)
+            field = {'tags': tags}
+            if updated:
+                hc.stack_update(obj.physical_id, **field)
+        except exc.InternalError as ex:
+            LOG.error('Failed in updating stack tags: %s.', ex)
+            return False
+
+        return True
+
+    def do_leave(self, obj):
+        if not obj.physical_id:
+            return False
+
+        hc = self.orchestration(obj)
+        try:
+            stack = hc.stack_get(obj.physical_id)
+            tags, updated = self._refresh_tags(stack.tags, obj, False)
+            field = {'tags': tags}
+            if updated:
+                hc.stack_update(obj.physical_id, **field)
+        except exc.InternalError as ex:
+            LOG.error('Failed in updating stack tags: %s.', ex)
+            return False
+
+        return True
+
     def handle_abandon(self, obj, **options):
         """Handler for abandoning a heat stack node."""
         pass
