@@ -194,7 +194,7 @@ class ServerProfile(base.Profile):
                         ),
                     ),
                     FLOATING_NETWORK: schema.String(
-                        _('The nework on which to create a floating IP'),
+                        _('The network on which to create a floating IP'),
                     ),
                     FLOATING_IP: schema.String(
                         _('The floating IP address to be associated with '
@@ -241,11 +241,11 @@ class ServerProfile(base.Profile):
     OP_NAMES = (
         OP_REBOOT, OP_REBUILD, OP_CHANGE_PASSWORD, OP_PAUSE, OP_UNPAUSE,
         OP_SUSPEND, OP_RESUME, OP_LOCK, OP_UNLOCK, OP_START, OP_STOP,
-        OP_RESCUE, OP_UNRESCUE, OP_EVACUATE,
+        OP_RESCUE, OP_UNRESCUE, OP_EVACUATE, OP_MIGRATE,
     ) = (
         'reboot', 'rebuild', 'change_password', 'pause', 'unpause',
         'suspend', 'resume', 'lock', 'unlock', 'start', 'stop',
-        'rescue', 'unrescue', 'evacuate',
+        'rescue', 'unrescue', 'evacuate', 'migrate',
     )
 
     REBOOT_TYPE = 'type'
@@ -417,6 +417,27 @@ class ServerProfile(base.Profile):
             else:
                 raise
 
+    def _validate_volume(self, obj, name_or_id, reason=None):
+        try:
+            volume = self.block_storage(obj).volume_get(name_or_id)
+            if volume.status == 'available':
+                return volume
+
+            msg = _("The volume %(k)s should be in 'available' status "
+                    "but is in '%(v)s' status."
+                    ) % {'k': name_or_id, 'v': volume.status}
+            raise exc.InvalidSpec(message=msg)
+        except exc.InternalError as ex:
+            if reason == 'create':
+                raise exc.EResourceCreation(type='server',
+                                            message=six.text_type(ex))
+            elif ex.code == 404:
+                msg = _("The specified volume '%(k)s' could not be found."
+                        ) % {'k': name_or_id}
+                raise exc.InvalidSpec(message=msg)
+            else:
+                raise
+
     def do_validate(self, obj):
         """Validate if the spec has provided valid info for server creation.
 
@@ -448,11 +469,17 @@ class ServerProfile(base.Profile):
 
         return True
 
-    def _resolve_bdm(self, bdm):
+    def _resolve_bdm(self, obj, bdm, reason=None):
         for bd in bdm:
             for key in self.BDM2_KEYS:
                 if bd[key] is None:
                     del bd[key]
+            if 'uuid' in bd and 'source_type' in bd:
+                if bd['source_type'] == 'image':
+                    self._validate_image(obj, bd['uuid'], reason)
+                elif bd['source_type'] == 'volume':
+                    self._validate_volume(obj, bd['uuid'], reason)
+
         return bdm
 
     def _check_security_groups(self, nc, net_spec, result):
@@ -820,7 +847,7 @@ class ServerProfile(base.Profile):
         block_device_mapping_v2 = self.properties[self.BLOCK_DEVICE_MAPPING_V2]
         if block_device_mapping_v2 is not None:
             kwargs['block_device_mapping_v2'] = self._resolve_bdm(
-                block_device_mapping_v2)
+                obj, block_device_mapping_v2, 'create')
 
         user_data = self.properties[self.USER_DATA]
         if user_data is not None:
@@ -844,7 +871,7 @@ class ServerProfile(base.Profile):
 
             if 'servergroup' in obj.data['placement']:
                 group_id = obj.data['placement']['servergroup']
-                hints = self.properties.get(self.SCHEDULER_HINTS, {})
+                hints = self.properties.get(self.SCHEDULER_HINTS) or {}
                 hints.update({'group': group_id})
                 kwargs['scheduler_hints'] = hints
 
@@ -860,7 +887,8 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             if server and server.id:
                 resource_id = server.id
-            raise exc.EResourceCreation(type='server', message=ex.message,
+            raise exc.EResourceCreation(type='server',
+                                        message=six.text_type(ex),
                                         resource_id=resource_id)
 
     def do_delete(self, obj, **params):
@@ -880,6 +908,8 @@ class ServerProfile(base.Profile):
         ignore_missing = params.get('ignore_missing', True)
         internal_ports = obj.data.get('internal_ports', [])
         force = params.get('force', False)
+        timeout = params.get('timeout', None)
+        delete_ports_on_failure = params.get('delete_ports_on_failure', False)
 
         try:
             driver = self.compute(obj)
@@ -887,16 +917,26 @@ class ServerProfile(base.Profile):
                 driver.server_force_delete(server_id, ignore_missing)
             else:
                 driver.server_delete(server_id, ignore_missing)
-            driver.wait_for_server_delete(server_id)
-            if internal_ports:
-                ex = self._delete_ports(obj, internal_ports)
-                if ex:
-                    raise ex
-            return True
         except exc.InternalError as ex:
             raise exc.EResourceDeletion(type='server', id=server_id,
                                         message=six.text_type(ex))
 
+        try:
+            driver.wait_for_server_delete(server_id, timeout)
+        except exc.InternalError as ex:
+            if delete_ports_on_failure and internal_ports:
+                del_ports_ex = self._delete_ports(obj, internal_ports)
+                if del_ports_ex:
+                    raise del_ports_ex
+            raise exc.EResourceDeletion(type='server', id=server_id,
+                                        message=six.text_type(ex))
+
+        if internal_ports:
+            ex = self._delete_ports(obj, internal_ports)
+            if ex:
+                raise ex
+        return True
+
     def _check_server_name(self, obj, profile):
         """Check if there is a new name to be assigned to the server.
 
@@ -1002,7 +1042,7 @@ class ServerProfile(base.Profile):
             msg = six.text_type(ex)
             try:
                 cc.server_resize_revert(obj.physical_id)
-                cc.wait_for_server(obj.physical_id, 'ACTIVE')
+                cc.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
             except exc.InternalError as ex1:
                 msg = six.text_type(ex1)
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
@@ -1010,7 +1050,7 @@ class ServerProfile(base.Profile):
 
         try:
             cc.server_resize_confirm(obj.physical_id)
-            cc.wait_for_server(obj.physical_id, 'ACTIVE')
+            cc.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
                                       message=six.text_type(ex))
@@ -1027,7 +1067,6 @@ class ServerProfile(base.Profile):
         :returns: A boolean indicating whether the image needs an update.
         :raises: ``InternalError`` if operation was a failure.
         """
-        old_image = self.properties[self.IMAGE]
         new_image = new_profile.properties[self.IMAGE]
         if not new_image:
             msg = _("Updating Nova server with image set to None is not "
@@ -1039,18 +1078,13 @@ class ServerProfile(base.Profile):
         new_image_id = img_new.id
 
         driver = self.compute(obj)
-        if old_image:
-            img_old = self._validate_image(obj, old_image, reason='update')
-            old_image_id = img_old.id
-        else:
-            try:
-                server = driver.server_get(obj.physical_id)
-            except exc.InternalError as ex:
-                raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                          message=six.text_type(ex))
-            # Still, this 'old_image_id' could be empty, but it doesn't matter
-            # because the comparison below would fail if that is the case
-            old_image_id = server.image.get('id', None)
+
+        try:
+            server = driver.server_get(obj.physical_id)
+        except exc.InternalError as ex:
+            raise exc.EResourceUpdate(type='server', id=obj.physical_id,
+                                      message=six.text_type(ex))
+        old_image_id = self._get_image_id(obj, server, 'updating')
 
         if new_image_id == old_image_id:
             return False
@@ -1058,7 +1092,7 @@ class ServerProfile(base.Profile):
         try:
             driver.server_rebuild(obj.physical_id, new_image_id,
                                   new_name, new_password)
-            driver.wait_for_server(obj.physical_id, 'ACTIVE')
+            driver.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
                                       message=six.text_type(ex))
@@ -1203,8 +1237,7 @@ class ServerProfile(base.Profile):
         :returns: True if update was successful or False otherwise.
         :raises: `EResourceUpdate` if operation fails.
         """
-        self.server_id = obj.physical_id
-        if not self.server_id:
+        if not obj.physical_id:
             return False
 
         if not new_profile:
@@ -1337,6 +1370,10 @@ class ServerProfile(base.Profile):
 
         if server.image:
             image_id = server.image['id'] or server.image
+        # when booting a nova server from volume, the image property
+        # can be ignored.
+        # we try to find a volume which is bootable and use its image_id
+        # for the server.
         elif server.attached_volumes:
             cinder_driver = self.block_storage(obj)
             for volume_ids in server.attached_volumes:
@@ -1356,6 +1393,26 @@ class ServerProfile(base.Profile):
                                          message=msg)
         return image_id
 
+    def _handle_generic_op(self, obj, driver_func_name,
+                           op_name, expected_server_status=None,
+                           **kwargs):
+        """Generic handler for standard server operations."""
+        if not obj.physical_id:
+            return False
+        server_id = obj.physical_id
+        nova_driver = self.compute(obj)
+
+        try:
+            driver_func = getattr(nova_driver, driver_func_name)
+            driver_func(server_id, **kwargs)
+            if expected_server_status:
+                nova_driver.wait_for_server(server_id, expected_server_status)
+            return True
+        except exc.InternalError as ex:
+            raise exc.EResourceOperation(op=op_name, type='server',
+                                         id=server_id,
+                                         message=six.text_type(ex))
+
     def do_adopt(self, obj, overrides=None, snapshot=False):
         """Adopt an existing server node for management.
 
@@ -1441,10 +1498,14 @@ class ServerProfile(base.Profile):
             return False
 
         driver = self.compute(obj)
-        metadata = driver.server_metadata_get(obj.physical_id) or {}
-        metadata['cluster_id'] = cluster_id
-        metadata['cluster_node_index'] = six.text_type(obj.index)
-        driver.server_metadata_update(obj.physical_id, metadata)
+        try:
+            metadata = driver.server_metadata_get(obj.physical_id) or {}
+            metadata['cluster_id'] = cluster_id
+            metadata['cluster_node_index'] = six.text_type(obj.index)
+            driver.server_metadata_update(obj.physical_id, metadata)
+        except exc.InternalError as ex:
+            raise exc.EResourceUpdate(type='server', id=obj.physical_id,
+                                      message=six.text_type(ex))
         return super(ServerProfile, self).do_join(obj, cluster_id)
 
     def do_leave(self, obj):
@@ -1452,7 +1513,11 @@ class ServerProfile(base.Profile):
             return False
 
         keys = ['cluster_id', 'cluster_node_index']
-        self.compute(obj).server_metadata_delete(obj.physical_id, keys)
+        try:
+            self.compute(obj).server_metadata_delete(obj.physical_id, keys)
+        except exc.InternalError as ex:
+            raise exc.EResourceDeletion(type='server', id=obj.physical_id,
+                                        message=six.text_type(ex))
         return super(ServerProfile, self).do_leave(obj)
 
     def do_check(self, obj):
@@ -1471,7 +1536,7 @@ class ServerProfile(base.Profile):
                                              id=obj.physical_id,
                                              message=six.text_type(ex))
 
-        if (server is None or server.status != 'ACTIVE'):
+        if (server is None or server.status != consts.VS_ACTIVE):
             return False
 
         return True
@@ -1482,6 +1547,10 @@ class ServerProfile(base.Profile):
         :param obj: The node object.
         :param dict options: A list for operations each of which has a name
             and optionally a map from parameter to values.
+        :return id: New id of the recovered resource or None if recovery
+            failed.
+        :return status: True indicates successful recovery, False indicates
+            failure.
         """
         operation = options.get('operation', None)
 
@@ -1492,10 +1561,15 @@ class ServerProfile(base.Profile):
             op_name = operation['name']
             if op_name.upper() != consts.RECOVER_RECREATE:
                 op_params = operation.get('params', {})
+                # nova recover operation always use hard reboot
+                # vm in error or stop status soft reboot can't succeed
+                if op_name.upper() == consts.RECOVER_REBOOT:
+                    if self.REBOOT_TYPE not in op_params:
+                        op_params[self.REBOOT_TYPE] = self.REBOOT_HARD
                 if op_name.lower() not in self.OP_NAMES:
                     LOG.error("The operation '%s' is not supported",
                               op_name)
-                    return False
+                    return obj.physical_id, False
 
                 method = getattr(self, "handle_" + op_name.lower())
                 return method(obj, **op_params)
@@ -1505,16 +1579,27 @@ class ServerProfile(base.Profile):
     def handle_reboot(self, obj, **options):
         """Handler for the reboot operation."""
         if not obj.physical_id:
-            return False
+            return None, False
 
+        server_id = obj.physical_id
         reboot_type = options.get(self.REBOOT_TYPE, self.REBOOT_SOFT)
         if (not isinstance(reboot_type, six.string_types) or
                 reboot_type not in self.REBOOT_TYPES):
-            return False
+            return server_id, False
 
-        self.compute(obj).server_reboot(obj.physical_id, reboot_type)
-        self.compute(obj).wait_for_server(obj.physical_id, 'ACTIVE')
-        return True
+        nova_driver = self.compute(obj)
+        try:
+            server = nova_driver.server_get(server_id)
+            if server is None:
+                return None, False
+            nova_driver.server_reboot(server_id, reboot_type)
+            nova_driver.wait_for_server(obj.physical_id,
+                                        consts.VS_ACTIVE)
+            return server_id, True
+        except exc.InternalError as ex:
+            raise exc.EResourceOperation(op='rebooting', type='server',
+                                         id=server_id,
+                                         message=six.text_type(ex))
 
     def handle_rebuild(self, obj, **options):
         """Handler for the rebuild operation.
@@ -1522,10 +1607,13 @@ class ServerProfile(base.Profile):
         :param obj: The node object.
         :param dict options: A list for operations each of which has a name
             and optionally a map from parameter to values.
-        :returns: The server ID if successful or None if failed.
+        :return id: New id of the recovered resource or None if recovery
+            failed.
+        :return status: True indicates successful recovery, False indicates
+            failure.
         """
         if not obj.physical_id:
-            return None
+            return None, False
 
         server_id = obj.physical_id
         nova_driver = self.compute(obj)
@@ -1537,38 +1625,18 @@ class ServerProfile(base.Profile):
                                          message=six.text_type(ex))
 
         if server is None:
-            return None
-        # when booting a nova server from volume, the image property
-        # can be ignored.
-        # we try to find a volume which is bootable and use its image_id
-        # for the server.
-        if server.image:
-            image_id = server.image
-        elif server.attached_volumes:
-            cinder_driver = self.block_storage(obj)
-            for volume_ids in server.attached_volumes:
-                try:
-                    vs = cinder_driver.volume_get(volume_ids['id'])
-                    if vs.is_bootable:
-                        image_id = vs.volume_image_metadata['image_id']
-                except exc.InternalError as ex:
-                    raise exc.EResourceOperation(op='rebuild', type='server',
-                                                 id=obj.physical_id,
-                                                 message=six.text_type(ex))
-        else:
-            msg = _("server doesn't have an image and it has no "
-                    "bootable volume")
-            raise exc.EResourceOperation(op="rebuild", type="server",
-                                         id=obj.physical_id,
-                                         message=msg)
+            return None, False
+        image_id = options.get(self.IMAGE, None)
+        if not image_id:
+            image_id = self._get_image_id(obj, server, 'rebuilding')
 
         admin_pass = self.properties.get(self.ADMIN_PASS)
         name = self.properties[self.NAME] or obj.name
         try:
             nova_driver.server_rebuild(server_id, image_id,
                                        name, admin_pass)
-            nova_driver.wait_for_server(server_id, 'ACTIVE')
-            return server_id
+            nova_driver.wait_for_server(server_id, consts.VS_ACTIVE)
+            return server_id, True
         except exc.InternalError as ex:
             raise exc.EResourceOperation(op='rebuilding', type='server',
                                          id=server_id,
@@ -1576,12 +1644,85 @@ class ServerProfile(base.Profile):
 
     def handle_change_password(self, obj, **options):
         """Handler for the change_password operation."""
-        if not obj.physical_id:
-            return False
-
         password = options.get(self.ADMIN_PASSWORD, None)
         if (password is None or not isinstance(password, six.string_types)):
             return False
+        return self._handle_generic_op(obj, 'server_change_password',
+                                       'change_password',
+                                       new_password=password)
 
-        self.compute(obj).server_change_password(obj.physical_id, password)
-        return True
+    def handle_suspend(self, obj):
+        """Handler for the suspend operation."""
+        return self._handle_generic_op(obj, 'server_suspend',
+                                       'suspend', consts.VS_SUSPENDED)
+
+    def handle_resume(self, obj):
+        """Handler for the resume operation."""
+        return self._handle_generic_op(obj, 'server_resume',
+                                       'resume', consts.VS_ACTIVE)
+
+    def handle_start(self, obj):
+        """Handler for the start operation."""
+        return self._handle_generic_op(obj, 'server_start',
+                                       'start', consts.VS_ACTIVE)
+
+    def handle_stop(self, obj):
+        """Handler for the stop operation."""
+        return self._handle_generic_op(obj, 'server_stop',
+                                       'stop', consts.VS_SHUTOFF)
+
+    def handle_lock(self, obj):
+        """Handler for the lock operation."""
+        return self._handle_generic_op(obj, 'server_lock',
+                                       'lock')
+
+    def handle_unlock(self, obj):
+        """Handler for the unlock operation."""
+        return self._handle_generic_op(obj, 'server_unlock',
+                                       'unlock')
+
+    def handle_pause(self, obj):
+        """Handler for the pause operation."""
+        return self._handle_generic_op(obj, 'server_pause',
+                                       'pause', consts.VS_PAUSED)
+
+    def handle_unpause(self, obj):
+        """Handler for the unpause operation."""
+        return self._handle_generic_op(obj, 'server_unpause',
+                                       'unpause', consts.VS_ACTIVE)
+
+    def handle_rescue(self, obj, **options):
+        """Handler for the rescue operation."""
+        password = options.get(self.ADMIN_PASSWORD, None)
+        image = options.get(self.IMAGE, None)
+        if not image:
+            return False
+
+        self._validate_image(obj, image)
+        return self._handle_generic_op(obj, 'server_rescue',
+                                       'rescue', consts.VS_RESCUE,
+                                       admin_pass=password,
+                                       image_ref=image)
+
+    def handle_unrescue(self, obj):
+        """Handler for the unrescue operation."""
+        return self._handle_generic_op(obj, 'server_unrescue',
+                                       'unrescue', consts.VS_ACTIVE)
+
+    def handle_migrate(self, obj):
+        """Handler for the migrate operation."""
+        return self._handle_generic_op(obj, 'server_migrate',
+                                       'migrate', consts.VS_ACTIVE)
+
+    def handle_snapshot(self, obj):
+        """Handler for the snapshot operation."""
+        return self._handle_generic_op(obj, 'server_create_image',
+                                       'snapshot', consts.VS_ACTIVE,
+                                       name=obj.name)
+
+    def handle_restore(self, obj, **options):
+        """Handler for the restore operation."""
+        image = options.get(self.IMAGE, None)
+        if not image:
+            return False
+        return self.handle_rebuild(obj, **options)
