@@ -10,13 +10,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import copy
 import mock
 import six
 from webob import exc
 
 from oslo_serialization import jsonutils
+from oslo_utils import uuidutils
 
+from senlin.api.common import util
 from senlin.api.middleware import fault
 from senlin.api.openstack.v1 import profiles
 from senlin.common import exception as senlin_exc
@@ -26,36 +27,6 @@ from senlin.tests.unit.api import shared
 from senlin.tests.unit.common import base
 
 
-class ProfileDataTest(base.SenlinTestCase):
-    def test_profile_data(self):
-        body = {
-            'name': 'test_profile',
-            'spec': {
-                'type': 'test_profile_type',
-                'version': '1.0',
-                'properties': {
-                    'param1': 'value1',
-                    'param2': 'value2',
-                    'context': {
-                        'region': 'region1',
-                    },
-                }
-            },
-            'metadata': {}
-        }
-        data = profiles.ProfileData(body)
-        self.assertEqual('test_profile', data.name())
-        self.assertEqual(body['spec'], data.spec())
-        self.assertEqual({}, data.metadata())
-
-    def test_required_fields_missing(self):
-        body = {'not a profile name': 'wibble'}
-        data = profiles.ProfileData(body)
-        self.assertRaises(exc.HTTPBadRequest, data.name)
-        self.assertRaises(exc.HTTPBadRequest, data.spec)
-        self.assertIsNone(data.metadata())
-
-
 @mock.patch.object(policy, 'enforce')
 class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
     def setUp(self):
@@ -67,7 +38,10 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         cfgopts = DummyConfig()
         self.controller = profiles.ProfileController(options=cfgopts)
 
-    def test_profile_index_normal(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_index_normal(self, mock_call, mock_parse,
+                                  mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
         req = self._get('/profiles')
 
@@ -86,106 +60,100 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
             }
         ]
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     return_value=engine_resp)
+        mock_call.return_value = engine_resp
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
         result = self.controller.index(req)
 
-        default_args = {'limit': None, 'marker': None, 'filters': None,
-                        'sort': None, 'project_safe': True}
+        self.assertEqual(engine_resp, result['profiles'])
+        mock_parse.assert_called_once_with(
+            'ProfileListRequest', req, {'project_safe': True})
+        mock_call.assert_called_once_with(req.context, 'profile_list', obj)
 
-        mock_call.assert_called_with(req.context,
-                                     ('profile_list', default_args))
-
-        expected = {'profiles': engine_resp}
-        self.assertEqual(expected, result)
-
-    def test_profile_index_whitelists_params(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_index_whitelists_params(self, mock_call, mock_parse,
+                                             mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
+        marker_uuid = uuidutils.generate_uuid()
         params = {
+            'name': 'foo',
+            'type': 'fake_type',
             'limit': 20,
-            'marker': 'fake marker',
-            'sort': 'fake sorting string',
+            'marker': marker_uuid,
+            'sort': 'name:asc',
             'global_project': False
         }
         req = self._get('/profiles', params=params)
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         mock_call.return_value = []
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
-        self.controller.index(req)
+        result = self.controller.index(req)
 
-        rpc_call_args, _ = mock_call.call_args
-        engine_args = rpc_call_args[1][1]
+        self.assertEqual([], result['profiles'])
+        mock_parse.assert_called_once_with(
+            'ProfileListRequest', req,
+            {
+                'sort': 'name:asc',
+                'name': ['foo'],
+                'limit': '20',
+                'marker': marker_uuid,
+                'type': ['fake_type'],
+                'project_safe': True
+            })
+        mock_call.assert_called_once_with(req.context, 'profile_list', obj)
 
-        self.assertEqual(5, len(engine_args))
-        self.assertIn('limit', engine_args)
-        self.assertIn('marker', engine_args)
-        self.assertIn('sort', engine_args)
-        self.assertIn('filters', engine_args)
-        self.assertIn('project_safe', engine_args)
-
-    def test_profile_index_whitelist_bad_params(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_index_whitelist_bad_params(self, mock_call, mock_parse,
+                                                mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
         params = {
             'balrog': 'fake_value'
         }
         req = self._get('/profiles', params=params)
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
-        mock_call.return_value = []
-
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
         self.assertEqual("Invalid parameter balrog", six.text_type(ex))
+        self.assertFalse(mock_parse.called)
         self.assertFalse(mock_call.called)
 
-    def test_profile_index_whitelist_filter_params(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_index_global_project_not_bool(self, mock_call,
+                                                   mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
-        params = {
-            'type': 'some_type',
-            'name': 'fake name',
-        }
+        params = {'global_project': 'No'}
         req = self._get('/profiles', params=params)
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
-        mock_call.return_value = []
-
-        self.controller.index(req)
-
-        rpc_call_args, _ = mock_call.call_args
-        engine_args = rpc_call_args[1][1]
-        self.assertIn('filters', engine_args)
-
-        filters = engine_args['filters']
-        self.assertEqual(2, len(filters))
-        self.assertIn('name', filters)
-        self.assertIn('type', filters)
-
-    def test_profile_index_whitelist_filter_bad_params(self, mock_enforce):
-        self._mock_enforce_setup(mock_enforce, 'index', True)
-        params = {
-            'balrog': 'fake_value'
-        }
-        req = self._get('/profiles', params=params)
-
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
-
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
-        self.assertEqual("Invalid parameter balrog", six.text_type(ex))
+
+        self.assertEqual("Invalid value 'No' specified for 'global_project'",
+                         six.text_type(ex))
+        self.assertFalse(mock_parse.called)
         self.assertFalse(mock_call.called)
 
-    def test_profile_index_limit_non_int(self, mock_enforce):
-        mock_call = self.patchobject(rpc_client.EngineClient, 'profile_list',
-                                     return_value=[])
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_index_limit_non_int(self, mock_call, mock_parse,
+                                         mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'index', True)
 
         params = {'limit': 'abc'}
         req = self._get('/profiles', params=params)
-        ex = self.assertRaises(senlin_exc.InvalidParameter,
+
+        mock_parse.side_effect = exc.HTTPBadRequest("bad limit")
+        ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
-        self.assertIn("Invalid value 'abc' specified for 'limit'",
-                      six.text_type(ex))
+
+        self.assertEqual("bad limit", six.text_type(ex))
+        mock_parse.assert_called_once_with(
+            'ProfileListRequest', req, mock.ANY)
         self.assertFalse(mock_call.called)
 
     def test_profile_index_denied_policy(self, mock_enforce):
@@ -198,7 +166,10 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
-    def test_profile_create_success(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_create_success(self, mock_call, mock_parse,
+                                    mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'create', True)
         body = {
             'profile': {
@@ -231,46 +202,59 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         }
 
         req = self._post('/profiles', jsonutils.dumps(body))
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     return_value=engine_response)
+        mock_call.return_value = engine_response
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
         resp = self.controller.create(req, body=body)
 
-        mock_call.assert_called_with(
-            req.context,
-            ('profile_create', {
-                'name': 'test_profile',
-                'spec': {
-                    'type': 'test_profile_type',
-                    'version': '1.0',
-                    'properties': {
-                        'param_1': 'value1',
-                        'param_2': 2
-                    },
-                },
-                'metadata': {},
-            })
-        )
+        self.assertEqual(engine_response, resp['profile'])
+        mock_parse.assert_called_once_with(
+            'ProfileCreateRequest', req, body, 'profile')
+        mock_call.assert_called_once_with(
+            req.context, 'profile_create', obj)
 
-        expected = {'profile': engine_response}
-        self.assertEqual(expected, resp)
-
-    def test_profile_create_with_bad_body(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_create_with_no_profile(self, mock_call, mock_parse,
+                                            mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'create', True)
         body = {'name': 'test_profile'}
 
         req = self._post('/profiles', jsonutils.dumps(body))
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
+
+        mock_parse.side_effect = exc.HTTPBadRequest("bad body")
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.create,
                                req, body=body)
 
-        self.assertEqual("Malformed request data, missing 'profile' key "
-                         "in request body.", six.text_type(ex))
-
+        self.assertEqual("bad body", six.text_type(ex))
+        mock_parse.assert_called_once_with(
+            'ProfileCreateRequest', mock.ANY, body, 'profile')
         self.assertFalse(mock_call.called)
 
-    def test_profile_create_with_bad_type(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_create_with_profile_no_spec(self, mock_call,
+                                                 mock_parse, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'create', True)
+        body = {'profile': {'name': 'test_profile'}}
+
+        req = self._post('/profiles', jsonutils.dumps(body))
+        mock_parse.side_effect = exc.HTTPBadRequest("miss spec")
+        ex = self.assertRaises(exc.HTTPBadRequest,
+                               self.controller.create,
+                               req, body=body)
+
+        self.assertEqual("miss spec", six.text_type(ex))
+        mock_parse.assert_called_once_with(
+            'ProfileCreateRequest', mock.ANY, body, 'profile')
+        self.assertFalse(mock_call.called)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_create_with_bad_type(self, mock_call,
+                                          mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'create', True)
         type_name = 'unknown_type'
         body = {
@@ -286,20 +270,27 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         }
         req = self._post('/profiles', jsonutils.dumps(body))
 
+        obj = mock.Mock()
+        mock_parse.return_value = obj
         error = senlin_exc.ResourceNotFound(type='profile_type', id=type_name)
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     side_effect=error)
+        mock_call.side_effect = shared.to_remote_error(error)
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.create,
                                               req, body=body)
 
-        mock_call.assert_called_once_with(req.context,
-                                          ('profile_create', body['profile']))
         self.assertEqual(404, resp.json['code'])
         self.assertEqual('ResourceNotFound', resp.json['error']['type'])
+        mock_parse.assert_called_once_with(
+            'ProfileCreateRequest', mock.ANY, body, 'profile')
+        mock_call.assert_called_once_with(
+            req.context, 'profile_create', obj)
 
-    def test_profile_create_with_spec_validation_failed(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_create_with_spec_validation_failed(self, mock_call,
+                                                        mock_parse,
+                                                        mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'create', True)
         body = {
             'profile': {
@@ -313,19 +304,23 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
             }
         }
         req = self._post('/profiles', jsonutils.dumps(body))
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
         msg = 'Spec validation error (param): value'
-        error = senlin_exc.SpecValidationFailed(message=msg)
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     side_effect=error)
+        error = senlin_exc.InvalidSpec(message=msg)
+        mock_call.side_effect = shared.to_remote_error(error)
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.create,
                                               req, body=body)
-        mock_call.assert_called_once_with(req.context,
-                                          ('profile_create', body['profile']))
+
         self.assertEqual(400, resp.json['code'])
-        self.assertEqual('SpecValidationFailed', resp.json['error']['type'])
+        self.assertEqual('InvalidSpec', resp.json['error']['type'])
+        mock_parse.assert_called_once_with(
+            'ProfileCreateRequest', mock.ANY, body, 'profile')
+        mock_call.assert_called_once_with(
+            req.context, 'profile_create', obj)
 
     def test_profile_create_denied_policy(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'create', False)
@@ -347,7 +342,9 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
-    def test_profile_get_normal(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_get_normal(self, mock_call, mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'get', True)
         pid = 'aaaa-bbbb-cccc'
         req = self._get('/profiles/%(profile_id)s' % {'profile_id': pid})
@@ -365,25 +362,30 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
             u'metadata': {},
         }
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     return_value=engine_resp)
+        mock_call.return_value = engine_resp
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
         result = self.controller.get(req, profile_id=pid)
 
-        mock_call.assert_called_with(req.context,
-                                     ('profile_get', {'identity': pid}))
+        self.assertEqual(engine_resp, result['profile'])
+        mock_parse.assert_called_once_with(
+            'ProfileGetRequest', req, {'identity': pid})
+        mock_call.assert_called_once_with(
+            req.context, 'profile_get', obj)
 
-        expected = {'profile': engine_resp}
-        self.assertEqual(expected, result)
-
-    def test_profile_get_not_found(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_get_not_found(self, mock_call, mock_parse,
+                                   mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'get', True)
         pid = 'non-existent-profile'
         req = self._get('/profiles/%(profile_id)s' % {'profile_id': pid})
 
         error = senlin_exc.ResourceNotFound(type='profile', id=pid)
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         mock_call.side_effect = shared.to_remote_error(error)
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.get,
@@ -392,6 +394,11 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(404, resp.json['code'])
         self.assertEqual('ResourceNotFound', resp.json['error']['type'])
 
+        mock_parse.assert_called_once_with(
+            'ProfileGetRequest', mock.ANY, {'identity': pid})
+        mock_call.assert_called_once_with(
+            req.context, 'profile_get', obj)
+
     def test_profile_get_denied_policy(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'get', False)
         pid = 'non-existent-profile'
@@ -404,7 +411,10 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
-    def test_profile_update_normal(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_update_normal(self, mock_call, mock_parse,
+                                   mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'aaaa-bbbb-cccc'
         body = {
@@ -428,31 +438,42 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
             u'metadata': {u'author': u'thomas j'},
         }
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     return_value=engine_resp)
+        mock_call.return_value = engine_resp
+        obj = mock.Mock()
+        mock_parse.return_value = obj
+
         result = self.controller.update(req, profile_id=pid, body=body)
 
-        args = copy.deepcopy(body['profile'])
-        args['identity'] = pid
-        mock_call.assert_called_with(req.context, ('profile_update', args))
+        self.assertEqual(engine_resp, result['profile'])
+        mock_parse.assert_called_once_with(
+            'ProfileUpdateRequest', req, mock.ANY)
+        mock_call.assert_called_once_with(
+            req.context, 'profile_update', obj)
 
-        expected = {'profile': engine_resp}
-        self.assertEqual(expected, result)
-
-    def test_profile_update_no_body(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_update_no_body(self, mock_call, mock_parse,
+                                    mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'aaaa-bbbb-cccc'
         body = {'foo': 'bar'}
         req = self._put('/profiles/%(profile_id)s' % {'profile_id': pid},
                         jsonutils.dumps(body))
 
+        mock_parse.side_effect = exc.HTTPBadRequest("bad body")
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.update,
                                req, profile_id=pid, body=body)
-        self.assertEqual("Malformed request data, missing 'profile' key in "
-                         "request body.", six.text_type(ex))
 
-    def test_profile_update_no_name(self, mock_enforce):
+        self.assertEqual("Malformed request data, missing 'profile' key "
+                         "in request body.", six.text_type(ex))
+        self.assertFalse(mock_parse.called)
+        self.assertFalse(mock_call.called)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_update_no_name(self, mock_call, mock_parse,
+                                    mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'aaaa-bbbb-cccc'
         body = {
@@ -462,31 +483,48 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         req = self._put('/profiles/%(profile_id)s' % {'profile_id': pid},
                         jsonutils.dumps(body))
 
-        self.patchobject(rpc_client.EngineClient, 'call', return_value={})
-        result = self.controller.update(req, profile_id=pid, body=body)
-        self.assertEqual({'profile': {}}, result)
+        mock_call.return_value = {}
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
-    def test_profile_update_with_spec(self, mock_enforce):
+        result = self.controller.update(req, profile_id=pid, body=body)
+
+        self.assertEqual({}, result['profile'])
+        mock_parse.assert_called_once_with(
+            'ProfileUpdateRequest', req, mock.ANY)
+        mock_call.assert_called_once_with(
+            req.context, 'profile_update', obj)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_update_with_unexpected_field(self, mock_call,
+                                                  mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'aaaa-bbbb-cccc'
         body = {
             'profile': {
                 'name': 'new_profile',
                 'metadata': {'author': 'john d'},
-                'spec': {'param_1': 'value1'}
+                'foo': 'bar'
             }
         }
         req = self._put('/profiles/%(profile_id)s' % {'profile_id': pid},
                         jsonutils.dumps(body))
 
+        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.update,
                                req, profile_id=pid, body=body)
-        self.assertEqual("Updating the spec of a profile is not supported "
-                         "because it may cause state conflicts in engine.",
-                         six.text_type(ex))
 
-    def test_profile_update_not_found(self, mock_enforce):
+        self.assertEqual("bad param", six.text_type(ex))
+        mock_parse.assert_called_once_with(
+            'ProfileUpdateRequest', req, mock.ANY)
+        self.assertFalse(mock_call.called)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_update_not_found(self, mock_call, mock_parse,
+                                      mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'update', True)
         pid = 'non-existent-profile'
         body = {
@@ -499,7 +537,6 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
                         jsonutils.dumps(body))
 
         error = senlin_exc.ResourceNotFound(type='profile', id=pid)
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         mock_call.side_effect = shared.to_remote_error(error)
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
@@ -527,28 +564,36 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
-    def test_profile_delete_success(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_delete_success(self, mock_call, mock_parse,
+                                    mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'delete', True)
         pid = 'aaaa-bbbb-cccc'
         req = self._delete('/profiles/%(profile_id)s' % {'profile_id': pid})
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     return_value=None)
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
         self.assertRaises(exc.HTTPNoContent,
                           self.controller.delete, req, profile_id=pid)
 
-        mock_call.assert_called_with(
-            req.context, ('profile_delete', {'identity': pid}))
+        mock_parse.assert_called_once_with(
+            'ProfileDeleteRequest', req, {'identity': pid})
+        mock_call.assert_called_once_with(req.context, 'profile_delete', obj)
 
-    def test_profile_delete_not_found(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_delete_not_found(self, mock_call, mock_parse,
+                                      mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'delete', True)
         pid = 'aaaa-bbbb-cccc'
         req = self._delete('/profiles/%(profile_id)s' % {'profile_id': pid})
 
         error = senlin_exc.ResourceNotFound(type='profile', id=pid)
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         mock_call.side_effect = shared.to_remote_error(error)
+        obj = mock.Mock()
+        mock_parse.return_value = obj
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.delete,
@@ -557,14 +602,21 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(404, resp.json['code'])
         self.assertEqual('ResourceNotFound', resp.json['error']['type'])
 
-    def test_profile_delete_resource_in_use(self, mock_enforce):
+        mock_parse.assert_called_once_with(
+            'ProfileDeleteRequest', mock.ANY, {'identity': pid})
+        mock_call.assert_called_once_with(
+            req.context, 'profile_delete', obj)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_delete_resource_in_use(self, mock_call, mock_parse,
+                                            mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'delete', True)
         pid = 'aaaa-bbbb-cccc'
         req = self._delete('/profiles/%(profile_id)s' % {'profile_id': pid})
 
         error = senlin_exc.ResourceInUse(type='profile', id=pid,
                                          reason='still in use')
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
         mock_call.side_effect = shared.to_remote_error(error)
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
@@ -586,21 +638,24 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
-    def test_profile_validate_version_mismatch(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_validate_version_mismatch(self, mock_call, mock_parse,
+                                               mock_enforce):
         body = {
             'profile': {}
         }
         req = self._post('/profiles/validate', jsonutils.dumps(body),
                          version='1.1')
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
 
         ex = self.assertRaises(senlin_exc.MethodVersionNotFound,
                                self.controller.validate,
                                req, body=body)
 
-        mock_call.assert_not_called()
-        self.assertEqual('API version 1.1 is not supported on this method.',
-                         six.text_type(ex))
+        self.assertFalse(mock_parse.called)
+        self.assertFalse(mock_call.called)
+        self.assertEqual("API version '1.1' is not supported on this "
+                         "method.", six.text_type(ex))
 
     def test_profile_validate_denied_policy(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'validate', False)
@@ -630,19 +685,103 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(403, resp.status_int)
         self.assertIn('403 Forbidden', six.text_type(resp))
 
-    def test_profile_validate_no_body(self, mock_enforce):
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_validate_no_body(self, mock_call, mock_parse,
+                                      mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'validate', True)
         body = {'foo': 'bar'}
         req = self._post('/profiles/validate', jsonutils.dumps(body),
                          version='1.2')
 
+        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.validate,
                                req, body=body)
-        self.assertEqual("Malformed request data, missing 'profile' key in "
-                         "request body.", six.text_type(ex))
+        self.assertEqual("bad param", six.text_type(ex))
 
-    def test_profile_validate_success(self, mock_enforce):
+        mock_parse.assert_called_once_with(
+            'ProfileValidateRequest', req, body, 'profile')
+        self.assertFalse(mock_call.called)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_validate_no_spec(self, mock_call, mock_parse,
+                                      mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'validate', True)
+        body = {
+            'profile': {}
+        }
+        req = self._post('/profiles/validate', jsonutils.dumps(body),
+                         version='1.2')
+
+        mock_parse.side_effect = exc.HTTPBadRequest("miss spec")
+        ex = self.assertRaises(exc.HTTPBadRequest,
+                               self.controller.validate,
+                               req, body=body)
+        self.assertEqual("miss spec", six.text_type(ex))
+        mock_parse.assert_called_once_with(
+            'ProfileValidateRequest', req, body, 'profile')
+        self.assertFalse(mock_call.called)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_validate_unsupported_field(self, mock_call,
+                                                mock_parse, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'validate', True)
+        body = {
+            'profile': {
+                'spec': {'type': 'os.nova.server',
+                         'version': '1.0'},
+                'foo': 'bar'
+            }
+        }
+        req = self._post('/profiles/validate', jsonutils.dumps(body),
+                         version='1.2')
+        mock_parse.side_effect = exc.HTTPBadRequest("bad param")
+        ex = self.assertRaises(exc.HTTPBadRequest,
+                               self.controller.validate,
+                               req, body=body)
+
+        self.assertEqual("bad param", six.text_type(ex))
+        mock_parse.assert_called_once_with(
+            'ProfileValidateRequest', req, body, 'profile')
+        self.assertFalse(mock_call.called)
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_validate_invalid_spec(self, mock_call,
+                                           mock_parse, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'validate', True)
+        body = {
+            'profile': {
+                'spec': {
+                    'type': 'os.nova.server',
+                    'version': '1.0'
+                }
+            }
+        }
+
+        req = self._post('/profiles/validate', jsonutils.dumps(body),
+                         version='1.2')
+
+        msg = 'Spec validation error'
+        error = senlin_exc.InvalidSpec(message=msg)
+        mock_call.side_effect = shared.to_remote_error(error)
+        obj = mock.Mock()
+        mock_parse.return_value = obj
+
+        resp = shared.request_with_middleware(fault.FaultWrapper,
+                                              self.controller.validate,
+                                              req, body=body)
+
+        self.assertEqual(400, resp.json['code'])
+        self.assertEqual('InvalidSpec', resp.json['error']['type'])
+
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_profile_validate_success(self, mock_call,
+                                      mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'validate', True)
         spec = {
             'spec': {
@@ -657,10 +796,14 @@ class ProfileControllerTest(shared.ControllerTest, base.SenlinTestCase):
         req = self._post('/profiles/validate', jsonutils.dumps(body),
                          version='1.2')
 
-        mock_call = self.patchobject(rpc_client.EngineClient, 'call',
-                                     return_value=spec)
+        obj = mock.Mock()
+        mock_parse.return_value = obj
+        mock_call.return_value = spec
+
         result = self.controller.validate(req, body=body)
-        mock_call.assert_called_with(req.context,
-                                     ('profile_validate', spec))
-        expected = {'profile': spec}
-        self.assertEqual(expected, result)
+
+        self.assertEqual(spec, result['profile'])
+        mock_parse.assert_called_once_with(
+            'ProfileValidateRequest', req, body, 'profile')
+        mock_call.assert_called_with(
+            req.context, 'profile_validate', obj)
