@@ -33,7 +33,7 @@ def request_with_middleware(middleware, func, req, *args, **kwargs):
 
 
 def to_remote_error(error):
-    '''Prepend the given exception with the _Remote suffix.'''
+    """Prepend the given exception with the _Remote suffix."""
 
     exc_info = (type(error), error, None)
     serialized = rpc_common.serialize_remote_exception(exc_info)
@@ -43,7 +43,7 @@ def to_remote_error(error):
 
 
 class ControllerTest(object):
-    '''Common utilities for testing API Controllers.'''
+    """Common utilities for testing API Controllers."""
 
     def __init__(self, *args, **kwargs):
         super(ControllerTest, self).__init__(*args, **kwargs)
@@ -81,8 +81,8 @@ class ControllerTest(object):
     def _get(self, path, params=None, version=None):
         return self._simple_request(path, params=params, version=version)
 
-    def _delete(self, path, version=None):
-        return self._simple_request(path, method='DELETE')
+    def _delete(self, path, params=None, version=None):
+        return self._simple_request(path, params=params, method='DELETE')
 
     def _data_request(self, path, data, content_type='application/json',
                       method='POST', version=None):
