@@ -10,16 +10,16 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import uuid
 
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils as tu
+from oslo_utils import uuidutils
 
 from senlin.db.sqlalchemy import api as db_api
 from senlin.engine import parser
 
 
-sample_profile = '''
+sample_profile = """
   name: test_profile_name
   type: my_test_profile_type
   spec:
@@ -29,9 +29,9 @@ sample_profile = '''
         myrandom: OS::Heat::RandomString
       files:
         myfile: contents
-'''
+"""
 
-sample_action = '''
+sample_action = """
   name: test_cluster_create_action
   target: cluster_001
   action: create
@@ -44,18 +44,18 @@ sample_action = '''
     min_size: 1
     max_size: 10
     pause_time: PT10M
-'''
+"""
 
 
-UUIDs = (UUID1, UUID2, UUID3) = sorted([str(uuid.uuid4())
+UUIDs = (UUID1, UUID2, UUID3) = sorted([uuidutils.generate_uuid()
                                         for x in range(3)])
 
 
 def create_profile(context, profile=sample_profile, **kwargs):
     data = parser.simple_parse(profile)
-    data['user'] = context.user
-    data['project'] = context.project
-    data['domain'] = context.domain
+    data['user'] = context.user_id
+    data['project'] = context.project_id
+    data['domain'] = context.domain_id
     data.update(kwargs)
     return db_api.profile_create(context, data)
 
@@ -64,8 +64,8 @@ def create_cluster(ctx, profile, **kwargs):
     values = {
         'name': 'db_test_cluster_name',
         'profile_id': profile.id,
-        'user': ctx.user,
-        'project': ctx.project,
+        'user': ctx.user_id,
+        'project': ctx.project_id,
         'domain': 'unknown',
         'parent': None,
         'next_index': 1,
@@ -75,6 +75,8 @@ def create_cluster(ctx, profile, **kwargs):
         'status': 'INIT',
         'status_reason': 'Just Initialized',
         'meta_data': {},
+        'dependents': {},
+        'config': {},
     }
     values.update(kwargs)
     if 'project' in kwargs:
@@ -95,7 +97,7 @@ def create_node(ctx, cluster, profile, **kwargs):
         'physical_id': UUID1,
         'cluster_id': cluster_id,
         'profile_id': profile.id,
-        'project': ctx.project,
+        'project': ctx.project_id,
         'index': index,
         'role': None,
         'created_at': None,
@@ -104,6 +106,7 @@ def create_node(ctx, cluster, profile, **kwargs):
         'status_reason': 'create complete',
         'meta_data': jsonutils.loads('{"foo": "123"}'),
         'data': jsonutils.loads('{"key1": "value1"}'),
+        'dependents': {},
     }
     values.update(kwargs)
     return db_api.node_create(ctx, values)
@@ -112,9 +115,9 @@ def create_node(ctx, cluster, profile, **kwargs):
 def create_webhook(ctx, obj_id, obj_type, action, **kwargs):
     values = {
         'name': 'test_webhook_name',
-        'user': ctx.user,
-        'project': ctx.project,
-        'domain': ctx.domain,
+        'user': ctx.user_id,
+        'project': ctx.project_id,
+        'domain': ctx.domain_id,
         'created_at': None,
         'obj_id': obj_id,
         'obj_type': obj_type,
@@ -148,9 +151,9 @@ def create_policy(ctx, **kwargs):
     values = {
         'name': 'test_policy',
         'type': 'senlin.policy.scaling',
-        'user': ctx.user,
-        'project': ctx.project,
-        'domain': ctx.domain,
+        'user': ctx.user_id,
+        'project': ctx.project_id,
+        'domain': ctx.domain_id,
         'spec': {
             'type': 'senlin.policy.scaling',
             'version': '1.0',
@@ -174,8 +177,8 @@ def create_event(ctx, **kwargs):
         'obj_type': 'CLUSTER',
         'cluster_id': 'FAKE_CLUSTER',
         'level': '20',
-        'user': ctx.user,
-        'project': ctx.project,
+        'user': ctx.user_id,
+        'project': ctx.project_id,
         'action': 'DANCE',
         'status': 'READY',
         'status_reason': 'Just created.',
