@@ -32,9 +32,9 @@ class DBAPIClusterPolicyTest(base.SenlinTestCase):
         data = {
             'name': 'test_policy',
             'type': 'ScalingPolicy',
-            'user': self.ctx.user,
-            'project': self.ctx.project,
-            'domain': self.ctx.domain,
+            'user': self.ctx.user_id,
+            'project': self.ctx.project_id,
+            'domain': self.ctx.domain_id,
             'spec': {
                 'min_size': 1,
                 'max_size': 10,
@@ -140,6 +140,17 @@ class DBAPIClusterPolicyTest(base.SenlinTestCase):
         self.assertEqual(1, len(bindings))
         self.assertEqual(timestamp, bindings[0].last_op)
 
+    def test_cluster_policy_get(self):
+        policy = self.create_policy()
+
+        db_api.cluster_policy_attach(self.ctx, self.cluster.id, policy.id, {})
+
+        binding = db_api.cluster_policy_get(self.ctx, self.cluster.id,
+                                            policy.id)
+        self.assertIsNotNone(binding)
+        self.assertEqual(self.cluster.id, binding.cluster_id)
+        self.assertEqual(policy.id, binding.policy_id)
+
     def test_policy_get_all_with_empty_filters(self):
         for pid in ['policy1', 'policy2']:
             self.create_policy(id=pid)
@@ -206,6 +217,19 @@ class DBAPIClusterPolicyTest(base.SenlinTestCase):
                                                     'UnknownPolicy')
         self.assertEqual(0, len(results))
 
+    def test_policy_get_all_by_policy_name(self):
+        for pid in ['policy1', 'policy2']:
+            self.create_policy(id=pid)
+            db_api.cluster_policy_attach(self.ctx, self.cluster.id, pid, {})
+
+        results = db_api.cluster_policy_get_by_name(self.ctx, self.cluster.id,
+                                                    'test_policy')
+        self.assertEqual(2, len(results))
+
+        results = db_api.cluster_policy_get_by_name(self.ctx, self.cluster.id,
+                                                    'unknown_policy')
+        self.assertEqual(0, len(results))
+
     def test_policy_get_all_by_policy_type_with_filter(self):
         for pid in ['policy1', 'policy2']:
             self.create_policy(id=pid)
@@ -223,3 +247,82 @@ class DBAPIClusterPolicyTest(base.SenlinTestCase):
                                                     'ScalingPolicy',
                                                     filters=filters)
         self.assertEqual(0, len(results))
+
+    def test_policy_get_all_by_policy_name_with_filter(self):
+        for pid in ['policy1', 'policy2']:
+            self.create_policy(id=pid)
+            db_api.cluster_policy_attach(self.ctx, self.cluster.id, pid,
+                                         {'enabled': True})
+
+        filters = {'enabled': True}
+        results = db_api.cluster_policy_get_by_name(self.ctx, self.cluster.id,
+                                                    'test_policy',
+                                                    filters=filters)
+        self.assertEqual(2, len(results))
+
+        filters = {'enabled': False}
+        results = db_api.cluster_policy_get_by_name(self.ctx, self.cluster.id,
+                                                    'test_policy',
+                                                    filters=filters)
+        self.assertEqual(0, len(results))
+
+    def test_policy_get_all_with_all_filters(self):
+        for pid in ['policy1', 'policy2']:
+            self.create_policy(id=pid)
+            db_api.cluster_policy_attach(self.ctx, self.cluster.id, pid,
+                                         {'enabled': True})
+
+        filters = {'enabled': True,
+                   'policy_name': 'test_policy',
+                   'policy_type': 'ScalingPolicy'}
+        results = db_api.cluster_policy_get_all(self.ctx, self.cluster.id,
+                                                filters=filters)
+        self.assertEqual(2, len(results))
+
+        filters = {'enabled': True,
+                   'policy_type': 'ScalingPolicy'}
+        results = db_api.cluster_policy_get_all(self.ctx, self.cluster.id,
+                                                filters=filters)
+        self.assertEqual(2, len(results))
+
+        filters = {'enabled': True,
+                   'policy_name': 'test_policy'}
+        results = db_api.cluster_policy_get_all(self.ctx, self.cluster.id,
+                                                filters=filters)
+        self.assertEqual(2, len(results))
+
+        filters = {'enabled': True,
+                   'policy_name': 'wrong_name',
+                   'policy_type': 'wrong_type'}
+        results = db_api.cluster_policy_get_all(self.ctx, self.cluster.id,
+                                                filters=filters)
+        self.assertEqual(0, len(results))
+
+        filters = {'policy_name': 'test_policy'}
+        results = db_api.cluster_policy_get_all(self.ctx, self.cluster.id,
+                                                filters=filters)
+        self.assertEqual(2, len(results))
+
+        filters = {'policy_type': 'ScalingPolicy'}
+        results = db_api.cluster_policy_get_all(self.ctx, self.cluster.id,
+                                                filters=filters)
+        self.assertEqual(2, len(results))
+
+        filters = {'enabled': False}
+        results = db_api.cluster_policy_get_all(self.ctx, self.cluster.id,
+                                                filters=filters)
+        self.assertEqual(0, len(results))
+
+    def test_cluster_policy_ids_by_cluster(self):
+        # prepare
+        ids = []
+        for i in range(3):
+            policy_id = self.create_policy().id
+            ids.append(policy_id)
+            db_api.cluster_policy_attach(self.ctx, self.cluster.id, policy_id,
+                                         {'enabled': True})
+
+        # sorted by enabled, the 2nd and 3rd are unpredictable
+        results = db_api.cluster_policy_ids_by_cluster(self.ctx,
+                                                       self.cluster.id)
+        self.assertEqual(set(ids), set(results))
