@@ -14,6 +14,7 @@ from oslo_log import log as logging
 from oslo_utils import reflection
 from oslo_utils import timeutils as tu
 
+from senlin.common import utils as common_utils
 from senlin.db.sqlalchemy import api as db_api
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
@@ -61,8 +62,8 @@ class DBAPIEventTest(base.SenlinTestCase):
             'action': action or '',
             'status': status or '',
             'status_reason': status_reason or '',
-            'user': ctx.user,
-            'project': ctx.project,
+            'user': ctx.user_id,
+            'project': ctx.project_id,
         }
 
         # Make sure all fields can be customized
@@ -75,8 +76,8 @@ class DBAPIEventTest(base.SenlinTestCase):
         tst_timestamp = tu.parse_strtime('2014-12-19 11:51:54.670244',
                                          '%Y-%m-%d %H:%M:%S.%f')
 
-        self.assertEqual(tu.isotime(tst_timestamp),
-                         tu.isotime(ret_event.timestamp))
+        self.assertEqual(common_utils.isotime(tst_timestamp),
+                         common_utils.isotime(ret_event.timestamp))
         self.assertEqual('20', ret_event.level)
         self.assertEqual('', ret_event.oid)
         self.assertEqual('', ret_event.otype)
@@ -84,8 +85,8 @@ class DBAPIEventTest(base.SenlinTestCase):
         self.assertEqual('', ret_event.action)
         self.assertEqual('', ret_event.status)
         self.assertEqual('', ret_event.status_reason)
-        self.assertEqual(self.ctx.user, ret_event.user)
-        self.assertEqual(self.ctx.project, ret_event.project)
+        self.assertEqual(self.ctx.user_id, ret_event.user)
+        self.assertEqual(self.ctx.project_id, ret_event.project)
 
     def test_event_get_diff_project(self):
         event = self.create_event(self.ctx)
@@ -101,6 +102,8 @@ class DBAPIEventTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a-different-project',
                                         is_admin=True)
         res = db_api.event_get(admin_ctx, event.id, project_safe=True)
+        self.assertIsNone(res)
+        res = db_api.event_get(admin_ctx, event.id, project_safe=False)
         self.assertIsNotNone(res)
 
     def test_event_get_by_short_id(self):
@@ -225,11 +228,11 @@ class DBAPIEventTest(base.SenlinTestCase):
         self.assertEqual(event3.id, events[2].id)
 
     def test_event_get_all_project_safe(self):
-        self.ctx.project = 'project_1'
+        self.ctx.project_id = 'project_1'
         cluster1 = shared.create_cluster(self.ctx, self.profile,
                                          name='cluster1')
         self.create_event(self.ctx, entity=cluster1)
-        self.ctx.project = 'project_2'
+        self.ctx.project_id = 'project_2'
         cluster2 = shared.create_cluster(self.ctx, self.profile,
                                          name='cluster2')
         self.create_event(self.ctx, entity=cluster2, action='CLUSTER_CREATE')
@@ -258,11 +261,11 @@ class DBAPIEventTest(base.SenlinTestCase):
         self.assertIn(cluster2.name, onames)
 
     def test_event_get_all_admin_context(self):
-        self.ctx.project = 'project_1'
+        self.ctx.project_id = 'project_1'
         cluster1 = shared.create_cluster(self.ctx, self.profile,
                                          name='cluster1')
         self.create_event(self.ctx, entity=cluster1)
-        self.ctx.project = 'project_2'
+        self.ctx.project_id = 'project_2'
         cluster2 = shared.create_cluster(self.ctx, self.profile,
                                          name='cluster2')
         self.create_event(self.ctx, entity=cluster2, action='CLUSTER_CREATE')
@@ -351,6 +354,9 @@ class DBAPIEventTest(base.SenlinTestCase):
                                         is_admin=True)
         events = db_api.event_get_all_by_cluster(admin_ctx, cluster1.id,
                                                  project_safe=True)
+        self.assertEqual(0, len(events))
+        events = db_api.event_get_all_by_cluster(admin_ctx, cluster1.id,
+                                                 project_safe=False)
         self.assertEqual(2, len(events))
 
     def test_event_count_all_by_cluster(self):
@@ -417,6 +423,9 @@ class DBAPIEventTest(base.SenlinTestCase):
 
         res = db_api.event_count_by_cluster(admin_ctx, cluster1.id,
                                             project_safe=True)
+        self.assertEqual(0, res)
+        res = db_api.event_count_by_cluster(admin_ctx, cluster1.id,
+                                            project_safe=False)
         self.assertEqual(1, res)
 
     def test_event_get_all_filtered(self):
@@ -520,3 +529,22 @@ class DBAPIEventTest(base.SenlinTestCase):
         db_api.event_prune(self.ctx, cluster1.id)
         res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
         self.assertEqual(0, len(res))
+
+    def test_event_purge(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+        node1_1 = shared.create_node(self.ctx, cluster1, self.profile)
+        node1_2 = shared.create_node(self.ctx, cluster1, self.profile)
+
+        self.create_event(self.ctx, entity=cluster1, status='start')
+        self.create_event(self.ctx, entity=cluster1, status='end')
+        self.create_event(self.ctx, entity=node1_1, status='start')
+        self.create_event(self.ctx, entity=node1_1, status='end')
+        timestamp = tu.utcnow()
+        self.create_event(self.ctx, timestamp=timestamp,
+                          entity=node1_2, status='start')
+
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(5, len(res))
+        db_api.event_purge(project=None, granularity='days', age=5)
+        res = db_api.event_get_all_by_cluster(self.ctx, cluster1.id)
+        self.assertEqual(1, len(res))
