@@ -36,10 +36,10 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.cluster = shared.create_cluster(self.ctx, self.profile)
 
     def test_node_create(self):
-        nodes = db_api.node_get_all_by_cluster(self.ctx, self.cluster.id)
-        self.assertEqual(0, len(nodes))
         res = shared.create_node(self.ctx, self.cluster, self.profile)
+
         node = db_api.node_get(self.ctx, res.id)
+
         self.assertIsNotNone(node)
         self.assertEqual('test_node_name', node.name)
         self.assertEqual(UUID1, node.physical_id)
@@ -54,18 +54,14 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(self.cluster.id, node.cluster_id)
         self.assertEqual(self.profile.id, node.profile_id)
 
-        nodes = db_api.node_get_all_by_cluster(self.ctx, self.cluster.id)
-        self.assertEqual(1, len(nodes))
-
     def test_node_get(self):
         res = shared.create_node(self.ctx, self.cluster, self.profile)
+
         node = db_api.node_get(self.ctx, res.id)
         self.assertIsNotNone(node)
 
         node = db_api.node_get(self.ctx, UUID2)
         self.assertIsNone(node)
-        nodes = db_api.node_get_all_by_cluster(self.ctx, self.cluster.id)
-        self.assertEqual(1, len(nodes))
 
     def test_node_get_diff_project(self):
         res = shared.create_node(self.ctx, self.cluster, self.profile)
@@ -83,6 +79,8 @@ class DBAPINodeTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a_different_project',
                                         is_admin=True)
         node = db_api.node_get(admin_ctx, res.id, project_safe=True)
+        self.assertIsNone(node)
+        node = db_api.node_get(admin_ctx, res.id, project_safe=False)
         self.assertIsNotNone(node)
 
     def test_node_get_by_name(self):
@@ -95,6 +93,12 @@ class DBAPINodeTest(base.SenlinTestCase):
         res = db_api.node_get_by_name(self.ctx, 'BogusName')
         self.assertIsNone(res)
 
+        # duplicated name
+        shared.create_node(self.ctx, self.cluster, self.profile)
+        self.assertRaises(exception.MultipleChoices,
+                          db_api.node_get_by_name,
+                          self.ctx, 'test_node_name')
+
     def test_node_get_by_name_diff_project(self):
         shared.create_node(self.ctx, self.cluster, self.profile)
         res = db_api.node_get_by_name(self.ctx, 'test_node_name')
@@ -150,6 +154,9 @@ class DBAPINodeTest(base.SenlinTestCase):
                                         is_admin=True)
         res = db_api.node_get_by_short_id(admin_ctx, node_id[:11],
                                           project_safe=True)
+        self.assertIsNone(res)
+        res = db_api.node_get_by_short_id(admin_ctx, node_id[:11],
+                                          project_safe=False)
         self.assertIsNotNone(res)
 
     def test_node_get_all(self):
@@ -162,6 +169,68 @@ class DBAPINodeTest(base.SenlinTestCase):
         names = [node.name for node in nodes]
         [self.assertIn(val['name'], names) for val in values]
 
+    def test_node_add_node_dependents(self):
+        node_id = 'host_node'
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, name='node-1')
+        db_api.node_add_dependents(self.ctx, node_id, 'NODE1')
+        node = db_api.node_get(self.ctx, node_id)
+        nodes = node.dependents['nodes']
+        self.assertEqual(['NODE1'], nodes)
+
+        db_api.node_add_dependents(self.ctx, node_id, 'NODE2')
+        new_node = db_api.node_get(self.ctx, node_id)
+        nodes = new_node.dependents['nodes']
+        self.assertEqual(['NODE1', 'NODE2'], nodes)
+
+    def test_node_add_profile_dependents(self):
+        node_id = 'host_node'
+        new_profile = shared.create_profile(self.ctx)
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, name='node-1')
+        db_api.node_add_dependents(self.ctx, node_id, new_profile.id,
+                                   'profile')
+        node = db_api.node_get(self.ctx, node_id)
+        nodes = node.dependents['profiles']
+        self.assertEqual([new_profile.id], nodes)
+
+        new_profile_1 = shared.create_profile(self.ctx)
+        db_api.node_add_dependents(self.ctx, node_id, new_profile_1.id,
+                                   'profile')
+        new_node = db_api.node_get(self.ctx, node_id)
+        nodes = new_node.dependents['profiles']
+        self.assertEqual([new_profile.id, new_profile_1.id], nodes)
+
+    def test_node_remove_node_dependents(self):
+        node_id = 'host_node'
+        dependents = {'nodes': ['NODE1', 'NODE2']}
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, dependents=dependents)
+        db_api.node_remove_dependents(self.ctx, node_id, 'NODE1')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({'nodes': ['NODE2']}, dependents)
+
+        db_api.node_remove_dependents(self.ctx, node_id, 'NODE2', 'node')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({}, dependents)
+
+    def test_node_remove_profile_dependents(self):
+        node_id = 'host_node'
+        dependents = {'profiles': ['P1', 'P2']}
+        node = shared.create_node(self.ctx, None, self.profile,
+                                  id=node_id, dependents=dependents)
+        db_api.node_remove_dependents(self.ctx, node_id, 'P1', 'profile')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({'profiles': ['P2']}, dependents)
+
+        db_api.node_remove_dependents(self.ctx, node_id, 'P2', 'profile')
+        node = db_api.node_get(self.ctx, node_id)
+        dependents = node.dependents
+        self.assertEqual({}, dependents)
+
     def test_node_get_all_with_cluster_id(self):
         values = [{'name': 'node1'}, {'name': 'node2'}, {'name': 'node3'}]
         for v in values:
@@ -282,11 +351,11 @@ class DBAPINodeTest(base.SenlinTestCase):
         shared.create_node(self.ctx, None, self.profile, name='node1')
         shared.create_node(self.ctx, None, self.profile, name='node2')
 
-        self.ctx.project = 'a-different-project'
+        self.ctx.project_id = 'a-different-project'
         results = db_api.node_get_all(self.ctx, project_safe=False)
         self.assertEqual(2, len(results))
 
-        self.ctx.project = 'a-different-project'
+        self.ctx.project_id = 'a-different-project'
         results = db_api.node_get_all(self.ctx)
         self.assertEqual(0, len(results))
 
@@ -304,7 +373,7 @@ class DBAPINodeTest(base.SenlinTestCase):
         results = db_api.node_get_all(admin_ctx, project_safe=False)
         self.assertEqual(2, len(results))
 
-    def test_node_get_by_cluster(self):
+    def test_get_all_by_cluster(self):
         cluster1 = shared.create_cluster(self.ctx, self.profile)
 
         node0 = shared.create_node(self.ctx, None, self.profile)
@@ -331,7 +400,25 @@ class DBAPINodeTest(base.SenlinTestCase):
         self.assertEqual(1, len(nodes))
         self.assertEqual(node3.id, nodes[0].id)
 
-    def test_node_get_by_cluster_diff_project(self):
+    def test_get_all_by_cluster_with_filters(self):
+        cluster1 = shared.create_cluster(self.ctx, self.profile)
+
+        shared.create_node(self.ctx, None, self.profile, role="slave")
+        node1 = shared.create_node(self.ctx, self.cluster, self.profile,
+                                   role="slave")
+        shared.create_node(self.ctx, self.cluster, self.profile, role="master")
+        shared.create_node(self.ctx, cluster1, self.profile, role="unknown")
+
+        nodes = db_api.node_get_all_by_cluster(self.ctx, self.cluster.id,
+                                               filters={"role": ["slave"]})
+        self.assertEqual(1, len(nodes))
+        self.assertEqual(node1.id, nodes[0].id)
+
+        nodes = db_api.node_get_all_by_cluster(self.ctx, cluster1.id,
+                                               filters={"role": "master"})
+        self.assertEqual(0, len(nodes))
+
+    def test_get_all_by_cluster_diff_project(self):
         shared.create_cluster(self.ctx, self.profile)
 
         node1 = shared.create_node(self.ctx, self.cluster, self.profile)
@@ -349,7 +436,7 @@ class DBAPINodeTest(base.SenlinTestCase):
                                                project_safe=False)
         self.assertEqual(2, len(nodes))
 
-    def test_node_get_by_cluster_admin_context(self):
+    def test_get_all_by_cluster_admin_context(self):
         shared.create_cluster(self.ctx, self.profile)
 
         node1 = shared.create_node(self.ctx, self.cluster, self.profile)
@@ -417,6 +504,37 @@ class DBAPINodeTest(base.SenlinTestCase):
                                            project_safe=False)
         self.assertEqual(2, res)
 
+    def test_ids_by_cluster(self):
+        node0 = shared.create_node(self.ctx, None, self.profile)
+        node1 = shared.create_node(self.ctx, self.cluster, self.profile)
+        node2 = shared.create_node(self.ctx, self.cluster, self.profile)
+
+        results = db_api.node_ids_by_cluster(self.ctx, self.cluster.id)
+        self.assertEqual(2, len(results))
+        self.assertEqual(set([node1.id, node2.id]), set(results))
+
+        # retrieve orphan nodes
+        results = db_api.node_ids_by_cluster(self.ctx, '')
+        self.assertEqual(1, len(results))
+        self.assertEqual(node0.id, results[0])
+
+    def test_ids_by_cluster_with_filters(self):
+        node0 = shared.create_node(self.ctx, None, self.profile,
+                                   role='slave')
+        node1 = shared.create_node(self.ctx, self.cluster, self.profile,
+                                   role='master')
+        shared.create_node(self.ctx, self.cluster, self.profile)
+
+        results = db_api.node_ids_by_cluster(self.ctx, self.cluster.id,
+                                             filters={'role': 'master'})
+        self.assertEqual(1, len(results))
+        self.assertEqual(node1.id, results[0])
+
+        # retrieve orphan nodes
+        results = db_api.node_ids_by_cluster(self.ctx, '')
+        self.assertEqual(1, len(results))
+        self.assertEqual(node0.id, results[0])
+
     def test_node_update(self):
         node = shared.create_node(self.ctx, self.cluster, self.profile)
         new_attributes = {
@@ -436,7 +554,7 @@ class DBAPINodeTest(base.SenlinTestCase):
         ex = self.assertRaises(exception.ResourceNotFound,
                                db_api.node_update,
                                self.ctx, 'BogusId', new_attributes)
-        self.assertEqual('The node (BogusId) could not be found.',
+        self.assertEqual("The node 'BogusId' could not be found.",
                          six.text_type(ex))
 
     def test_node_update_cluster_status_updated(self):
