@@ -12,15 +12,17 @@
 
 import mock
 from oslo_config import cfg
-from oslo_utils import timeutils
 import six
 
+from senlin.common import consts
 from senlin.common import exception
 from senlin.engine import cluster as cm
 from senlin.engine import cluster_policy as cpm
+from senlin.engine import health_manager
 from senlin.engine import node as node_mod
 from senlin.objects import cluster as co
 from senlin.objects import cluster_policy as cpo
+from senlin.objects import node as no
 from senlin.policies import base as pcb
 from senlin.profiles import base as pfb
 from senlin.tests.unit.common import base
@@ -60,6 +62,8 @@ class TestCluster(base.SenlinTestCase):
         self.assertEqual('Initializing', cluster.status_reason)
         self.assertEqual({}, cluster.data)
         self.assertEqual({}, cluster.metadata)
+        self.assertEqual({}, cluster.dependents)
+        self.assertEqual({}, cluster.config)
         self.assertEqual({'profile': None, 'nodes': [], 'policies': []},
                          cluster.rt)
 
@@ -82,7 +86,7 @@ class TestCluster(base.SenlinTestCase):
     @mock.patch.object(cpo.ClusterPolicy, 'get_all')
     @mock.patch.object(pcb.Policy, 'load')
     @mock.patch.object(pfb.Profile, 'load')
-    @mock.patch.object(node_mod.Node, 'load_all')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     def test__load_runtime_data(self, mock_nodes, mock_profile, mock_policy,
                                 mock_pb):
         x_binding = mock.Mock()
@@ -104,29 +108,35 @@ class TestCluster(base.SenlinTestCase):
         rt = cluster.rt
         self.assertEqual(x_profile, rt['profile'])
         self.assertEqual([x_node_1, x_node_2], rt['nodes'])
+        self.assertEqual(2, len(rt['nodes']))
+        self.assertIsInstance(rt['nodes'], list)
         self.assertEqual([x_policy], rt['policies'])
 
         mock_pb.assert_called_once_with(self.context, CLUSTER_ID)
-        mock_policy.assert_called_once_with(self.context, POLICY_ID)
+        mock_policy.assert_called_once_with(self.context,
+                                            POLICY_ID,
+                                            project_safe=False)
         mock_profile.assert_called_once_with(self.context,
                                              profile_id=PROFILE_ID,
                                              project_safe=False)
-        mock_nodes.assert_called_once_with(self.context,
-                                           cluster_id=CLUSTER_ID)
+        mock_nodes.assert_called_once_with(self.context, CLUSTER_ID)
 
     def test__load_runtime_data_id_is_none(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
 
         cluster._load_runtime_data(self.context)
 
-        self.assertIsNone(cluster.rt['profile'])
-        self.assertEqual([], cluster.rt['nodes'])
-        self.assertEqual([], cluster.rt['policies'])
+        rt = cluster.rt
+        self.assertIsNone(rt['profile'])
+        self.assertEqual([], rt['nodes'])
+        self.assertEqual(0, len(rt['nodes']))
+        self.assertIsInstance(rt['nodes'], list)
+        self.assertEqual([], rt['policies'])
 
     def test_store_for_create(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
-                             user=self.context.user,
-                             project=self.context.project)
+                             user=self.context.user_id,
+                             project=self.context.project_id)
         mock_load = self.patchobject(cluster, '_load_runtime_data')
         self.assertIsNone(cluster.id)
 
@@ -139,9 +149,9 @@ class TestCluster(base.SenlinTestCase):
         self.assertIsNotNone(result)
         self.assertEqual('test-cluster', result.name)
         self.assertEqual(PROFILE_ID, result.profile_id)
-        self.assertEqual(self.context.user, result.user)
-        self.assertEqual(self.context.project, result.project)
-        self.assertEqual(self.context.domain, result.domain)
+        self.assertEqual(self.context.user_id, result.user)
+        self.assertEqual(self.context.project_id, result.project)
+        self.assertEqual(self.context.domain_id, result.domain)
 
         self.assertIsNotNone(result.init_at)
         self.assertIsNone(result.created_at)
@@ -159,8 +169,8 @@ class TestCluster(base.SenlinTestCase):
 
     def test_store_for_update(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
-                             user=self.context.user,
-                             project=self.context.project)
+                             user=self.context.user_id,
+                             project=self.context.project_id)
         mock_load = self.patchobject(cluster, '_load_runtime_data')
         self.assertIsNone(cluster.id)
 
@@ -178,6 +188,7 @@ class TestCluster(base.SenlinTestCase):
         cluster.timeout = 120
         cluster.data = {'FOO': 'BAR'}
         cluster.metadata = {'KEY': 'VALUE'}
+        cluster.config = {'KEY': 'VALUE'}
 
         new_id = cluster.store(self.context)
         self.assertEqual(cluster_id, new_id)
@@ -186,8 +197,8 @@ class TestCluster(base.SenlinTestCase):
         self.assertIsNotNone(result)
 
         self.assertEqual('test-cluster-1', result.name)
-        self.assertEqual(self.context.user, result.user)
-        self.assertEqual(self.context.project, result.project)
+        self.assertEqual(self.context.user_id, result.user)
+        self.assertEqual(self.context.project_id, result.project)
 
         self.assertEqual(1, result.min_size)
         self.assertEqual(3, result.max_size)
@@ -196,6 +207,7 @@ class TestCluster(base.SenlinTestCase):
         self.assertEqual(120, result.timeout)
         self.assertEqual({'FOO': 'BAR'}, result.data)
         self.assertEqual({'KEY': 'VALUE'}, result.metadata)
+        self.assertEqual({'KEY': 'VALUE'}, result.config)
 
     @mock.patch.object(cm.Cluster, '_from_object')
     def test_load_via_db_object(self, mock_init):
@@ -225,7 +237,7 @@ class TestCluster(base.SenlinTestCase):
         ex = self.assertRaises(exception.ResourceNotFound,
                                cm.Cluster.load,
                                self.context, cluster_id=CLUSTER_ID)
-        self.assertEqual('The cluster (%s) could not be found.' % CLUSTER_ID,
+        self.assertEqual("The cluster '%s' could not be found." % CLUSTER_ID,
                          six.text_type(ex))
         mock_get.assert_called_once_with(self.context, CLUSTER_ID,
                                          project_safe=True)
@@ -252,56 +264,14 @@ class TestCluster(base.SenlinTestCase):
             mock.call(self.context, x_obj_1),
             mock.call(self.context, x_obj_2)])
 
-    @mock.patch.object(cm.Cluster, '_load_runtime_data')
-    def test_to_dict(self, mock_load):
-        values = {
-            'id': CLUSTER_ID,
-            'profile_id': PROFILE_ID,
-            'name': 'test-cluster',
-            'desired_capacity': 1,
-            'status': 'INIT',
-            'init_at': timeutils.utcnow(True),
-            'user': self.context.user,
-            'project': self.context.project,
-        }
-
-        cluster = co.Cluster.create(self.context, values)
-
-        expected = {
-            'id': CLUSTER_ID,
-            'name': cluster.name,
-            'profile_id': PROFILE_ID,
-            'user': cluster.user,
-            'project': cluster.project,
-            'domain': cluster.domain,
-            'init_at': mock.ANY,
-            'created_at': None,
-            'updated_at': None,
-            'min_size': 0,
-            'max_size': -1,
-            'desired_capacity': 1,
-            'timeout': cfg.CONF.default_action_timeout,
-            'status': 'INIT',
-            'status_reason': None,
-            'metadata': {},
-            'data': None,
-            'dependents': {},
-            'nodes': [],
-            'policies': [],
-            'profile_name': None,
-        }
-
-        result = cm.Cluster.load(self.context, cluster_id=CLUSTER_ID)
-        self.assertEqual(expected, result.to_dict())
-
     @mock.patch.object(co.Cluster, 'update')
     def test_set_status_for_create(self, mock_update):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                              id=CLUSTER_ID, status='CREATING')
 
-        cluster.set_status(self.context, cluster.ACTIVE, 'Cluster created')
+        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster created')
 
-        self.assertEqual(cluster.ACTIVE, cluster.status)
+        self.assertEqual(consts.CS_ACTIVE, cluster.status)
         self.assertEqual('Cluster created', cluster.status_reason)
         self.assertIsNotNone(cluster.created_at)
         self.assertIsNone(cluster.updated_at)
@@ -310,7 +280,7 @@ class TestCluster(base.SenlinTestCase):
             self.context, CLUSTER_ID,
             {
                 'created_at': mock.ANY,
-                'status': cluster.ACTIVE,
+                'status': consts.CS_ACTIVE,
                 'status_reason': 'Cluster created'
             }
         )
@@ -320,9 +290,9 @@ class TestCluster(base.SenlinTestCase):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                              id=CLUSTER_ID, status='UPDATING')
 
-        cluster.set_status(self.context, cluster.ACTIVE, 'Cluster updated')
+        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster updated')
 
-        self.assertEqual(cluster.ACTIVE, cluster.status)
+        self.assertEqual(consts.CS_ACTIVE, cluster.status)
         self.assertEqual('Cluster updated', cluster.status_reason)
         self.assertIsNotNone(cluster.updated_at)
 
@@ -331,9 +301,9 @@ class TestCluster(base.SenlinTestCase):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
                              id=CLUSTER_ID, status='RESIZING')
 
-        cluster.set_status(self.context, cluster.ACTIVE, 'Cluster resized')
+        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster resized')
 
-        self.assertEqual(cluster.ACTIVE, cluster.status)
+        self.assertEqual(consts.CS_ACTIVE, cluster.status)
         self.assertEqual('Cluster resized', cluster.status_reason)
         self.assertIsNotNone(cluster.updated_at)
 
@@ -347,10 +317,10 @@ class TestCluster(base.SenlinTestCase):
                              status='UPDATING')
 
         new_profile_id = 'a64f0b03-4b77-49d5-89e0-7bcc77c4ce67'
-        cluster.set_status(self.context, cluster.ACTIVE, 'Cluster updated',
+        cluster.set_status(self.context, consts.CS_ACTIVE, 'Cluster updated',
                            profile_id=new_profile_id)
 
-        self.assertEqual(cluster.ACTIVE, cluster.status)
+        self.assertEqual(consts.CS_ACTIVE, cluster.status)
         self.assertEqual('Cluster updated', cluster.status_reason)
         self.assertIsNotNone(cluster.updated_at)
         self.assertEqual(x_profile, cluster.rt['profile'])
@@ -360,7 +330,7 @@ class TestCluster(base.SenlinTestCase):
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
             {
-                'status': cluster.ACTIVE,
+                'status': consts.CS_ACTIVE,
                 'status_reason': 'Cluster updated',
                 'profile_id': new_profile_id,
                 'updated_at': mock.ANY,
@@ -373,23 +343,17 @@ class TestCluster(base.SenlinTestCase):
                              status='UPDATING',
                              status_reason='Update in progress')
 
-        cluster.set_status(self.context, cluster.WARNING)
+        cluster.set_status(self.context, consts.CS_WARNING)
 
-        self.assertEqual(cluster.WARNING, cluster.status)
+        self.assertEqual(consts.CS_WARNING, cluster.status)
         self.assertEqual('Update in progress', cluster.status_reason)
         mock_update.assert_called_once_with(self.context, CLUSTER_ID,
-                                            {'status': cluster.WARNING})
+                                            {'status': consts.CS_WARNING})
 
-    @mock.patch.object(co.Cluster, 'update')
-    def test_update_dependents(self, mock_update):
-        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID,
-                             id=CLUSTER_ID, status='ACTIVE')
-        dependents = {'containers': ['container1']}
-        cluster.update_dependents(self.context, dependents)
-        values = {'dependents': {'containers': ['container1']}}
-        mock_update.assert_called_once_with(self.context, CLUSTER_ID, values)
+    @mock.patch.object(pfb.Profile, "create_cluster_object")
+    def test_do_create(self, mock_create_cluster):
+        mock_create_cluster.return_value = None
 
-    def test_do_create(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
         mock_status = self.patchobject(cluster, 'set_status')
 
@@ -397,27 +361,32 @@ class TestCluster(base.SenlinTestCase):
 
         self.assertTrue(res)
         mock_status.assert_called_once_with(
-            self.context, cluster.CREATING, 'Creation in progress')
+            self.context, consts.CS_CREATING, 'Creation in progress')
 
     def test_do_create_wrong_status(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
-        cluster.status = cluster.ACTIVE
+        cluster.status = consts.CS_ACTIVE
 
         res = cluster.do_create(self.context)
 
         self.assertFalse(res)
 
+    @mock.patch.object(pfb.Profile, "delete_cluster_object")
     @mock.patch.object(co.Cluster, 'delete')
-    def test_do_delete(self, mock_delete):
+    def test_do_delete(self, mock_delete, mock_delete_cluster):
         mock_delete.return_value = None
+        mock_delete_cluster.return_value = None
 
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
         cluster.id = CLUSTER_ID
+        mock_status = self.patchobject(cluster, 'set_status')
 
         res = cluster.do_delete(self.context)
 
         mock_delete.assert_called_once_with(self.context, CLUSTER_ID)
         self.assertTrue(res)
+        mock_status.assert_called_once_with(
+            self.context, consts.CS_DELETING, 'Deletion in progress')
 
     def test_do_update(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
@@ -425,7 +394,7 @@ class TestCluster(base.SenlinTestCase):
 
         res = cluster.do_update(self.context)
 
-        mock_status.assert_called_once_with(self.context, cluster.UPDATING,
+        mock_status.assert_called_once_with(self.context, consts.CS_UPDATING,
                                             'Update in progress')
         self.assertTrue(res)
 
@@ -435,7 +404,7 @@ class TestCluster(base.SenlinTestCase):
 
         res = cluster.do_check(self.context)
 
-        mock_status.assert_called_once_with(self.context, cluster.CHECKING,
+        mock_status.assert_called_once_with(self.context, consts.CS_CHECKING,
                                             'Check in progress')
         self.assertTrue(res)
 
@@ -445,10 +414,20 @@ class TestCluster(base.SenlinTestCase):
 
         res = cluster.do_recover(self.context)
 
-        mock_status.assert_called_once_with(self.context, cluster.RECOVERING,
+        mock_status.assert_called_once_with(self.context, consts.CS_RECOVERING,
                                             'Recovery in progress')
         self.assertTrue(res)
 
+    def test_do_operation(self):
+        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
+        mock_status = self.patchobject(cluster, 'set_status')
+
+        res = cluster.do_operation(self.context, operation='dance')
+
+        mock_status.assert_called_once_with(self.context, consts.CS_OPERATING,
+                                            'Operation dance in progress')
+        self.assertTrue(res)
+
     def test_nodes_property(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
         self.assertEqual([], cluster.nodes)
@@ -522,6 +501,24 @@ class TestCluster(base.SenlinTestCase):
         self.assertIsNone(res)
         self.assertEqual([], cluster.nodes)
 
+    def test_update_node(self):
+        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
+        self.assertEqual([], cluster.nodes)
+
+        node1 = mock.Mock(id='fake', status='ACTIVE')
+        # add one
+        cluster.add_node(node1)
+
+        node1.status = 'ERROR'
+        cluster.update_node([node1])
+        self.assertEqual([node1], cluster.nodes)
+
+        # update new ones
+        node2 = mock.Mock(id='fake1', status='ACTIVE')
+        node3 = mock.Mock(id='fake2', status='ERROR')
+        cluster.update_node([node2, node3])
+        self.assertEqual([node2, node3], cluster.nodes)
+
     @mock.patch.object(pcb.Policy, 'load')
     @mock.patch.object(cpm, 'ClusterPolicy')
     def test_attach_policy(self, mock_cp, mock_load):
@@ -537,8 +534,8 @@ class TestCluster(base.SenlinTestCase):
         mock_cp.return_value = binding
 
         values = {'enabled': True}
-        res, reason = cluster.attach_policy(self.context, POLICY_ID, values)
-        policy.attach.assert_called_once_with(cluster)
+        cluster.attach_policy(self.context, POLICY_ID, values)
+        policy.attach.assert_called_once_with(cluster, enabled=True)
         mock_load.assert_called_once_with(self.context, POLICY_ID)
         mock_cp.assert_called_once_with(CLUSTER_ID, POLICY_ID, priority=10,
                                         enabled=True, data=None)
@@ -621,7 +618,7 @@ class TestCluster(base.SenlinTestCase):
         self.assertTrue(res)
         self.assertEqual('Policy attached.', reason)
 
-        policy.attach.assert_called_once_with(cluster)
+        policy.attach.assert_called_once_with(cluster, enabled=True)
         mock_load.assert_called_once_with(self.context, new_policy_id)
         mock_cp.assert_called_once_with(cluster.id, new_policy_id, priority=10,
                                         enabled=True, data=None)
@@ -642,7 +639,7 @@ class TestCluster(base.SenlinTestCase):
 
         self.assertFalse(res)
         self.assertEqual('Bad things happened.', reason)
-        policy.attach.assert_called_once_with(cluster)
+        policy.attach.assert_called_once_with(cluster, enabled=True)
         mock_load.assert_called_once_with(self.context, new_id)
 
     @mock.patch.object(cpo.ClusterPolicy, 'delete')
@@ -701,11 +698,14 @@ class TestCluster(base.SenlinTestCase):
 
         existing = mock.Mock()
         existing.id = POLICY_ID
+        existing.type = "senlin.policy.foo"
         cluster.rt['policies'] = [existing]
         values = {
             'enabled': False
         }
+
         res, reason = cluster.update_policy(self.context, POLICY_ID, **values)
+
         self.assertTrue(res)
         self.assertEqual('Policy updated.', reason)
         mock_update.assert_called_once_with(
@@ -714,10 +714,11 @@ class TestCluster(base.SenlinTestCase):
     def test_update_policy_not_attached(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
         cluster.rt['policies'] = []
+        values = {'enabled': False}
 
         # do it
-        values = {'enabled': False}
         res, reason = cluster.update_policy(self.context, POLICY_ID, **values)
+
         self.assertFalse(res)
         self.assertEqual('Policy not attached.', reason)
 
@@ -726,14 +727,46 @@ class TestCluster(base.SenlinTestCase):
         existing = mock.Mock()
         existing.id = POLICY_ID
         cluster.rt['policies'] = [existing]
+        values = {}
 
         # do it
-        values = {}
         res, reason = cluster.update_policy(self.context, POLICY_ID, **values)
 
         self.assertTrue(res)
         self.assertEqual('No update is needed.', reason)
 
+    @mock.patch.object(cpo.ClusterPolicy, "update")
+    @mock.patch.object(health_manager, "enable")
+    def test_update_policy_enable_health(self, mock_enable, mock_update):
+        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID, id=CLUSTER_ID)
+        existing = mock.Mock(id=POLICY_ID, type="senlin.policy.health")
+        cluster.rt['policies'] = [existing]
+        values = {"enabled": True}
+
+        # do it
+        res, reason = cluster.update_policy(self.context, POLICY_ID, **values)
+
+        self.assertTrue(res)
+        mock_enable.assert_called_once_with(CLUSTER_ID)
+        mock_update.assert_called_once_with(
+            self.context, CLUSTER_ID, POLICY_ID, {'enabled': True})
+
+    @mock.patch.object(cpo.ClusterPolicy, "update")
+    @mock.patch.object(health_manager, "disable")
+    def test_update_policy_disable_health(self, mock_disable, mock_update):
+        cluster = cm.Cluster('test-cluster', 0, PROFILE_ID, id=CLUSTER_ID)
+        existing = mock.Mock(id=POLICY_ID, type="senlin.policy.health")
+        cluster.rt['policies'] = [existing]
+        values = {"enabled": False}
+
+        # do it
+        res, reason = cluster.update_policy(self.context, POLICY_ID, **values)
+
+        self.assertTrue(res)
+        mock_disable.assert_called_once_with(CLUSTER_ID)
+        mock_update.assert_called_once_with(
+            self.context, CLUSTER_ID, POLICY_ID, {'enabled': False})
+
     def test_get_region_distribution(self):
         cluster = cm.Cluster('test-cluster', 0, PROFILE_ID)
 
@@ -834,6 +867,33 @@ class TestCluster(base.SenlinTestCase):
         result = cluster.nodes_by_region('AZ3')
         self.assertEqual(0, len(result))
 
+    @mock.patch.object(node_mod.Node, 'load_all')
+    @mock.patch.object(node_mod.Node, 'do_check')
+    @mock.patch.object(cm.Cluster, 'update_node')
+    def test_health_check(self, mock_update, mock_check, mock_load):
+        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID,
+                             min_size=2, id=CLUSTER_ID)
+        node1 = node_mod.Node('fake1', PROFILE_ID, status='ACTIVE')
+        node2 = node_mod.Node('fake2', PROFILE_ID, status='ACTIVE')
+        nodes = [node1, node2]
+        for node in nodes:
+            cluster.add_node(node)
+
+        node1.status = 'ERROR'
+        mock_load.return_value = [node1, node2]
+
+        cluster.health_check(self.context)
+
+        self.assertEqual(2, len(cluster.nodes))
+        self.assertEqual([node1, node2], cluster.nodes)
+
+        mock_update.assert_called_once_with([node1, node2])
+        mock_check.assert_has_calls([
+            mock.call(self.context),
+            mock.call(self.context)
+        ])
+        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
+
     @mock.patch.object(co.Cluster, 'update')
     @mock.patch.object(node_mod.Node, 'load_all')
     def test_eval_status_below_min_size(self, mock_load, mock_update):
@@ -845,11 +905,13 @@ class TestCluster(base.SenlinTestCase):
         mock_load.return_value = [node1, node2, node3]
 
         cluster.eval_status(self.context, 'TEST')
-
+        rt = cluster.rt
+        self.assertEqual(3, len(rt['nodes']))
+        self.assertIsInstance(rt['nodes'], list)
         mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
-            {'status': cluster.ERROR,
+            {'status': consts.CS_ERROR,
              'status_reason': 'TEST: number of active nodes is below '
                               'min_size (2).'})
 
@@ -868,10 +930,29 @@ class TestCluster(base.SenlinTestCase):
         mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
-            {'status': cluster.WARNING,
+            {'status': consts.CS_WARNING,
              'status_reason': 'TEST: number of active nodes is below '
                               'desired_capacity (5).'})
 
+    @mock.patch.object(co.Cluster, 'update')
+    @mock.patch.object(node_mod.Node, 'load_all')
+    def test_eval_status_equal_desired_capacity(self, mock_load, mock_update):
+        cluster = cm.Cluster('test-cluster', 3, PROFILE_ID,
+                             min_size=1, id=CLUSTER_ID)
+        node1 = mock.Mock(status='ACTIVE')
+        node2 = mock.Mock(status='ACTIVE')
+        node3 = mock.Mock(status='ACTIVE')
+        mock_load.return_value = [node1, node2, node3]
+
+        cluster.eval_status(self.context, 'TEST')
+
+        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
+        mock_update.assert_called_once_with(
+            self.context, CLUSTER_ID,
+            {'status': consts.CS_ACTIVE,
+             'status_reason': 'TEST: number of active nodes is equal or above '
+                              'desired_capacity (3).'})
+
     @mock.patch.object(co.Cluster, 'update')
     @mock.patch.object(node_mod.Node, 'load_all')
     def test_eval_status_above_desired_capacity(self, mock_load, mock_update):
@@ -887,8 +968,8 @@ class TestCluster(base.SenlinTestCase):
         mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
-            {'status': cluster.ACTIVE,
-             'status_reason': 'TEST: number of active nodes is above '
+            {'status': consts.CS_ACTIVE,
+             'status_reason': 'TEST: number of active nodes is equal or above '
                               'desired_capacity (2).'})
 
     @mock.patch.object(co.Cluster, 'update')
@@ -906,7 +987,7 @@ class TestCluster(base.SenlinTestCase):
         mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
-            {'status': cluster.WARNING,
+            {'status': consts.CS_WARNING,
              'status_reason': 'TEST: number of active nodes is above '
                               'max_size (2).'})
 
@@ -925,10 +1006,29 @@ class TestCluster(base.SenlinTestCase):
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
             {'desired_capacity': 2,
-             'status': cluster.WARNING,
+             'status': consts.CS_WARNING,
              'status_reason': 'TEST: number of active nodes is below '
                               'desired_capacity (2).'})
 
+    @mock.patch.object(co.Cluster, 'update')
+    @mock.patch.object(node_mod.Node, 'load_all')
+    def test_eval_status__new_desired_is_zero(self, mock_load, mock_update):
+        cluster = cm.Cluster('test-cluster', 5, PROFILE_ID, id=CLUSTER_ID)
+        node1 = mock.Mock(status='ACTIVE')
+        node2 = mock.Mock(status='ERROR')
+        node3 = mock.Mock(status='WARNING')
+        mock_load.return_value = [node1, node2, node3]
+
+        cluster.eval_status(self.context, 'TEST', desired_capacity=0)
+
+        mock_load.assert_called_once_with(self.context, cluster_id=CLUSTER_ID)
+        mock_update.assert_called_once_with(
+            self.context, CLUSTER_ID,
+            {'desired_capacity': 0,
+             'status': consts.CS_ACTIVE,
+             'status_reason': 'TEST: number of active nodes is equal or above '
+                              'desired_capacity (0).'})
+
     @mock.patch.object(co.Cluster, 'update')
     @mock.patch.object(node_mod.Node, 'load_all')
     def test_eval_status_with_new_min(self, mock_load, mock_update):
@@ -945,7 +1045,7 @@ class TestCluster(base.SenlinTestCase):
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
             {'min_size': 2,
-             'status': cluster.ERROR,
+             'status': consts.CS_ERROR,
              'status_reason': 'TEST: number of active nodes is below '
                               'min_size (2).'})
 
@@ -965,6 +1065,6 @@ class TestCluster(base.SenlinTestCase):
         mock_update.assert_called_once_with(
             self.context, CLUSTER_ID,
             {'max_size': 6,
-             'status': cluster.ACTIVE,
-             'status_reason': 'TEST: number of active nodes is above '
+             'status': consts.CS_ACTIVE,
+             'status_reason': 'TEST: number of active nodes is equal or above '
                               'desired_capacity (2).'})
