@@ -11,6 +11,7 @@
 # under the License.
 
 import mock
+from oslo_config import cfg
 from oslo_context import context
 import oslo_messaging
 
@@ -106,57 +107,53 @@ class TestDispatcher(base.SenlinTestCase):
     @mock.patch.object(context, 'get_current')
     @mock.patch.object(messaging, 'get_rpc_client')
     def test_notify_broadcast(self, mock_rpc, mock_get_current):
+        cfg.CONF.set_override('host', 'HOSTNAME')
         fake_ctx = mock.Mock()
         mock_get_current.return_value = fake_ctx
         mock_rpc.return_value = mock.Mock()
+
         dispatcher.notify('METHOD')
 
-        mock_rpc.assert_called_once_with(version=consts.RPC_API_VERSION)
+        mock_rpc.assert_called_once_with(consts.DISPATCHER_TOPIC, 'HOSTNAME')
         mock_client = mock_rpc.return_value
-        mock_client.prepare.assert_called_once_with(
-            version=consts.RPC_API_VERSION,
-            topic=consts.ENGINE_DISPATCHER_TOPIC)
+        mock_client.prepare.assert_called_once_with(fanout=True)
 
         mock_context = mock_client.prepare.return_value
-        mock_context.call.assert_called_once_with(fake_ctx, 'METHOD')
+        mock_context.cast.assert_called_once_with(fake_ctx, 'METHOD')
 
     @mock.patch.object(context, 'get_current')
     @mock.patch.object(messaging, 'get_rpc_client')
     def test_notify_single_server(self, mock_rpc, mock_get_current):
+        cfg.CONF.set_override('host', 'HOSTNAME')
         fake_ctx = mock.Mock()
         mock_get_current.return_value = fake_ctx
         mock_rpc.return_value = mock.Mock()
+
         result = dispatcher.notify('METHOD', 'FAKE_ENGINE')
 
         self.assertTrue(result)
-
-        mock_rpc.assert_called_once_with(version=consts.RPC_API_VERSION)
+        mock_rpc.assert_called_once_with(consts.DISPATCHER_TOPIC, 'HOSTNAME')
         mock_client = mock_rpc.return_value
-        mock_client.prepare.assert_called_once_with(
-            version=consts.RPC_API_VERSION,
-            topic=consts.ENGINE_DISPATCHER_TOPIC,
-            server='FAKE_ENGINE')
+        mock_client.prepare.assert_called_once_with(server='FAKE_ENGINE')
 
         mock_context = mock_client.prepare.return_value
-        mock_context.call.assert_called_once_with(fake_ctx, 'METHOD')
+        mock_context.cast.assert_called_once_with(fake_ctx, 'METHOD')
 
     @mock.patch.object(messaging, 'get_rpc_client')
     def test_notify_timeout(self, mock_rpc):
+        cfg.CONF.set_override('host', 'HOSTNAME')
         mock_rpc.return_value = mock.Mock()
         mock_client = mock_rpc.return_value
         mock_context = mock_client.prepare.return_value
-        mock_context.call.side_effect = oslo_messaging.MessagingTimeout
+        mock_context.cast.side_effect = oslo_messaging.MessagingTimeout
 
         result = dispatcher.notify('METHOD')
 
         self.assertFalse(result)
+        mock_rpc.assert_called_once_with(consts.DISPATCHER_TOPIC, 'HOSTNAME')
+        mock_client.prepare.assert_called_once_with(fanout=True)
 
-        mock_rpc.assert_called_once_with(version=consts.RPC_API_VERSION)
-        mock_client.prepare.assert_called_once_with(
-            version=consts.RPC_API_VERSION,
-            topic=consts.ENGINE_DISPATCHER_TOPIC)
-
-        mock_context.call.assert_called_once_with(mock.ANY, 'METHOD')
+        mock_context.cast.assert_called_once_with(mock.ANY, 'METHOD')
 
     @mock.patch.object(dispatcher, 'notify')
     def test_start_action_function(self, mock_notify):
