@@ -11,6 +11,7 @@
 # under the License.
 
 import datetime
+import eventlet
 import mock
 
 from oslo_config import cfg
@@ -74,9 +75,10 @@ class EngineBasicTest(base.SenlinTestCase):
         mock_msg_cls.assert_called_once_with(version=consts.RPC_API_VERSION,
                                              server=self.eng.host,
                                              topic=self.eng.topic)
-        self.assertEqual(mock_target, self.eng.target)
 
-        self.get_rpc.assert_called_once_with(mock_target, self.eng)
+        self.get_rpc.assert_called_once_with(mock_target, self.eng,
+                                             serializer=mock.ANY)
+
         self.assertEqual(self.fake_rpc_server, self.eng._rpc_server)
         self.fake_rpc_server.start.assert_called_once_with()
 
@@ -95,7 +97,7 @@ class EngineBasicTest(base.SenlinTestCase):
         mock_disp.stop.assert_called_once_with()
         mock_hm.stop.assert_called_once_with()
 
-        mock_delete.assert_called_once_with(mock.ANY, self.fake_id)
+        mock_delete.assert_called_once_with(self.fake_id)
 
     def test_engine_stop_with_exception(self, mock_msg_cls, mock_hm_cls,
                                         mock_disp_cls):
@@ -115,6 +117,7 @@ class EngineBasicTest(base.SenlinTestCase):
 
 
 class EngineStatusTest(base.SenlinTestCase):
+
     def setUp(self):
         super(EngineStatusTest, self).setUp()
         self.eng = service.EngineService('host-a', 'topic-a')
@@ -126,16 +129,36 @@ class EngineStatusTest(base.SenlinTestCase):
         self.get_rpc = self.patchobject(rpc_messaging, 'get_rpc_server',
                                         return_value=self.fake_rpc_server)
 
-    @mock.patch.object(service_obj.Service, 'create')
+    @mock.patch('senlin.engine.dispatcher.Dispatcher')
+    @mock.patch('senlin.engine.health_manager.HealthManager')
+    @mock.patch('oslo_messaging.Target')
     @mock.patch.object(service_obj.Service, 'update')
-    def test_service_manage_report_create(self, mock_update, mock_create):
-        mock_update.return_value = None
+    def test_engine_manage_report(self, mock_update, mock_msg_cls, mock_hm_cls,
+                                  mock_disp_cls):
+        cfg.CONF.set_override('periodic_interval', 1)
 
-        self.eng.service_manage_report()
+        # start engine and verify that update is being called more than once
+        self.eng.start()
+        eventlet.sleep(6)
+        self.assertGreater(mock_update.call_count, 1)
+        self.eng.stop()
 
-        mock_create.assert_called_once_with(
-            mock.ANY, self.eng.engine_id, self.eng.host, 'senlin-engine',
-            self.eng.topic)
+    @mock.patch('senlin.engine.dispatcher.Dispatcher')
+    @mock.patch('senlin.engine.health_manager.HealthManager')
+    @mock.patch('oslo_messaging.Target')
+    @mock.patch.object(service_obj.Service, 'update')
+    def test_engine_manage_report_with_exception(self, mock_update,
+                                                 mock_msg_cls, mock_hm_cls,
+                                                 mock_disp_cls):
+        cfg.CONF.set_override('periodic_interval', 1)
+
+        # start engine and verify that update is being called more than once
+        # even with the exception being thrown
+        mock_update.side_effect = Exception('blah')
+        self.eng.start()
+        eventlet.sleep(6)
+        self.assertGreater(mock_update.call_count, 1)
+        self.eng.stop()
 
     @mock.patch.object(service_obj.Service, 'update')
     def test_service_manage_report_update(self, mock_update):
@@ -143,19 +166,44 @@ class EngineStatusTest(base.SenlinTestCase):
         self.eng.service_manage_report()
         mock_update.assert_called_once_with(mock.ANY, self.eng.engine_id)
 
-    @mock.patch.object(service_obj.Service, 'update')
-    def test_service_manage_report_error(self, mock_update):
-        mock_update.side_effect = [Exception]
-        self.eng.service_manage_report()
-        mock_update.assert_called_once_with(mock.ANY, self.eng.engine_id)
-        expect_str = 'Service %s update failed' % self.eng.engine_id
-        self.assertIn(expect_str, self.LOG.output)
-
+    @mock.patch.object(service_obj.Service, 'gc_by_engine')
     @mock.patch.object(service_obj.Service, 'get_all')
     @mock.patch.object(service_obj.Service, 'delete')
-    def test__service_manage_cleanup(self, mock_delete, mock_get_all):
+    def test__service_manage_cleanup(self, mock_delete, mock_get_all, mock_gc):
         delta = datetime.timedelta(seconds=2 * cfg.CONF.periodic_interval)
         ages_a_go = timeutils.utcnow(True) - delta
         mock_get_all.return_value = [{'id': 'foo', 'updated_at': ages_a_go}]
         self.eng._service_manage_cleanup()
-        mock_delete.assert_called_once_with(mock.ANY, 'foo')
+        mock_delete.assert_called_once_with('foo')
+        mock_gc.assert_called_once_with('foo')
+
+    @mock.patch('senlin.engine.dispatcher.Dispatcher')
+    @mock.patch('senlin.engine.health_manager.HealthManager')
+    @mock.patch('oslo_messaging.Target')
+    @mock.patch.object(service_obj.Service, 'get_all')
+    def test_service_manage_cleanup(self, mock_get_all, mock_msg_cls,
+                                    mock_hm_cls, mock_disp_cls):
+        cfg.CONF.set_override('periodic_interval', 1)
+
+        # start engine and verify that get_all is being called more than once
+        self.eng.start()
+        eventlet.sleep(6)
+        self.assertGreater(mock_get_all.call_count, 1)
+        self.eng.stop()
+
+    @mock.patch('senlin.engine.dispatcher.Dispatcher')
+    @mock.patch('senlin.engine.health_manager.HealthManager')
+    @mock.patch('oslo_messaging.Target')
+    @mock.patch.object(service_obj.Service, 'get_all')
+    def test_service_manage_cleanup_with_exception(self, mock_get_all,
+                                                   mock_msg_cls, mock_hm_cls,
+                                                   mock_disp_cls):
+        cfg.CONF.set_override('periodic_interval', 1)
+
+        # start engine and verify that get_all is being called more than once
+        # even with the exception being thrown
+        mock_get_all.side_effect = Exception('blah')
+        self.eng.start()
+        eventlet.sleep(6)
+        self.assertGreater(mock_get_all.call_count, 1)
+        self.eng.stop()
