@@ -38,9 +38,11 @@ class TestEnvironment(base.SenlinTestCase):
         self.assertEqual('profiles', e.profile_registry.registry_name)
         self.assertEqual('policies', e.policy_registry.registry_name)
         self.assertEqual('drivers', e.driver_registry.registry_name)
+        self.assertEqual('endpoints', e.endpoint_registry.registry_name)
         self.assertTrue(e.profile_registry.is_global)
         self.assertTrue(e.policy_registry.is_global)
         self.assertTrue(e.driver_registry.is_global)
+        self.assertTrue(e.endpoint_registry.is_global)
 
     def test_create_default(self):
         ge = environment.global_env()
@@ -49,20 +51,25 @@ class TestEnvironment(base.SenlinTestCase):
         reg_prof = e.profile_registry
         reg_plcy = e.policy_registry
         reg_driv = e.driver_registry
+        reg_endp = e.endpoint_registry
 
         self.assertEqual({}, e.params)
         self.assertEqual('profiles', reg_prof.registry_name)
         self.assertEqual('policies', reg_plcy.registry_name)
         self.assertEqual('drivers', reg_driv.registry_name)
+        self.assertEqual('endpoints', reg_endp.registry_name)
         self.assertFalse(reg_prof.is_global)
         self.assertFalse(reg_plcy.is_global)
         self.assertFalse(reg_driv.is_global)
+        self.assertFalse(reg_endp.is_global)
         self.assertEqual('profiles', ge.profile_registry.registry_name)
         self.assertEqual('policies', ge.policy_registry.registry_name)
         self.assertEqual('drivers', ge.driver_registry.registry_name)
+        self.assertEqual('endpoints', ge.endpoint_registry.registry_name)
         self.assertEqual(ge.profile_registry, reg_prof.global_registry)
         self.assertEqual(ge.policy_registry, reg_plcy.global_registry)
         self.assertEqual(ge.driver_registry, reg_driv.global_registry)
+        self.assertEqual(ge.endpoint_registry, reg_endp.global_registry)
 
     def test_create_with_env(self):
         env = {
@@ -143,7 +150,7 @@ class TestEnvironment(base.SenlinTestCase):
     def test_check_plugin_name(self):
         env = environment.Environment()
 
-        for pt in ['Profile', 'Policy', 'Driver']:
+        for pt in ['Profile', 'Policy', 'Driver', 'Endpoint']:
             res = env._check_plugin_name(pt, 'abc')
             self.assertIsNone(res)
 
@@ -169,7 +176,7 @@ class TestEnvironment(base.SenlinTestCase):
 
         ex = self.assertRaises(exception.ResourceNotFound,
                                env.get_profile, 'foo')
-        self.assertEqual('The profile_type (foo) could not be found.',
+        self.assertEqual("The profile_type 'foo' could not be found.",
                          six.text_type(ex))
 
         env.register_profile('foo', plugin)
@@ -177,14 +184,18 @@ class TestEnvironment(base.SenlinTestCase):
 
     def test_get_profile_types(self):
         env = environment.Environment()
-        plugin1 = mock.Mock()
-        env.register_profile('foo', plugin1)
-        plugin2 = mock.Mock()
-        env.register_profile('bar', plugin2)
+        plugin1 = mock.Mock(VERSIONS={'1.0': 'v'})
+        env.register_profile('foo-1.0', plugin1)
+        plugin2 = mock.Mock(VERSIONS={'1.2': 'v1'})
+        env.register_profile('bar-1.2', plugin2)
 
         actual = env.get_profile_types()
-        self.assertIn({'name': 'foo'}, actual)
-        self.assertIn({'name': 'bar'}, actual)
+        self.assertIn(
+            {'name': 'foo', 'version': '1.0', 'support_status': {'1.0': 'v'}},
+            actual)
+        self.assertIn(
+            {'name': 'bar', 'version': '1.2', 'support_status': {'1.2': 'v1'}},
+            actual)
 
     def test_register_and_get_policy(self):
         plugin = mock.Mock()
@@ -192,7 +203,7 @@ class TestEnvironment(base.SenlinTestCase):
 
         ex = self.assertRaises(exception.ResourceNotFound,
                                env.get_policy, 'foo')
-        self.assertEqual('The policy_type (foo) could not be found.',
+        self.assertEqual("The policy_type 'foo' could not be found.",
                          six.text_type(ex))
 
         env.register_policy('foo', plugin)
@@ -200,14 +211,18 @@ class TestEnvironment(base.SenlinTestCase):
 
     def test_get_policy_types(self):
         env = environment.Environment()
-        plugin1 = mock.Mock()
-        env.register_policy('foo', plugin1)
-        plugin2 = mock.Mock()
-        env.register_policy('bar', plugin2)
+        plugin1 = mock.Mock(VERSIONS={'0.1': 'v'})
+        env.register_policy('foo-0.1', plugin1)
+        plugin2 = mock.Mock(VERSIONS={'0.1': 'v1'})
+        env.register_policy('bar-0.1', plugin2)
 
         actual = env.get_policy_types()
-        self.assertIn({'name': 'foo'}, actual)
-        self.assertIn({'name': 'bar'}, actual)
+        self.assertIn(
+            {'name': 'foo', 'version': '0.1', 'support_status': {'0.1': 'v'}},
+            actual)
+        self.assertIn(
+            {'name': 'bar', 'version': '0.1', 'support_status': {'0.1': 'v1'}},
+            actual)
 
     def test_register_and_get_driver_types(self):
         plugin = mock.Mock()
@@ -223,14 +238,30 @@ class TestEnvironment(base.SenlinTestCase):
 
     def test_get_driver_types(self):
         env = environment.Environment()
-        plugin1 = mock.Mock()
+        plugin1 = mock.Mock(VERSIONS={})
         env.register_driver('foo', plugin1)
-        plugin2 = mock.Mock()
+        plugin2 = mock.Mock(VERSIONS={})
         env.register_driver('bar', plugin2)
 
         actual = env.get_driver_types()
-        self.assertIn({'name': 'foo'}, actual)
-        self.assertIn({'name': 'bar'}, actual)
+        self.assertIn(
+            {'name': 'foo', 'version': '', 'support_status': {'': ''}},
+            actual)
+        self.assertIn(
+            {'name': 'bar', 'version': '', 'support_status': {'': ''}},
+            actual)
+
+    def test_register_and_get_endpoints(self):
+        plugin = mock.Mock()
+        env = environment.Environment()
+
+        ex = self.assertRaises(exception.InvalidPlugin,
+                               env.get_endpoint, 'foo')
+        self.assertEqual('Endpoint plugin foo is not found.',
+                         six.text_type(ex))
+
+        env.register_endpoint('foo', plugin)
+        self.assertEqual(plugin, env.get_endpoint('foo'))
 
     def test_read_global_environment(self):
         mock_dir = self.patchobject(glob, 'glob')
@@ -243,8 +274,8 @@ class TestEnvironment(base.SenlinTestCase):
                         create=True) as mock_open:
             env.read_global_environment()
 
-        mock_dir.assert_called_once_with(env_dir + '/*')
-        mock_open.assert_called_once_with('%s/e.yaml' % env_dir)
+        mock_dir.assert_called_with(env_dir + '/*')
+        mock_open.assert_called_with('%s/e.yaml' % env_dir)
 
     def test_empty_environment_dir(self):
         mock_dir = self.patchobject(glob, 'glob', return_value=[])
@@ -304,12 +335,13 @@ class TestEnvironment(base.SenlinTestCase):
 
         expected = [mock.call('senlin.profiles'),
                     mock.call('senlin.policies'),
-                    mock.call('senlin.drivers')]
+                    mock.call('senlin.drivers'),
+                    mock.call('senlin.endpoints')]
 
         self.assertIsNotNone(environment._environment)
         self.assertEqual(expected, mock_mapping.call_args_list)
         self.assertIsNotNone(environment.global_env().get_profile('aaa'))
         self.assertIsNotNone(environment.global_env().get_policy('aaa'))
         self.assertIsNotNone(environment.global_env().get_driver('aaa'))
-
+        self.assertIsNotNone(environment.global_env().get_endpoint('aaa'))
         environment._environment = None
