@@ -10,12 +10,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
+
 import mock
+import six
+
+from oslo_config import cfg
+
 from senlin.common import consts
+from senlin.common import exception as exc
 from senlin.common import scaleutils as su
 from senlin.engine import health_manager
-from senlin.objects import cluster as co
-from senlin.objects import node as no
 from senlin.policies import base as pb
 from senlin.policies import health_policy
 from senlin.tests.unit.common import base
@@ -40,16 +45,19 @@ class TestHealthPolicy(base.SenlinTestCase):
                 },
                 'recovery': {
                     'fencing': ['COMPUTE'],
-                    'actions': ['REBUILD']
+                    'actions': [
+                        {'name': 'REBUILD'}
+                    ]
                 }
             }
         }
-        cluster = mock.Mock()
-        cluster.id = 'CLUSTER_ID'
-        node = mock.Mock()
-        node.status = 'ACTIVE'
-        cluster.nodes = [node]
-        self.cluster = cluster
+
+        fake_profile = mock.Mock(type_name='os.nova.server',
+                                 type='os.nova.server-1.0',)
+        fake_node = mock.Mock(status='ACTIVE')
+        fake_cluster = mock.Mock(id='CLUSTER_ID', nodes=[fake_node],
+                                 rt={'profile': fake_profile})
+        self.cluster = fake_cluster
         self.patch('senlin.rpc.client.EngineClient')
         self.hp = health_policy.HealthPolicy('test-policy', self.spec)
 
@@ -59,7 +67,48 @@ class TestHealthPolicy(base.SenlinTestCase):
         self.assertEqual('senlin.policy.health-1.0', self.hp.type)
         self.assertEqual('NODE_STATUS_POLLING', self.hp.check_type)
         self.assertEqual(60, self.hp.interval)
-        self.assertEqual(['REBUILD'], self.hp.recover_actions)
+        self.assertEqual([{'name': 'REBUILD', 'params': None}],
+                         self.hp.recover_actions)
+
+    def test_validate(self):
+        spec = copy.deepcopy(self.spec)
+        spec["properties"]["recovery"]["actions"] = [
+            {"name": "REBUILD"}, {"name": "RECREATE"}
+        ]
+        self.hp = health_policy.HealthPolicy('test-policy', spec)
+
+        ex = self.assertRaises(exc.ESchema,
+                               self.hp.validate,
+                               self.context)
+
+        self.assertEqual("Only one 'actions' is supported for now.",
+                         six.text_type(ex))
+
+    def test_validate_valid_interval(self):
+        spec = copy.deepcopy(self.spec)
+        spec["properties"]["detection"]["options"]["interval"] = 20
+        self.hp = health_policy.HealthPolicy('test-policy', spec)
+
+        cfg.CONF.set_override('health_check_interval_min', 20)
+
+        self.hp.validate(self.context)
+
+    def test_validate_invalid_interval(self):
+        spec = copy.deepcopy(self.spec)
+        spec["properties"]["detection"]["options"]["interval"] = 10
+        self.hp = health_policy.HealthPolicy('test-policy', spec)
+
+        cfg.CONF.set_override('health_check_interval_min', 20)
+
+        ex = self.assertRaises(exc.InvalidSpec,
+                               self.hp.validate,
+                               self.context)
+
+        expected_error = ("Specified interval of %(interval)d seconds has to "
+                          "be larger than health_check_interval_min of "
+                          "%(min_interval)d seconds set in configuration."
+                          ) % {"interval": 10, "min_interval": 20}
+        self.assertEqual(expected_error, six.text_type(ex))
 
     @mock.patch.object(health_manager, 'register')
     def test_attach(self, mock_hm_reg):
@@ -68,7 +117,16 @@ class TestHealthPolicy(base.SenlinTestCase):
             'HealthPolicy': {
                 'data': {
                     'check_type': self.hp.check_type,
-                    'interval': self.hp.interval},
+                    'interval': self.hp.interval,
+                    'poll_url': '',
+                    'poll_url_ssl_verify': True,
+                    'poll_url_healthy_response': '',
+                    'poll_url_retry_limit': 3,
+                    'poll_url_retry_interval': 3,
+                    'node_update_timeout': 300,
+                    'node_delete_timeout': 20,
+                    'node_force_recreate': False
+                },
                 'version': '1.0'
             }
         }
@@ -79,12 +137,52 @@ class TestHealthPolicy(base.SenlinTestCase):
         kwargs = {
             'check_type': self.hp.check_type,
             'interval': self.hp.interval,
-            'params': {},
+            'params': {
+                'recover_action': self.hp.recover_actions,
+                'poll_url': '',
+                'poll_url_ssl_verify': True,
+                'poll_url_healthy_response': '',
+                'poll_url_retry_limit': 3,
+                'poll_url_retry_interval': 3,
+                'node_update_timeout': 300,
+                'node_delete_timeout': 20,
+                'node_force_recreate': False
+            },
+            'enabled': True
         }
         mock_hm_reg.assert_called_once_with('CLUSTER_ID',
                                             engine_id=None,
                                             **kwargs)
 
+    @mock.patch.object(health_manager, 'register')
+    def test_attach_failed_action_matching_rebuild(self, mock_hm_reg):
+
+        fake_profile = mock.Mock(type_name='os.heat.stack-1.0',
+                                 type='os.heat.stack')
+        fake_cluster = mock.Mock(id='CLUSTER_ID', rt={'profile': fake_profile})
+
+        res, data = self.hp.attach(fake_cluster)
+
+        self.assertFalse(res)
+        self.assertEqual("Recovery action REBUILD is only applicable to "
+                         "os.nova.server clusters.", data)
+
+    @mock.patch.object(health_manager, 'register')
+    def test_attach_failed_action_matching_reboot(self, mock_hm_reg):
+        spec = copy.deepcopy(self.spec)
+        spec['properties']['recovery']['actions'] = [{'name': 'REBOOT'}]
+        hp = health_policy.HealthPolicy('test-policy-1', spec)
+
+        fake_profile = mock.Mock(type_name='os.heat.stack-1.0',
+                                 type='os.heat.stack')
+        fake_cluster = mock.Mock(id='CLUSTER_ID', rt={'profile': fake_profile})
+
+        res, data = hp.attach(fake_cluster)
+
+        self.assertFalse(res)
+        self.assertEqual("Recovery action REBOOT is only applicable to "
+                         "os.nova.server clusters.", data)
+
     @mock.patch.object(health_manager, 'unregister')
     def test_detach(self, mock_hm_reg):
         res, data = self.hp.detach(self.cluster)
@@ -101,7 +199,7 @@ class TestHealthPolicy(base.SenlinTestCase):
         self.assertTrue(res)
         data = {
             'health': {
-                'recover_action': ['REBUILD'],
+                'recover_action': [{'name': 'REBUILD', 'params': None}],
                 'fencing': ['COMPUTE'],
             }
         }
@@ -148,41 +246,33 @@ class TestHealthPolicy(base.SenlinTestCase):
         mock_disable.assert_called_once_with(self.cluster.id)
 
     @mock.patch.object(su, 'parse_resize_params')
-    @mock.patch.object(no.Node, 'count_by_cluster')
-    @mock.patch.object(co.Cluster, 'get')
     @mock.patch.object(health_manager, 'disable')
-    def test_pre_op_resize_without_data(self, mock_disable, mock_get,
-                                        mock_count, mock_parse):
+    def test_pre_op_resize_without_data(self, mock_disable, mock_parse):
         def fake_check(action, cluster, current):
             action.data['deletion'] = {'foo': 'bar'}
             return pb.CHECK_OK, 'good'
 
         x_cluster = mock.Mock()
-        mock_get.return_value = x_cluster
-        mock_count.return_value = 10
+        x_cluster.nodes = [mock.Mock(), mock.Mock(), mock.Mock()]
         action = mock.Mock(context='action_context', data={},
                            action=consts.CLUSTER_RESIZE)
+        action.entity = x_cluster
         mock_parse.side_effect = fake_check
 
         res = self.hp.pre_op(self.cluster.id, action)
 
         self.assertTrue(res)
         mock_disable.assert_called_once_with(self.cluster.id)
-        mock_get.assert_called_once_with(action.context, self.cluster.id)
-        mock_count.assert_called_once_with(action.context, self.cluster.id)
-        mock_parse.assert_called_once_with(action, x_cluster, 10)
+        mock_parse.assert_called_once_with(action, x_cluster, 3)
 
     @mock.patch.object(su, 'parse_resize_params')
-    @mock.patch.object(no.Node, 'count_by_cluster')
-    @mock.patch.object(co.Cluster, 'get')
     @mock.patch.object(health_manager, 'disable')
-    def test_pre_op_resize_parse_error(self, mock_disable, mock_get,
-                                       mock_count, mock_parse):
+    def test_pre_op_resize_parse_error(self, mock_disable, mock_parse):
         x_cluster = mock.Mock()
-        mock_get.return_value = x_cluster
-        mock_count.return_value = 10
+        x_cluster.nodes = [mock.Mock(), mock.Mock()]
         action = mock.Mock(context='action_context', data={},
                            action=consts.CLUSTER_RESIZE)
+        action.entity = x_cluster
         mock_parse.return_value = pb.CHECK_ERROR, 'no good'
 
         res = self.hp.pre_op(self.cluster.id, action)
@@ -190,9 +280,7 @@ class TestHealthPolicy(base.SenlinTestCase):
         self.assertFalse(res)
         self.assertEqual(pb.CHECK_ERROR, action.data['status'])
         self.assertEqual('no good', action.data['reason'])
-        mock_get.assert_called_once_with(action.context, self.cluster.id)
-        mock_count.assert_called_once_with(action.context, self.cluster.id)
-        mock_parse.assert_called_once_with(action, x_cluster, 10)
+        mock_parse.assert_called_once_with(action, x_cluster, 2)
         self.assertEqual(0, mock_disable.call_count)
 
     def test_post_op_default(self):
@@ -230,41 +318,33 @@ class TestHealthPolicy(base.SenlinTestCase):
         mock_enable.assert_called_once_with(self.cluster.id)
 
     @mock.patch.object(su, 'parse_resize_params')
-    @mock.patch.object(no.Node, 'count_by_cluster')
-    @mock.patch.object(co.Cluster, 'get')
     @mock.patch.object(health_manager, 'enable')
-    def test_post_op_resize_without_data(self, mock_enable, mock_get,
-                                         mock_count, mock_parse):
+    def test_post_op_resize_without_data(self, mock_enable, mock_parse):
         def fake_check(action, cluster, current):
             action.data['deletion'] = {'foo': 'bar'}
             return pb.CHECK_OK, 'good'
 
         x_cluster = mock.Mock()
-        mock_get.return_value = x_cluster
-        mock_count.return_value = 10
+        x_cluster.nodes = [mock.Mock(), mock.Mock()]
         action = mock.Mock(context='action_context', data={},
                            action=consts.CLUSTER_RESIZE)
+        action.entity = x_cluster
         mock_parse.side_effect = fake_check
 
         res = self.hp.post_op(self.cluster.id, action)
 
         self.assertTrue(res)
         mock_enable.assert_called_once_with(self.cluster.id)
-        mock_get.assert_called_once_with(action.context, self.cluster.id)
-        mock_count.assert_called_once_with(action.context, self.cluster.id)
-        mock_parse.assert_called_once_with(action, x_cluster, 10)
+        mock_parse.assert_called_once_with(action, x_cluster, 2)
 
     @mock.patch.object(su, 'parse_resize_params')
-    @mock.patch.object(no.Node, 'count_by_cluster')
-    @mock.patch.object(co.Cluster, 'get')
     @mock.patch.object(health_manager, 'enable')
-    def test_post_op_resize_parse_error(self, mock_enable, mock_get,
-                                        mock_count, mock_parse):
+    def test_post_op_resize_parse_error(self, mock_enable, mock_parse):
         x_cluster = mock.Mock()
-        mock_get.return_value = x_cluster
-        mock_count.return_value = 10
+        x_cluster.nodes = [mock.Mock()]
         action = mock.Mock(context='action_context', data={},
                            action=consts.CLUSTER_RESIZE)
+        action.entity = x_cluster
         mock_parse.return_value = pb.CHECK_ERROR, 'no good'
 
         res = self.hp.post_op(self.cluster.id, action)
@@ -273,7 +353,5 @@ class TestHealthPolicy(base.SenlinTestCase):
         self.assertEqual(pb.CHECK_ERROR, action.data['status'])
         self.assertEqual('no good', action.data['reason'])
 
-        mock_get.assert_called_once_with(action.context, self.cluster.id)
-        mock_count.assert_called_once_with(action.context, self.cluster.id)
-        mock_parse.assert_called_once_with(action, x_cluster, 10)
+        mock_parse.assert_called_once_with(action, x_cluster, 1)
         self.assertEqual(0, mock_enable.call_count)
