@@ -11,16 +11,20 @@
 # under the License.
 
 import mock
-from oslo_utils import timeutils
 import six
 
 from senlin.common import consts
-from senlin.db.sqlalchemy import api as db_api
-from senlin.policies import base as policy_base
+from senlin.common import exception as exc
+from senlin.objects import node as no
+from senlin.policies import base as pb
 from senlin.policies import scaling_policy as sp
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
+PROFILE_ID = 'aa5f86b8-e52b-4f2b-828a-4c14c770938d'
+CLUSTER_ID = '2c5139a6-24ba-4a6f-bd53-a268f61536de'
+CLUSTER_NOMAXSIZE_ID = 'e470c11d-910d-491b-a7c3-93b047a6108d'
+
 
 class TestScalingPolicy(base.SenlinTestCase):
 
@@ -41,59 +45,29 @@ class TestScalingPolicy(base.SenlinTestCase):
                 }
             }
         }
-        self.profile = self._create_profile('PROFILE1')
-        self.cluster = self._create_cluster('CLUSTER1',
-                                            self.profile['id'])
-        self.nodes = self._create_nodes(self.cluster['id'],
-                                        self.profile['id'], 3)
+        self.profile = utils.create_profile(self.context, PROFILE_ID)
+        self.cluster = utils.create_cluster(self.context, CLUSTER_ID,
+                                            PROFILE_ID)
+        self.cluster_no_maxsize = utils.create_cluster(
+            self.context, CLUSTER_NOMAXSIZE_ID, PROFILE_ID, max_size=-1)
 
-    def _create_profile(self, profile_id):
-        values = {
-            'id': profile_id,
-            'type': 'os.heat.stack',
-            'name': 'test-profile',
-            'created_at': timeutils.utcnow(),
-            'user': self.context.user,
-            'project': self.context.project,
-        }
-        return db_api.profile_create(self.context, values)
+    def _create_nodes(self, count):
+        NODE_IDS = [
+            '6eaa45fa-bd2e-426d-ae49-f75db1a4bd73',
+            '8bf73953-b57b-4e6b-bdef-83fa9420befb',
+            'c3058ea0-5241-466b-89bc-6a85f6050a11',
+        ]
+        PHYSICAL_IDS = [
+            '2417c5d6-9a89-4637-9ba6-82c00b180cb7',
+            '374bf2b9-30ba-4a9b-822b-1196f6d4a368',
+            '2a1b7e37-de18-4b22-9489-a7a413fdfe48',
+        ]
 
-    def _create_cluster(self, cluster_id, profile_id):
-        values = {
-            'id': cluster_id,
-            'profile_id': profile_id,
-            'name': 'test-cluster',
-            'user': self.context.user,
-            'project': self.context.project,
-            'next_index': 1,
-            'min_size': 1,
-            'max_size': 5,
-            'desired_capacity': 3,
-        }
-
-        return db_api.cluster_create(self.context, values)
-
-    def _create_nodes(self, cluster_id, profile_id, count):
         nodes = []
         for i in range(count):
-            values = {
-                'id': 'FAKE_NODE_%s_%s' % (profile_id, (i + 1)),
-                'name': 'test_node_%s' % (i + 1),
-                'physical_id': 'FAKE_PHY_ID_%s' % (i + 1),
-                'cluster_id': cluster_id,
-                'profile_id': profile_id,
-                'project': self.context.project,
-                'index': i + 1,
-                'role': None,
-                'created_at': timeutils.utcnow(),
-                'updated_at': None,
-                'status': 'ACTIVE',
-                'status_reason': 'create complete',
-                'metadata': {'foo': '123'},
-                'data': {'key1': 'value1'},
-            }
-            db_node = db_api.node_create(self.context, values)
-            nodes.append(six.text_type(db_node.id))
+            node = utils.create_node(self.context, NODE_IDS[i], PROFILE_ID,
+                                     CLUSTER_ID, PHYSICAL_IDS[i])
+            nodes.append(node)
         return nodes
 
     def test_policy_init(self):
@@ -124,6 +98,39 @@ class TestScalingPolicy(base.SenlinTestCase):
         self.assertFalse(policy.best_effort)
         self.assertEqual(0, policy.cooldown)
 
+    def test_validate(self):
+        self.spec['properties']['adjustment'] = {}
+        policy = sp.ScalingPolicy('p1', self.spec)
+
+        policy.validate(self.context)
+
+    def test_validate_bad_number(self):
+        self.spec['properties']['adjustment'] = {"number": -1}
+        policy = sp.ScalingPolicy('p1', self.spec)
+
+        ex = self.assertRaises(exc.InvalidSpec, policy.validate, self.context)
+
+        self.assertEqual("the 'number' for 'adjustment' must be > 0",
+                         six.text_type(ex))
+
+    def test_validate_bad_min_step(self):
+        self.spec['properties']['adjustment'] = {"min_step": -1}
+        policy = sp.ScalingPolicy('p1', self.spec)
+
+        ex = self.assertRaises(exc.InvalidSpec, policy.validate, self.context)
+
+        self.assertEqual("the 'min_step' for 'adjustment' must be >= 0",
+                         six.text_type(ex))
+
+    def test_validate_bad_cooldown(self):
+        self.spec['properties']['adjustment'] = {"cooldown": -1}
+        policy = sp.ScalingPolicy('p1', self.spec)
+
+        ex = self.assertRaises(exc.InvalidSpec, policy.validate, self.context)
+
+        self.assertEqual("the 'cooldown' for 'adjustment' must be >= 0",
+                         six.text_type(ex))
+
     def test_calculate_adjustment_count(self):
         adjustment = self.spec['properties']['adjustment']
         # adjustment_type as EXACT_CAPACITY and event as cluster_scale_in
@@ -168,10 +175,15 @@ class TestScalingPolicy(base.SenlinTestCase):
         self.assertEqual(2, count)
 
     def test_pre_op_pass_without_input(self):
+        nodes = self._create_nodes(3)
+        self.cluster.nodes = nodes
+
         action = mock.Mock()
         action.context = self.context
         action.action = consts.CLUSTER_SCALE_IN
         action.inputs = {}
+        action.entity = self.cluster
+
         adjustment = self.spec['properties']['adjustment']
         adjustment['type'] = consts.EXACT_CAPACITY
         adjustment['number'] = 1
@@ -183,16 +195,21 @@ class TestScalingPolicy(base.SenlinTestCase):
                 'count': 2,
             },
             'reason': 'Scaling request validated.',
-            'status': policy_base.CHECK_OK,
+            'status': pb.CHECK_OK,
         }
         action.data.update.assert_called_with(pd)
         action.store.assert_called_with(self.context)
 
     def test_pre_op_pass_with_input(self):
+        nodes = self._create_nodes(3)
+        self.cluster.nodes = nodes
+
         action = mock.Mock()
         action.context = self.context
         action.action = consts.CLUSTER_SCALE_IN
         action.inputs = {'count': 1}
+        action.entity = self.cluster
+
         adjustment = self.spec['properties']['adjustment']
         adjustment['type'] = consts.CHANGE_IN_CAPACITY
         adjustment['number'] = 2
@@ -204,7 +221,7 @@ class TestScalingPolicy(base.SenlinTestCase):
                 'count': 1,
             },
             'reason': 'Scaling request validated.',
-            'status': policy_base.CHECK_OK,
+            'status': pb.CHECK_OK,
         }
         action.data.update.assert_called_with(pd)
         action.store.assert_called_with(self.context)
@@ -217,15 +234,45 @@ class TestScalingPolicy(base.SenlinTestCase):
                 'count': 1,
             },
             'reason': 'Scaling request validated.',
-            'status': policy_base.CHECK_OK,
+            'status': pb.CHECK_OK,
         }
         action.data.update.assert_called_with(pd)
 
+    @mock.patch.object(sp.ScalingPolicy, '_calculate_adjustment_count')
+    def test_pre_op_pass_check_effort(self, mock_adjustmentcount):
+        # Cluster with maxsize and best_effort is False
+        self.cluster.nodes = [mock.Mock(), mock.Mock()]
+        action = mock.Mock()
+        action.context = self.context
+        action.action = consts.CLUSTER_SCALE_OUT
+        action.inputs = {}
+        action.entity = self.cluster
+
+        mock_adjustmentcount.return_value = 1
+        policy = sp.ScalingPolicy('test-policy', self.spec)
+        policy.event = consts.CLUSTER_SCALE_OUT
+        policy.best_effort = True
+        policy.pre_op(self.cluster_no_maxsize['id'], action)
+        pd = {
+            'creation': {
+                'count': 1,
+            },
+            'reason': 'Scaling request validated.',
+            'status': pb.CHECK_OK,
+        }
+        action.data.update.assert_called_with(pd)
+        action.store.assert_called_with(self.context)
+
     def test_pre_op_fail_negative_count(self):
+        nodes = self._create_nodes(3)
+        self.cluster.nodes = nodes
+
         action = mock.Mock()
         action.context = self.context
         action.action = consts.CLUSTER_SCALE_IN
         action.inputs = {}
+        action.entity = self.cluster
+
         adjustment = self.spec['properties']['adjustment']
         adjustment['type'] = consts.EXACT_CAPACITY
         adjustment['number'] = 5
@@ -234,17 +281,22 @@ class TestScalingPolicy(base.SenlinTestCase):
         policy.pre_op(self.cluster['id'], action)
 
         pd = {
-            'status': policy_base.CHECK_ERROR,
+            'status': pb.CHECK_ERROR,
             'reason': "Invalid count (-2) for action 'CLUSTER_SCALE_IN'.",
         }
         action.data.update.assert_called_with(pd)
         action.store.assert_called_with(self.context)
 
     def test_pre_op_fail_below_min_size(self):
+        nodes = self._create_nodes(3)
+        self.cluster.nodes = nodes
+
         action = mock.Mock()
         action.action = consts.CLUSTER_SCALE_IN
         action.context = self.context
         action.inputs = {}
+        action.entity = self.cluster
+
         adjustment = self.spec['properties']['adjustment']
         adjustment['type'] = consts.CHANGE_IN_CAPACITY
         adjustment['number'] = 3
@@ -253,7 +305,7 @@ class TestScalingPolicy(base.SenlinTestCase):
         policy.pre_op(self.cluster['id'], action)
 
         pd = {
-            'status': policy_base.CHECK_ERROR,
+            'status': pb.CHECK_ERROR,
             'reason': ("The target capacity (0) is less than the cluster's "
                        "min_size (1)."),
         }
@@ -261,10 +313,15 @@ class TestScalingPolicy(base.SenlinTestCase):
         action.store.assert_called_with(self.context)
 
     def test_pre_op_pass_best_effort(self):
+        nodes = self._create_nodes(3)
+        self.cluster.nodes = nodes
+
         action = mock.Mock()
         action.context = self.context
         action.action = consts.CLUSTER_SCALE_IN
         action.inputs = {}
+        action.entity = self.cluster
+
         adjustment = self.spec['properties']['adjustment']
         adjustment['best_effort'] = True
         adjustment['type'] = consts.CHANGE_IN_CAPACITY
@@ -277,20 +334,23 @@ class TestScalingPolicy(base.SenlinTestCase):
             'deletion': {
                 'count': 2,
             },
-            'status': policy_base.CHECK_OK,
+            'status': pb.CHECK_OK,
             'reason': 'Scaling request validated.',
         }
         action.data.update.assert_called_with(pd)
         action.store.assert_called_with(self.context)
 
     def test_pre_op_with_bad_nodes(self):
-        node_id = self.nodes[0]
-        db_api.node_update(self.context, node_id, {'status': 'ERROR'})
+        nodes = self._create_nodes(3)
+        no.Node.update(self.context, nodes[0].id, {'status': 'ERROR'})
+        self.cluster.nodes = nodes
 
         action = mock.Mock()
         action.context = self.context
         action.action = consts.CLUSTER_SCALE_IN
         action.inputs = {}
+        action.entity = self.cluster
+
         adjustment = self.spec['properties']['adjustment']
         adjustment['type'] = consts.EXACT_CAPACITY
         adjustment['number'] = 1
@@ -302,7 +362,7 @@ class TestScalingPolicy(base.SenlinTestCase):
                 'count': 2,
             },
             'reason': 'Scaling request validated.',
-            'status': policy_base.CHECK_OK,
+            'status': pb.CHECK_OK,
         }
         action.data.update.assert_called_with(pd)
         action.store.assert_called_with(self.context)
