@@ -96,13 +96,13 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc = mock.Mock()
         profile = stack.StackProfile('t', self.spec)
         profile._orchestrationclient = oc
-        test_stack = mock.Mock()
-        test_stack.name = 'test_stack'
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+        node.name = 'test_node'
         fake_stack = mock.Mock(id='FAKE_ID')
         oc.stack_create = mock.Mock(return_value=fake_stack)
 
         # do it
-        res = profile.do_create(test_stack)
+        res = profile.do_create(node)
 
         # assertions
         kwargs = {
@@ -114,6 +114,9 @@ class TestHeatStackProfile(base.SenlinTestCase):
             'parameters': self.spec['properties']['parameters'],
             'files': self.spec['properties']['files'],
             'environment': self.spec['properties']['environment'],
+            'tags': ",".join(['cluster_node_id=NODE_ID',
+                              'cluster_id=CLUSTER_ID',
+                              'cluster_node_index=123'])
         }
         self.assertEqual('FAKE_ID', res)
         oc.stack_create.assert_called_once_with(**kwargs)
@@ -138,13 +141,13 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc = mock.Mock()
         profile = stack.StackProfile('t', spec)
         profile._orchestrationclient = oc
-        test_stack = mock.Mock()
-        test_stack.name = 'test_stack'
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+        node.name = 'test_node'
         fake_stack = mock.Mock(id='FAKE_ID')
         oc.stack_create = mock.Mock(return_value=fake_stack)
 
         # do it
-        res = profile.do_create(test_stack)
+        res = profile.do_create(node)
 
         # assertions
         kwargs = {
@@ -156,6 +159,9 @@ class TestHeatStackProfile(base.SenlinTestCase):
             'parameters': spec['properties']['parameters'],
             'files': spec['properties']['files'],
             'environment': spec['properties']['environment'],
+            'tags': ",".join(['cluster_node_id=NODE_ID',
+                              'cluster_id=CLUSTER_ID',
+                              'cluster_node_index=123'])
         }
         self.assertEqual('FAKE_ID', res)
         oc.stack_create.assert_called_once_with(**kwargs)
@@ -168,15 +174,15 @@ class TestHeatStackProfile(base.SenlinTestCase):
         profile = stack.StackProfile('t', spec)
         oc = mock.Mock()
         profile._orchestrationclient = oc
-        test_stack = mock.Mock()
-        test_stack.name = 'test_stack'
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+        node.name = 'test_node'
         fake_stack = mock.Mock(id='FAKE_ID')
 
         oc.stack_create = mock.Mock(return_value=fake_stack)
         oc.wait_for_stack = mock.Mock()
 
         # do it
-        res = profile.do_create(test_stack)
+        res = profile.do_create(node)
 
         # assertions
         self.assertEqual('FAKE_ID', res)
@@ -189,6 +195,9 @@ class TestHeatStackProfile(base.SenlinTestCase):
             'parameters': self.spec['properties']['parameters'],
             'files': self.spec['properties']['files'],
             'environment': self.spec['properties']['environment'],
+            'tags': ",".join(['cluster_node_id=NODE_ID',
+                              'cluster_id=CLUSTER_ID',
+                              'cluster_node_index=123'])
         }
         oc.stack_create.assert_called_once_with(**kwargs)
         oc.wait_for_stack.assert_called_once_with('FAKE_ID', 'CREATE_COMPLETE',
@@ -198,8 +207,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc = mock.Mock()
         profile = stack.StackProfile('t', self.spec)
 
-        stack_node = mock.Mock()
-        stack_node.name = 'test_stack'
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+        node.name = 'test_node'
         err = exc.InternalError(code=400, message='Too Bad')
         oc.stack_create = mock.Mock(side_effect=err)
         profile._orchestrationclient = oc
@@ -207,7 +216,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         # do it
         ex = self.assertRaises(exc.EResourceCreation,
                                profile.do_create,
-                               stack_node)
+                               node)
 
         # assertions
         self.assertEqual('Failed in creating stack: Too Bad.',
@@ -221,6 +230,9 @@ class TestHeatStackProfile(base.SenlinTestCase):
             'parameters': self.spec['properties']['parameters'],
             'files': self.spec['properties']['files'],
             'environment': self.spec['properties']['environment'],
+            'tags': ",".join(['cluster_node_id=NODE_ID',
+                              'cluster_id=CLUSTER_ID',
+                              'cluster_node_index=123'])
         }
         oc.stack_create.assert_called_once_with(**call_args)
         self.assertEqual(0, oc.wait_for_stack.call_count)
@@ -230,8 +242,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
         del spec['properties']['timeout']
         profile = stack.StackProfile('t', spec)
         oc = mock.Mock()
-        stack_node = mock.Mock()
-        stack_node.name = 'test_stack'
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+        node.name = 'test_node'
         fake_stack = mock.Mock(id='FAKE_ID')
 
         oc.stack_create = mock.Mock(return_value=fake_stack)
@@ -242,7 +254,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
         # do it
         ex = self.assertRaises(exc.EResourceCreation,
                                profile.do_create,
-                               stack_node)
+                               node)
 
         # assertions
         self.assertEqual('Failed in creating stack: Timeout.',
@@ -256,6 +268,9 @@ class TestHeatStackProfile(base.SenlinTestCase):
             'parameters': self.spec['properties']['parameters'],
             'files': self.spec['properties']['files'],
             'environment': self.spec['properties']['environment'],
+            'tags': ",".join(['cluster_node_id=NODE_ID',
+                              'cluster_id=CLUSTER_ID',
+                              'cluster_node_index=123'])
         }
         oc.stack_create.assert_called_once_with(**kwargs)
         oc.wait_for_stack.assert_called_once_with('FAKE_ID', 'CREATE_COMPLETE',
@@ -275,6 +290,20 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc.stack_delete.assert_called_once_with('FAKE_ID', True)
         oc.wait_for_stack_delete.assert_called_once_with('FAKE_ID')
 
+    def test_do_delete_no_physical_id(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        test_stack = mock.Mock(physical_id=None)
+        profile._orchestrationclient = oc
+
+        # do it
+        res = profile.do_delete(test_stack, ignore_missing=False)
+
+        # assertions
+        self.assertTrue(res)
+        self.assertFalse(oc.stack_delete.called)
+        self.assertFalse(oc.wait_for_stack_delete.called)
+
     def test_do_delete_ignore_missing(self):
         profile = stack.StackProfile('t', self.spec)
         oc = mock.Mock()
@@ -303,7 +332,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
                                test_stack)
 
         # assertions
-        self.assertEqual('Failed in deleting stack FAKE_ID: Boom.',
+        self.assertEqual("Failed in deleting stack 'FAKE_ID': Boom.",
                          six.text_type(ex))
         oc.stack_delete.assert_called_once_with('FAKE_ID', True)
         self.assertEqual(0, oc.wait_for_stack_delete.call_count)
@@ -321,7 +350,7 @@ class TestHeatStackProfile(base.SenlinTestCase):
                                profile.do_delete, test_stack)
 
         # assertions
-        self.assertEqual('Failed in deleting stack FAKE_ID: Boom.',
+        self.assertEqual("Failed in deleting stack 'FAKE_ID': Boom.",
                          six.text_type(ex))
         oc.stack_delete.assert_called_once_with('FAKE_ID', True)
         oc.wait_for_stack_delete.assert_called_once_with('FAKE_ID')
@@ -512,8 +541,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc.stack_update.assert_called_once_with(
             'FAKE_ID', environment={"new": "env1"})
         self.assertEqual(0, oc.wait_for_stack.call_count)
-        self.assertEqual('Failed in updating stack FAKE_ID: Failed.',
-                         six.text_type(ex))
+        self.assertEqual("Failed in updating stack 'FAKE_ID': "
+                         "Failed.", six.text_type(ex))
 
     def test_do_update_timeout(self):
         profile = stack.StackProfile('t', self.spec)
@@ -534,8 +563,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
             'FAKE_ID', environment={"new": "env1"})
         oc.wait_for_stack.assert_called_once_with(
             'FAKE_ID', 'UPDATE_COMPLETE', timeout=3600)
-        self.assertEqual('Failed in updating stack FAKE_ID: Timeout.',
-                         six.text_type(ex))
+        self.assertEqual("Failed in updating stack 'FAKE_ID': "
+                         "Timeout.", six.text_type(ex))
 
     def test_do_check(self):
         node_obj = mock.Mock(physical_id='FAKE_ID')
@@ -568,9 +597,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc.stack_check = mock.Mock(
             side_effect=exc.InternalError(code=400, message='BOOM'))
 
-        res = profile.do_check(node_obj)
+        self.assertRaises(exc.EResourceOperation, profile.do_check, node_obj)
 
-        self.assertFalse(res)
         oc.stack_check.assert_called_once_with('FAKE_ID')
         self.assertEqual(0, oc.wait_for_stack.call_count)
 
@@ -582,9 +610,8 @@ class TestHeatStackProfile(base.SenlinTestCase):
         oc.wait_for_stack = mock.Mock(
             side_effect=exc.InternalError(code=400, message='BOOM'))
 
-        res = profile.do_check(node_obj)
+        self.assertRaises(exc.EResourceOperation, profile.do_check, node_obj)
 
-        self.assertFalse(res)
         oc.stack_check.assert_called_once_with('FAKE_ID')
         oc.wait_for_stack.assert_called_once_with(
             'FAKE_ID', 'CHECK_COMPLETE', timeout=3600)
@@ -622,3 +649,361 @@ class TestHeatStackProfile(base.SenlinTestCase):
 
         self.assertEqual({'Error': {'code': 500, 'message': 'BOOM'}}, res)
         oc.stack_get.assert_called_once_with('STACK_ID')
+
+    def test_do_adopt(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock(
+            parameters={'p1': 'v1', 'OS::stack_id': 'FAKE_ID'},
+            timeout_mins=123,
+            is_rollback_disabled=False
+        )
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+
+        # mock template
+        templ = mock.Mock()
+        templ.to_dict.return_value = {'foo': 'bar'}
+        oc.stack_get_template = mock.Mock(return_value=templ)
+
+        # mock environment
+        env = mock.Mock()
+        env.to_dict.return_value = {'ke': 've'}
+        oc.stack_get_environment = mock.Mock(return_value=env)
+
+        oc.stack_get_files = mock.Mock(return_value={'fn': 'content'})
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {
+            'environment': {'ke': 've'},
+            'files': {'fn': 'content'},
+            'template': {'foo': 'bar'},
+            'parameters': {'p1': 'v1'},
+            'timeout': 123,
+            'disable_rollback': False
+        }
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+        oc.stack_get_environment.assert_called_once_with('FAKE_ID')
+        oc.stack_get_files.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        oc.stack_get.side_effect = exc.InternalError(message='BOOM')
+        profile._orchestrationclient = oc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get_template(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock()
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template.side_effect = exc.InternalError(message='BOOM')
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get_environment(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock()
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template = mock.Mock(return_value={'foo': 'bar'})
+        err = exc.InternalError(message='BOOM')
+        oc.stack_get_environment.side_effect = err
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+        oc.stack_get_environment.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_failed_get_files(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock()
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+        oc.stack_get_template = mock.Mock(return_value={'foo': 'bar'})
+        oc.stack_get_environment = mock.Mock(return_value={'ke': 've'})
+        oc.stack_get_files.side_effect = exc.InternalError(message='BOOM')
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.do_adopt(node_obj)
+
+        expected = {'Error': {'code': 500, 'message': 'BOOM'}}
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+        oc.stack_get_environment.assert_called_once_with('FAKE_ID')
+        oc.stack_get_files.assert_called_once_with('FAKE_ID')
+
+    def test_do_adopt_with_overrides(self):
+        profile = stack.StackProfile('t', self.spec)
+        x_stack = mock.Mock(
+            parameters={'p1': 'v1', 'OS::stack_id': 'FAKE_ID'},
+            timeout_mins=123,
+            is_rollback_disabled=False
+        )
+        oc = mock.Mock()
+        oc.stack_get = mock.Mock(return_value=x_stack)
+
+        # mock environment
+        env = mock.Mock()
+        env.to_dict.return_value = {'ke': 've'}
+        oc.stack_get_environment = mock.Mock(return_value=env)
+
+        # mock template
+        templ = mock.Mock()
+        templ.to_dict.return_value = {'foo': 'bar'}
+        oc.stack_get_template = mock.Mock(return_value=templ)
+
+        oc.stack_get_files = mock.Mock(return_value={'fn': 'content'})
+        profile._orchestrationclient = oc
+
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+        overrides = {'environment': {'ENV': 'SETTING'}}
+        res = profile.do_adopt(node_obj, overrides=overrides)
+
+        expected = {
+            'environment': {'ENV': 'SETTING'},
+            'files': {'fn': 'content'},
+            'template': {'foo': 'bar'},
+            'parameters': {'p1': 'v1'},
+            'timeout': 123,
+            'disable_rollback': False
+        }
+        self.assertEqual(expected, res)
+        oc.stack_get.assert_called_once_with('FAKE_ID')
+        oc.stack_get_template.assert_called_once_with('FAKE_ID')
+
+    def test__refresh_tags_empty_no_add(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock()
+
+        res = profile._refresh_tags([], node, False)
+
+        self.assertEqual(("", False), res)
+
+    def test__refresh_tags_with_contents_no_add(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock()
+
+        res = profile._refresh_tags(['foo'], node, False)
+
+        self.assertEqual(('foo', False), res)
+
+    def test__refresh_tags_deleted_no_add(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock()
+
+        res = profile._refresh_tags(['cluster_id=FOO', 'bar'], node, False)
+
+        self.assertEqual(('bar', True), res)
+
+    def test__refresh_tags_empty_and_add(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+
+        res = profile._refresh_tags([], node, True)
+
+        expected = ",".join(['cluster_id=CLUSTER_ID',
+                             'cluster_node_id=NODE_ID',
+                             'cluster_node_index=123'])
+        self.assertEqual((expected, True), res)
+
+    def test__refresh_tags_with_contents_and_add(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+
+        res = profile._refresh_tags(['foo'], node, True)
+
+        expected = ",".join(['foo',
+                             'cluster_id=CLUSTER_ID',
+                             'cluster_node_id=NODE_ID',
+                             'cluster_node_index=123'])
+        self.assertEqual((expected, True), res)
+
+    def test__refresh_tags_deleted_and_add(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock(id='NODE_ID', cluster_id='CLUSTER_ID', index=123)
+
+        res = profile._refresh_tags(['cluster_id=FOO', 'bar'], node, True)
+
+        expected = ",".join(['bar',
+                             'cluster_id=CLUSTER_ID',
+                             'cluster_node_id=NODE_ID',
+                             'cluster_node_index=123'])
+        self.assertEqual((expected, True), res)
+
+    def test_do_join(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        x_stack = mock.Mock(tags='foo')
+        oc.stack_get.return_value = x_stack
+        node = mock.Mock(physical_id='STACK_ID')
+        mock_tags = self.patchobject(profile, '_refresh_tags',
+                                     return_value=('bar', True))
+
+        res = profile.do_join(node, 'CLUSTER_ID')
+
+        self.assertTrue(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+        mock_tags.assert_called_once_with('foo', node, True)
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
+
+    def test_do_join_no_physical_id(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock(physical_id=None)
+
+        res = profile.do_join(node, 'CLUSTER_ID')
+
+        self.assertFalse(res)
+
+    def test_do_join_failed_get_stack(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        err = exc.InternalError(code=400, message='Boom')
+        oc.stack_get.side_effect = err
+        node = mock.Mock(physical_id='STACK_ID')
+
+        res = profile.do_join(node, 'CLUSTER_ID')
+
+        self.assertFalse(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+
+    def test_do_join_no_update(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        x_stack = mock.Mock(tags='foo')
+        oc.stack_get.return_value = x_stack
+        node = mock.Mock(physical_id='STACK_ID')
+        mock_tags = self.patchobject(profile, '_refresh_tags',
+                                     return_value=('foo', False))
+
+        res = profile.do_join(node, 'CLUSTER_ID')
+
+        self.assertTrue(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+        mock_tags.assert_called_once_with('foo', node, True)
+        self.assertEqual(0, oc.stack_update.call_count)
+
+    def test_do_join_failed_update(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        x_stack = mock.Mock(tags='foo')
+        oc.stack_get.return_value = x_stack
+        err = exc.InternalError(code=400, message='Boom')
+        oc.stack_update.side_effect = err
+        node = mock.Mock(physical_id='STACK_ID')
+        mock_tags = self.patchobject(profile, '_refresh_tags',
+                                     return_value=('bar', True))
+
+        res = profile.do_join(node, 'CLUSTER_ID')
+
+        self.assertFalse(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+        mock_tags.assert_called_once_with('foo', node, True)
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
+
+    def test_do_leave(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        x_stack = mock.Mock(tags='foo')
+        oc.stack_get.return_value = x_stack
+        node = mock.Mock(physical_id='STACK_ID')
+        mock_tags = self.patchobject(profile, '_refresh_tags',
+                                     return_value=('bar', True))
+
+        res = profile.do_leave(node)
+
+        self.assertTrue(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+        mock_tags.assert_called_once_with('foo', node, False)
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
+
+    def test_do_leave_no_physical_id(self):
+        profile = stack.StackProfile('t', self.spec)
+        node = mock.Mock(physical_id=None)
+
+        res = profile.do_leave(node)
+
+        self.assertFalse(res)
+
+    def test_do_leave_failed_get_stack(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        err = exc.InternalError(code=400, message='Boom')
+        oc.stack_get.side_effect = err
+        node = mock.Mock(physical_id='STACK_ID')
+
+        res = profile.do_leave(node)
+
+        self.assertFalse(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+
+    def test_do_leave_no_update(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        x_stack = mock.Mock(tags='foo')
+        oc.stack_get.return_value = x_stack
+        node = mock.Mock(physical_id='STACK_ID')
+        mock_tags = self.patchobject(profile, '_refresh_tags',
+                                     return_value=('foo', False))
+
+        res = profile.do_leave(node)
+
+        self.assertTrue(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+        mock_tags.assert_called_once_with('foo', node, False)
+        self.assertEqual(0, oc.stack_update.call_count)
+
+    def test_do_leave_failed_update(self):
+        profile = stack.StackProfile('t', self.spec)
+        oc = mock.Mock()
+        profile._orchestrationclient = oc
+        x_stack = mock.Mock(tags='foo')
+        oc.stack_get.return_value = x_stack
+        err = exc.InternalError(code=400, message='Boom')
+        oc.stack_update.side_effect = err
+        node = mock.Mock(physical_id='STACK_ID')
+        mock_tags = self.patchobject(profile, '_refresh_tags',
+                                     return_value=('bar', True))
+
+        res = profile.do_leave(node)
+
+        self.assertFalse(res)
+        oc.stack_get.assert_called_once_with('STACK_ID')
+        mock_tags.assert_called_once_with('foo', node, False)
+        oc.stack_update.assert_called_once_with('STACK_ID', **{'tags': 'bar'})
