@@ -16,6 +16,7 @@ import mock
 import six
 
 from senlin.common import exception as exc
+from senlin.objects import node as node_obj
 from senlin.profiles.os.nova import server
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
@@ -156,6 +157,7 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                 'user_data': 'FAKE_USER_DATA',
             }
         }
+        self.patchobject(node_obj.Node, 'update')
 
     def test__update_name(self):
         profile = server.ServerProfile('t', self.spec)
@@ -179,7 +181,7 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_name,
                                obj, 'NEW_NAME')
 
-        self.assertEqual('Failed in updating server NOVA_ID: BOOM.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': BOOM.",
                          six.text_type(ex))
         cc.server_update.assert_called_once_with('NOVA_ID', name='NEW_NAME')
 
@@ -207,7 +209,7 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_password,
                                obj, 'NEW_PASSWORD')
 
-        self.assertEqual('Failed in updating server NOVA_ID: BOOM.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': BOOM.",
                          six.text_type(ex))
         cc.server_change_password.assert_called_once_with(
             'NOVA_ID', 'NEW_PASSWORD')
@@ -265,8 +267,8 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_metadata,
                                obj, new_profile)
 
-        self.assertEqual('Failed in updating server NOVA_ID: Nova Error.',
-                         six.text_type(ex))
+        self.assertEqual("Failed in updating server 'NOVA_ID': "
+                         "Nova Error.", six.text_type(ex))
         cc.server_metadata_update.assert_called_once_with(
             'NOVA_ID', {'fooa': 'baaar', 'cluster_node_id': 'NODE_ID'}
         )
@@ -382,8 +384,8 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         cc.server_resize.assert_called_once_with('NOVA_ID', '456')
         cc.server_resize_revert.assert_called_once_with('NOVA_ID')
         cc.wait_for_server.assert_called_once_with('NOVA_ID', 'ACTIVE')
-        self.assertEqual('Failed in updating server NOVA_ID: Resize failed.',
-                         six.text_type(ex))
+        self.assertEqual("Failed in updating server 'NOVA_ID': Resize "
+                         "failed.", six.text_type(ex))
 
     def test__update_flavor_first_wait_for_server_failed(self):
         obj = mock.Mock(physical_id='NOVA_ID')
@@ -416,8 +418,8 @@ class TestNovaServerUpdate(base.SenlinTestCase):
             mock.call('NOVA_ID', 'VERIFY_RESIZE'),
             mock.call('NOVA_ID', 'ACTIVE')])
         cc.server_resize_revert.assert_called_once_with('NOVA_ID')
-        self.assertEqual('Failed in updating server NOVA_ID: TIMEOUT.',
-                         six.text_type(ex))
+        self.assertEqual("Failed in updating server 'NOVA_ID': "
+                         "TIMEOUT.", six.text_type(ex))
 
     def test__update_flavor_resize_failed_revert_failed(self):
         obj = mock.Mock(physical_id='NOVA_ID')
@@ -449,8 +451,8 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         cc.server_resize_revert.assert_called_once_with('NOVA_ID')
         # the wait_for_server wasn't called
         self.assertEqual(0, cc.wait_for_server.call_count)
-        self.assertEqual('Failed in updating server NOVA_ID: Revert.',
-                         six.text_type(ex))
+        self.assertEqual("Failed in updating server 'NOVA_ID': "
+                         "Revert.", six.text_type(ex))
 
     def test__update_flavor_confirm_failed(self):
         obj = mock.Mock(physical_id='NOVA_ID')
@@ -479,7 +481,7 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         cc.server_resize.assert_called_once_with('NOVA_ID', '456')
         cc.server_resize_confirm.assert_called_once_with('NOVA_ID')
         cc.wait_for_server.assert_called_once_with('NOVA_ID', 'VERIFY_RESIZE')
-        self.assertEqual('Failed in updating server NOVA_ID: Confirm.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': Confirm.",
                          six.text_type(ex))
 
     def test__update_flavor_wait_confirm_failed(self):
@@ -512,16 +514,18 @@ class TestNovaServerUpdate(base.SenlinTestCase):
             mock.call('NOVA_ID', 'VERIFY_RESIZE'),
             mock.call('NOVA_ID', 'ACTIVE')
         ])
-        self.assertEqual('Failed in updating server NOVA_ID: Wait.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': Wait.",
                          six.text_type(ex))
 
     def test__update_image(self):
         profile = server.ServerProfile('t', self.spec)
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
         cc = mock.Mock()
+        cc.server_get.return_value = x_server
         profile._computeclient = cc
-        x_old_image = mock.Mock(id='123')
         x_new_image = mock.Mock(id='456')
-        x_images = [x_new_image, x_old_image]
+        x_images = [x_new_image]
         mock_check = self.patchobject(profile, '_validate_image',
                                       side_effect=x_images)
         obj = mock.Mock(physical_id='NOVA_ID')
@@ -533,7 +537,6 @@ class TestNovaServerUpdate(base.SenlinTestCase):
 
         mock_check.assert_has_calls([
             mock.call(obj, 'new_image', reason='update'),
-            mock.call(obj, 'FAKE_IMAGE', reason='update')
         ])
         cc.server_rebuild.assert_called_once_with(
             'NOVA_ID', '456', 'new_name', 'new_pass')
@@ -550,8 +553,8 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_image,
                                obj, new_profile, 'new_name', '')
 
-        msg = ("Failed in updating server NOVA_ID: Updating Nova server with "
-               "image set to None is not supported by Nova.")
+        msg = ("Failed in updating server 'NOVA_ID': Updating Nova server"
+               " with image set to None is not supported by Nova.")
         self.assertEqual(msg, six.text_type(ex))
 
     def test__update_image_new_image_invalid(self):
@@ -571,7 +574,7 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_image,
                                obj, new_profile, 'new_name', 'new_pass')
 
-        msg = ("Failed in updating server NOVA_ID: BAD.")
+        msg = ("Failed in updating server 'NOVA_ID': BAD.")
         self.assertEqual(msg, six.text_type(ex))
         mock_check.assert_called_once_with(obj, 'new_image', reason='update')
 
@@ -579,10 +582,13 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         # NOTE: The image invalid could be caused by a non-existent image or
         # a compute driver failure
         profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = mock.Mock()
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
+        cc = mock.Mock()
+        cc.server_get.return_value = x_server
+        profile._computeclient = cc
         # _validate_image will always throw EResourceUpdate if driver fails
         results = [
-            mock.Mock(id='NEW_IMAGE'),
             exc.EResourceUpdate(type='server', id='NOVA_ID', message='BAD')
         ]
         mock_check = self.patchobject(profile, '_validate_image',
@@ -596,11 +602,10 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_image,
                                obj, new_profile, 'new_name', 'new_pass')
 
-        msg = ("Failed in updating server NOVA_ID: BAD.")
+        msg = ("Failed in updating server 'NOVA_ID': BAD.")
         self.assertEqual(msg, six.text_type(ex))
         mock_check.assert_has_calls([
             mock.call(obj, 'new_image', reason='update'),
-            mock.call(obj, 'FAKE_IMAGE', reason='update')
         ])
 
     def test__update_image_old_image_is_none_but_succeeded(self):
@@ -649,18 +654,20 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_image,
                                obj, new_profile, 'new_name', 'new_pass')
 
-        self.assertEqual('Failed in updating server NOVA_ID: DRIVER.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': DRIVER.",
                          six.text_type(ex))
         mock_check.assert_called_once_with(obj, 'new_image', reason='update')
         cc.server_get.assert_called_once_with('NOVA_ID')
 
     def test__update_image_updating_to_same_image(self):
         profile = server.ServerProfile('t', self.spec)
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
         cc = mock.Mock()
+        cc.server_get.return_value = x_server
         profile._computeclient = cc
-        x_old_image = mock.Mock(id='123')
         x_new_image = mock.Mock(id='123')
-        x_images = [x_old_image, x_new_image]
+        x_images = [x_new_image]
         mock_check = self.patchobject(profile, '_validate_image',
                                       side_effect=x_images)
         obj = mock.Mock(physical_id='NOVA_ID')
@@ -673,19 +680,20 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         self.assertFalse(res)
         mock_check.assert_has_calls([
             mock.call(obj, 'new_image', reason='update'),
-            mock.call(obj, 'FAKE_IMAGE', reason='update')
         ])
         self.assertEqual(0, cc.server_rebuild.call_count)
         self.assertEqual(0, cc.wait_for_server.call_count)
 
     def test__update_image_failed_rebuilding(self):
         profile = server.ServerProfile('t', self.spec)
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
         cc = mock.Mock()
+        cc.server_get.return_value = x_server
         cc.server_rebuild.side_effect = exc.InternalError(message='FAILED')
         profile._computeclient = cc
-        x_old_image = mock.Mock(id='123')
         x_new_image = mock.Mock(id='456')
-        x_images = [x_new_image, x_old_image]
+        x_images = [x_new_image]
         mock_check = self.patchobject(profile, '_validate_image',
                                       side_effect=x_images)
         obj = mock.Mock(physical_id='NOVA_ID')
@@ -697,11 +705,10 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_image,
                                obj, new_profile, 'new_name', 'new_pass')
 
-        self.assertEqual('Failed in updating server NOVA_ID: FAILED.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': FAILED.",
                          six.text_type(ex))
         mock_check.assert_has_calls([
             mock.call(obj, 'new_image', reason='update'),
-            mock.call(obj, 'FAKE_IMAGE', reason='update')
         ])
         cc.server_rebuild.assert_called_once_with(
             'NOVA_ID', '456', 'new_name', 'new_pass')
@@ -709,12 +716,14 @@ class TestNovaServerUpdate(base.SenlinTestCase):
 
     def test__update_image_failed_waiting(self):
         profile = server.ServerProfile('t', self.spec)
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
         cc = mock.Mock()
+        cc.server_get.return_value = x_server
         cc.wait_for_server.side_effect = exc.InternalError(message='TIMEOUT')
         profile._computeclient = cc
-        x_old_image = mock.Mock(id='123')
         x_new_image = mock.Mock(id='456')
-        x_images = [x_new_image, x_old_image]
+        x_images = [x_new_image]
         mock_check = self.patchobject(profile, '_validate_image',
                                       side_effect=x_images)
         obj = mock.Mock(physical_id='NOVA_ID')
@@ -726,11 +735,10 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile._update_image,
                                obj, new_profile, 'new_name', 'new_pass')
 
-        self.assertEqual('Failed in updating server NOVA_ID: TIMEOUT.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': TIMEOUT.",
                          six.text_type(ex))
         mock_check.assert_has_calls([
             mock.call(obj, 'new_image', reason='update'),
-            mock.call(obj, 'FAKE_IMAGE', reason='update')
         ])
         cc.server_rebuild.assert_called_once_with(
             'NOVA_ID', '456', 'new_name', 'new_pass')
@@ -743,23 +751,39 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         profile = server.ServerProfile('t', self.spec)
         profile._computeclient = cc
         validation_results = [
-            {'net_id': 'net1', 'fixed_ips': [{'ip_address': 'ip2'}]},
-            {'net_id': 'net2'},
-            {'port_id': 'port4'}
+            {'network': 'net1_id', 'fixed_ip': 'ip2'},
+            {'network': 'net2_id'},
+            {'port': 'port4'}
         ]
         mock_validate = self.patchobject(profile, '_validate_network',
                                          side_effect=validation_results)
+        ports_results = [
+            (mock.Mock(
+                id='port1_id', network_id='net1_id',
+                fixed_ips=[{'ip_address': 'ip2'}], security_group_ids=[]),
+             None),
+            (mock.Mock(
+                id='port2_id', network_id='net2_id',
+                fixed_ips=[{'ip_address': 'ip3'}], security_group_ids=[]),
+             None),
+            (mock.Mock(
+                id='port4_id', network_id='net3_id',
+                fixed_ips=[{'ip_address': 'ip4'}], security_group_ids=[]),
+             None)
+        ]
+        mock_get_port = self.patchobject(profile, '_get_port',
+                                         side_effect=ports_results)
         networks = [
             {'network': 'net1', 'port': None, 'fixed_ip': 'ip2'},
             {'network': 'net2', 'port': None, 'fixed_ip': None},
             {'network': None, 'port': 'port4', 'fixed_ip': None}
         ]
-        obj = mock.Mock(physical_id='NOVA_ID')
+        obj = mock.Mock(physical_id='NOVA_ID', data={})
 
-        res = profile._create_interfaces(obj, networks)
+        res = profile._update_network_add_port(obj, networks)
 
         self.assertIsNone(res)
-        cc.server_get.assert_called_once_with('NOVA_ID')
+        cc.server_get.assert_called_with('NOVA_ID')
         validation_calls = [
             mock.call(obj,
                       {'network': 'net1', 'port': None, 'fixed_ip': 'ip2'},
@@ -772,11 +796,11 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                       'update')
         ]
         mock_validate.assert_has_calls(validation_calls)
+        mock_get_port.assert_called_with(obj, {'port': 'port4'})
         create_calls = [
-            mock.call(server_obj, net_id='net1',
-                      fixed_ips=[{'ip_address': 'ip2'}]),
-            mock.call(server_obj, net_id='net2'),
-            mock.call(server_obj, port_id='port4'),
+            mock.call(server_obj, port='port1_id'),
+            mock.call(server_obj, port='port2_id'),
+            mock.call(server_obj, port='port4_id'),
         ]
         cc.server_interface_create.assert_has_calls(create_calls)
 
@@ -785,18 +809,19 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         cc.server_get.side_effect = exc.InternalError(message='Not valid')
         profile = server.ServerProfile('t', self.spec)
         profile._computeclient = cc
+        self.patchobject(profile, '_create_ports_from_properties')
 
         obj = mock.Mock(physical_id='NOVA_ID')
         networks = [{'foo': 'bar'}]  # not used
 
         ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._create_interfaces,
+                               profile._update_network_add_port,
                                obj, networks)
 
-        self.assertEqual('Failed in updating server NOVA_ID: Not valid.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': Not valid.",
                          six.text_type(ex))
         cc.server_get.assert_called_once_with('NOVA_ID')
-        self.assertEqual(0, cc.server_interface_create.call_count)
+        self.assertEqual(0, profile._create_ports_from_properties.call_count)
 
     def test__create_interfaces_failed_validation(self):
         cc = mock.Mock()
@@ -812,10 +837,10 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         obj = mock.Mock(physical_id='NOVA_ID')
 
         ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._create_interfaces,
+                               profile._update_network_add_port,
                                obj, networks)
 
-        self.assertEqual('Failed in updating server NOVA_ID: Driver error.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': Driver error.",
                          six.text_type(ex))
         cc.server_get.assert_called_once_with('NOVA_ID')
         mock_validate.assert_called_once_with(obj, networks[0], 'update')
@@ -825,37 +850,28 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         cc = mock.Mock()
         nc = mock.Mock()
         net1 = mock.Mock(id='net1')
-        interfaces = [
-            mock.Mock(port_id='port1', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip1'
-                      }]),
-            mock.Mock(port_id='port2', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip-random2'
-                      }]),
-            mock.Mock(port_id='port3', net_id='net2',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet2', 'ip_address': 'ip3'
-                      }])
-        ]
-        cc.server_interface_list.return_value = interfaces
-        nc.network_get.side_effect = [net1, net1]
+        nc.network_get.return_value = net1
+        nc.port_find.return_value = mock.Mock(id='port3', status='DOWN')
         profile = server.ServerProfile('t', self.spec)
         profile._computeclient = cc
         profile._networkclient = nc
-        obj = mock.Mock(physical_id='NOVA_ID')
+        obj = mock.Mock(physical_id='NOVA_ID', data={'internal_ports': [
+            {'id': 'port1', 'network_id': 'net1', 'remove': True,
+             'fixed_ips': [{'ip_address': 'ip1'}]},
+            {'id': 'port2', 'network_id': 'net1', 'remove': True,
+             'fixed_ips': [{'ip_address': 'ip-random2'}]},
+            {'id': 'port3', 'network_id': 'net1', 'remove': True,
+             'fixed_ips': [{'ip_address': 'ip3'}]}]})
         networks = [
             {'network': 'net1', 'port': None, 'fixed_ip': 'ip1'},
             {'network': 'net1', 'port': None, 'fixed_ip': None},
             {'network': None, 'port': 'port3', 'fixed_ip': None}
         ]
 
-        res = profile._delete_interfaces(obj, networks)
+        res = profile._update_network_remove_port(obj, networks)
 
         self.assertIsNone(res)
 
-        cc.server_interface_list.assert_called_once_with('NOVA_ID')
         nc.network_get.assert_has_calls([
             mock.call('net1'), mock.call('net1')
         ])
@@ -864,196 +880,44 @@ class TestNovaServerUpdate(base.SenlinTestCase):
             mock.call('port2', 'NOVA_ID'),
             mock.call('port3', 'NOVA_ID'),
         ])
+        nc.port_delete.assert_has_calls([
+            mock.call('port1', ignore_missing=True),
+            mock.call('port2', ignore_missing=True),
+            mock.call('port3', ignore_missing=True),
+        ])
 
-    def test__delete_interfaces_failed_listing_interfaces(self):
+    def test__delete_interfaces_failed_delete(self):
         cc = mock.Mock()
         profile = server.ServerProfile('t', self.spec)
         profile._computeclient = cc
         profile._networkclient = mock.Mock()
-        err = exc.InternalError(message='BANG')
-        cc.server_interface_list.side_effect = err
-        obj = mock.Mock(physical_id='NOVA_ID')
-
-        ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._delete_interfaces,
-                               obj, mock.Mock())
-
-        self.assertEqual('Failed in updating server NOVA_ID: BANG.',
-                         six.text_type(ex))
-        cc.server_interface_list.assert_called_once_with('NOVA_ID')
-
-    def test__delete_interfaces_failed_1st_delete(self):
-        cc = mock.Mock()
-        profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = cc
-        profile._networkclient = mock.Mock()
-        interfaces = [
-            mock.Mock(port_id='port1', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip1'
-                      }]),
-            mock.Mock(port_id='port2', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip-random2'
-                      }]),
-            mock.Mock(port_id='port3', net_id='net2',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet2', 'ip_address': 'ip3'
-                      }])
+        candidate_ports = [
+            [{'id': 'port1', 'network_id': 'net1',
+             'fixed_ips': [{'ip_address': 'ip1'}]}],
         ]
-        cc.server_interface_list.return_value = interfaces
+        self.patchobject(profile, '_find_port_by_net_spec',
+                         side_effect=candidate_ports)
         err = exc.InternalError(message='BANG')
         cc.server_interface_delete.side_effect = err
-        obj = mock.Mock(physical_id='NOVA_ID')
-        networks = [
-            {'network': 'net1', 'port': None, 'fixed_ip': 'ip1'},
-            {'network': 'net1', 'port': None, 'fixed_ip': None},
-            {'network': None, 'port': 'port3', 'fixed_ip': None}
+        internal_ports = [
+            {'id': 'port1', 'remove': True}
         ]
-
-        ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._delete_interfaces,
-                               obj, networks)
-
-        self.assertEqual('Failed in updating server NOVA_ID: BANG.',
-                         six.text_type(ex))
-        cc.server_interface_list.assert_called_once_with('NOVA_ID')
-        cc.server_interface_delete.assert_called_once_with('port3', 'NOVA_ID')
-
-    def test__delete_interfaces_failed_1st_get_network(self):
-        cc = mock.Mock()
-        nc = mock.Mock()
-        profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = cc
-        profile._networkclient = nc
-        interfaces = [
-            mock.Mock(port_id='port1', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip1'
-                      }]),
-        ]
-        cc.server_interface_list.return_value = interfaces
-        err = exc.InternalError(message='BANG')
-        nc.network_get.side_effect = err
-        obj = mock.Mock(physical_id='NOVA_ID')
+        obj = mock.Mock(physical_id='NOVA_ID',
+                        data={'internal_ports': internal_ports})
         networks = [
             {'network': 'net1', 'port': None, 'fixed_ip': 'ip1'},
         ]
 
         ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._delete_interfaces,
+                               profile._update_network_remove_port,
                                obj, networks)
 
-        self.assertEqual('Failed in updating server NOVA_ID: BANG.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': BANG.",
                          six.text_type(ex))
-        cc.server_interface_list.assert_called_once_with('NOVA_ID')
-        nc.network_get.assert_called_once_with('net1')
-
-    def test__delete_interfaces_failed_2nd_delete(self):
-        cc = mock.Mock()
-        nc = mock.Mock()
-        profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = cc
-        profile._networkclient = nc
-        interfaces = [
-            mock.Mock(port_id='port3', net_id='net2',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet2', 'ip_address': 'ip3'
-                      }])
-        ]
-        cc.server_interface_list.return_value = interfaces
-        net2 = mock.Mock(id='net2')
-        nc.network_get.return_value = net2
-        err = exc.InternalError(message='BANG')
-        cc.server_interface_delete.side_effect = err
-        obj = mock.Mock(physical_id='NOVA_ID')
-        networks = [
-            {'network': 'net2', 'port': None, 'fixed_ip': None},
-        ]
-
-        ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._delete_interfaces,
-                               obj, networks)
-
-        self.assertEqual('Failed in updating server NOVA_ID: BANG.',
-                         six.text_type(ex))
-        cc.server_interface_list.assert_called_once_with('NOVA_ID')
-        nc.network_get.assert_called_once_with('net2')
-
-    def test__delete_interfaces_failed_2nd_get_network(self):
-        cc = mock.Mock()
-        nc = mock.Mock()
-        profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = cc
-        profile._networkclient = nc
-        interfaces = [
-            mock.Mock(port_id='port1', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip1'
-                      }]),
-        ]
-        cc.server_interface_list.return_value = interfaces
-        net1 = mock.Mock(id='net1')
-        nc.network_get.return_value = net1
-        err = exc.InternalError(message='BANG')
-        cc.server_interface_delete.side_effect = err
-        obj = mock.Mock(physical_id='NOVA_ID')
-        networks = [
-            {'network': 'net1', 'port': None, 'fixed_ip': 'ip1'},
-        ]
-
-        ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._delete_interfaces,
-                               obj, networks)
-
-        self.assertEqual('Failed in updating server NOVA_ID: BANG.',
-                         six.text_type(ex))
-        cc.server_interface_list.assert_called_once_with('NOVA_ID')
-        nc.network_get.assert_called_once_with('net1')
         cc.server_interface_delete.assert_called_once_with('port1', 'NOVA_ID')
 
-    def test__delete_interfaces_failed_3rd_delete(self):
-        cc = mock.Mock()
-        nc = mock.Mock()
-        profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = cc
-        profile._networkclient = nc
-        interfaces = [
-            mock.Mock(port_id='port1', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip1'
-                      }]),
-            mock.Mock(port_id='port2', net_id='net1',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet1', 'ip_address': 'ip-random2'
-                      }]),
-            mock.Mock(port_id='port3', net_id='net2',
-                      fixed_ips=[{
-                          'subnet_id': 'subnet2', 'ip_address': 'ip3'
-                      }])
-        ]
-        cc.server_interface_list.return_value = interfaces
-        net2 = mock.Mock(id='net2')
-        nc.network_get.return_value = net2
-        err = exc.InternalError(message='BANG')
-        cc.server_interface_delete.side_effect = err
-        obj = mock.Mock(physical_id='NOVA_ID')
-        networks = [
-            {'network': 'net2', 'port': None, 'fixed_ip': None},
-        ]
-
-        ex = self.assertRaises(exc.EResourceUpdate,
-                               profile._delete_interfaces,
-                               obj, networks)
-
-        self.assertEqual('Failed in updating server NOVA_ID: BANG.',
-                         six.text_type(ex))
-        cc.server_interface_list.assert_called_once_with('NOVA_ID')
-        nc.network_get.assert_called_once_with('net2')
-        cc.server_interface_delete.assert_called_once_with('port3', 'NOVA_ID')
-
-    @mock.patch.object(server.ServerProfile, '_delete_interfaces')
-    @mock.patch.object(server.ServerProfile, '_create_interfaces')
+    @mock.patch.object(server.ServerProfile, '_update_network_remove_port')
+    @mock.patch.object(server.ServerProfile, '_update_network_add_port')
     def test__update_network(self, mock_create, mock_delete):
         obj = mock.Mock(physical_id='FAKE_ID')
 
@@ -1061,14 +925,14 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         old_spec['properties']['networks'] = [
             {'network': 'net1', 'fixed_ip': 'ip1'},
             {'network': 'net1'},
-            {'port': 'port3'}
+            {'port': 'port3'},
         ]
         profile = server.ServerProfile('t', old_spec)
         new_spec = copy.deepcopy(self.spec)
         new_spec['properties']['networks'] = [
             {'network': 'net1', 'fixed_ip': 'ip2'},
             {'network': 'net2'},
-            {'port': 'port4'}
+            {'port': 'port4'},
         ]
         new_profile = server.ServerProfile('t1', new_spec)
 
@@ -1077,15 +941,21 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         self.assertIsNone(res)
 
         networks_create = [
-            {'network': 'net1', 'port': None, 'fixed_ip': 'ip2'},
-            {'network': 'net2', 'port': None, 'fixed_ip': None},
-            {'network': None, 'port': 'port4', 'fixed_ip': None}
+            {'floating_network': None, 'network': 'net1', 'fixed_ip': 'ip2',
+             'floating_ip': None, 'port': None, 'security_groups': None},
+            {'floating_network': None, 'network': 'net2', 'fixed_ip': None,
+             'floating_ip': None, 'port': None, 'security_groups': None},
+            {'floating_network': None, 'network': None, 'fixed_ip': None,
+             'floating_ip': None, 'port': 'port4', 'security_groups': None}
         ]
         mock_create.assert_called_once_with(obj, networks_create)
         networks_delete = [
-            {'network': 'net1', 'port': None, 'fixed_ip': 'ip1'},
-            {'network': 'net1', 'port': None, 'fixed_ip': None},
-            {'network': None, 'port': 'port3', 'fixed_ip': None}
+            {'floating_network': None, 'network': 'net1', 'fixed_ip': 'ip1',
+             'floating_ip': None, 'port': None, 'security_groups': None},
+            {'floating_network': None, 'network': 'net1', 'fixed_ip': None,
+             'floating_ip': None, 'port': None, 'security_groups': None},
+            {'floating_network': None, 'network': None, 'fixed_ip': None,
+             'floating_ip': None, 'port': 'port3', 'security_groups': None}
         ]
         mock_delete.assert_called_once_with(obj, networks_delete)
 
@@ -1185,7 +1055,7 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile.do_update,
                                obj, new_profile)
 
-        self.assertEqual('Failed in updating server NOVA_ID: BANG.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': BANG.",
                          six.text_type(ex))
         mock_check_name.assert_called_once_with(obj, new_profile)
         mock_check_password.assert_called_once_with(obj, new_profile)
@@ -1255,14 +1125,18 @@ class TestNovaServerUpdate(base.SenlinTestCase):
 
         mock_update_image.assert_called_with(
             obj, new_profile, 'OLD_NAME', 'OLD_PASS')
-        self.assertEqual('Failed in updating server NOVA_ID: Image Not Found.',
-                         six.text_type(ex))
+        self.assertEqual("Failed in updating server 'NOVA_ID': "
+                         "Image Not Found.", six.text_type(ex))
 
     @mock.patch.object(server.ServerProfile, '_update_flavor')
     def test_do_update_update_flavor_succeeded(self, mock_update_flavor):
         obj = mock.Mock(physical_id='FAKE_ID')
         profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = mock.Mock()
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
+        cc = mock.Mock()
+        cc.server_get.return_value = x_server
+        profile._computeclient = cc
         new_spec = copy.deepcopy(self.spec)
         new_spec['properties']['flavor'] = 'FAKE_FLAVOR_NEW'
         new_profile = server.ServerProfile('t', new_spec)
@@ -1278,7 +1152,11 @@ class TestNovaServerUpdate(base.SenlinTestCase):
         mock_update_flavor.side_effect = ex
         obj = mock.Mock(physical_id='NOVA_ID')
         profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = mock.Mock()
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
+        cc = mock.Mock()
+        cc.server_get.return_value = x_server
+        profile._computeclient = cc
         new_spec = copy.deepcopy(self.spec)
         new_spec['properties']['flavor'] = 'FAKE_FLAVOR_NEW'
         new_profile = server.ServerProfile('t', new_spec)
@@ -1288,8 +1166,8 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                obj, new_profile)
 
         mock_update_flavor.assert_called_with(obj, new_profile)
-        self.assertEqual('Failed in updating server NOVA_ID: '
-                         'Flavor Not Found.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': "
+                         "Flavor Not Found.",
                          six.text_type(ex))
 
     @mock.patch.object(server.ServerProfile, '_update_flavor')
@@ -1298,7 +1176,11 @@ class TestNovaServerUpdate(base.SenlinTestCase):
             self, mock_update_network, mock_update_flavor):
         mock_update_network.return_value = True
         profile = server.ServerProfile('t', self.spec)
-        profile._computeclient = mock.Mock()
+        x_image = {'id': '123'}
+        x_server = mock.Mock(image=x_image)
+        cc = mock.Mock()
+        cc.server_get.return_value = x_server
+        profile._computeclient = cc
 
         obj = mock.Mock(physical_id='NOVA_ID')
 
@@ -1347,7 +1229,7 @@ class TestNovaServerUpdate(base.SenlinTestCase):
                                profile.do_update,
                                obj, new_profile)
 
-        self.assertEqual('Failed in updating server NOVA_ID: BOOM.',
+        self.assertEqual("Failed in updating server 'NOVA_ID': BOOM.",
                          six.text_type(ex))
         mock_check_name.assert_called_once_with(obj, new_profile)
         mock_check_password.assert_called_once_with(obj, new_profile)
