@@ -35,6 +35,9 @@ spec = {
         "metadata": {"meta var": "meta val"},
         'name': 'FAKE_SERVER_NAME',
         'networks': [{
+            'floating_ip': 'FAKE_FLOATING_IP',
+            'floating_network': 'FAKE_FLOATING_NET',
+            'security_groups': ['FAKE_SECURITY_GROUP'],
             'port': 'FAKE_PORT',
             'fixed_ip': 'FAKE_IP',
             'network': 'FAKE_NET',
@@ -192,21 +195,21 @@ class TestFlavorValidation(base.SenlinTestCase):
             validate_result=exc.InternalError(message='BANG'),
             result='FID',
             exception=exc.EResourceUpdate,
-            message='Failed in updating server NOVA_ID: BANG.')),
+            message="Failed in updating server 'NOVA_ID': BANG.")),
         ('update:not_found', dict(
             reason='update',
             success=False,
             validate_result=exc.InternalError(code=404, message='BANG'),
             result='FID',
             exception=exc.EResourceUpdate,
-            message="Failed in updating server NOVA_ID: BANG.")),
+            message="Failed in updating server 'NOVA_ID': BANG.")),
         ('update:disabled', dict(
             reason='update',
             success=False,
             validate_result=[mock.Mock(id='FID', is_disabled=True)],
             result='FID',
             exception=exc.EResourceUpdate,
-            message=("Failed in updating server NOVA_ID: The specified "
+            message=("Failed in updating server 'NOVA_ID': The specified "
                      "flavor 'FLAVOR' is disabled.")))
     ]
 
@@ -293,14 +296,14 @@ class TestImageValidation(base.SenlinTestCase):
             validate_result=exc.InternalError(message='BANG'),
             result='FID',
             exception=exc.EResourceUpdate,
-            message='Failed in updating server NOVA_ID: BANG.')),
+            message="Failed in updating server 'NOVA_ID': BANG.")),
         ('update:not_found', dict(
             reason='update',
             success=False,
             validate_result=exc.InternalError(code=404, message='BANG'),
             result='FID',
             exception=exc.EResourceUpdate,
-            message="Failed in updating server NOVA_ID: BANG.")),
+            message="Failed in updating server 'NOVA_ID': BANG.")),
     ]
 
     def setUp(self):
@@ -328,6 +331,106 @@ class TestImageValidation(base.SenlinTestCase):
         self.cc.image_find.assert_called_once_with(image, False)
 
 
+class TestVolumeValidation(base.SenlinTestCase):
+
+    scenarios = [
+        ('validate:success', dict(
+            reason=None,
+            success=True,
+            validate_result=[mock.Mock(id='VOLUME_ID', status='available')],
+            result='VOLUME_ID',
+            exception=None,
+            message='')),
+        ('validate:failure', dict(
+            reason=None,
+            success=False,
+            validate_result=[mock.Mock(id='VOLUME_ID', status='in-use')],
+            result='VOLUME_ID',
+            exception=exc.InvalidSpec,
+            message="The volume VOLUME should be in 'available' "
+                    "status but is in 'in-use' status.")),
+        ('validate:driver_failure', dict(
+            reason=None,
+            success=False,
+            validate_result=exc.InternalError(message='BANG.'),
+            result='FID',
+            exception=exc.InternalError,
+            message='BANG.')),
+        ('validate:not_found', dict(
+            reason=None,
+            success=False,
+            validate_result=exc.InternalError(code=404, message='BANG.'),
+            result='FID',
+            exception=exc.InvalidSpec,
+            message="The specified volume 'VOLUME' could not be found.")),
+        ('create:success', dict(
+            reason='create',
+            success=True,
+            validate_result=[mock.Mock(id='VOLUME_ID', status='available')],
+            result='VOLUME_ID',
+            exception=None,
+            message='')),
+        ('create:driver_failure', dict(
+            reason='create',
+            success=False,
+            validate_result=exc.InternalError(message='BANG'),
+            result='FID',
+            exception=exc.EResourceCreation,
+            message='Failed in creating server: BANG.')),
+        ('create:not_found', dict(
+            reason='create',
+            success=False,
+            validate_result=exc.InternalError(code=404, message='BANG'),
+            result='FID',
+            exception=exc.EResourceCreation,
+            message="Failed in creating server: BANG.")),
+    ]
+
+    def setUp(self):
+        super(TestVolumeValidation, self).setUp()
+        bdm_v2 = [
+            {
+                'volume_size': 1,
+                'uuid': '6ce0be68',
+                'source_type': 'volume',
+                'destination_type': 'volume',
+                'boot_index': 0,
+            },
+        ]
+
+        volume_spec = {
+            'type': 'os.nova.server',
+            'version': '1.0',
+            'properties': {
+                'flavor': 'FLAV',
+                'name': 'FAKE_SERVER_NAME',
+                'security_groups': ['HIGH_SECURITY_GROUP'],
+                'block_device_mapping_v2': bdm_v2,
+            }
+        }
+
+        self.vc = mock.Mock()
+        self.profile = server.ServerProfile('t', volume_spec)
+        self.profile._block_storageclient = self.vc
+
+    def test_validation(self):
+        self.vc.volume_get.side_effect = self.validate_result
+        node = mock.Mock(id='NODE_ID', physical_id='NOVA_ID')
+        volume = 'VOLUME'
+
+        if self.success:
+            res = self.profile._validate_volume(node, volume, self.reason)
+            self.assertIsNotNone(res)
+            self.assertEqual(self.result, res.id)
+        else:
+            ex = self.assertRaises(self.exception,
+                                   self.profile._validate_volume,
+                                   node, volume, self.reason)
+            self.assertEqual(self.message, six.text_type(ex))
+
+        self.vc.volume_get.assert_called_once_with(volume)
+
+
 class TestKeypairValidation(base.SenlinTestCase):
 
     scenarios = [
@@ -386,14 +489,14 @@ class TestKeypairValidation(base.SenlinTestCase):
             validate_result=exc.InternalError(message='BANG'),
             result='FID',
             exception=exc.EResourceUpdate,
-            message='Failed in updating server NOVA_ID: BANG.')),
+            message="Failed in updating server 'NOVA_ID': BANG.")),
         ('update:not_found', dict(
             reason='update',
             success=False,
             validate_result=exc.InternalError(code=404, message='BANG'),
             result='FID',
             exception=exc.EResourceUpdate,
-            message="Failed in updating server NOVA_ID: BANG.")),
+            message="Failed in updating server 'NOVA_ID': BANG.")),
     ]
 
     def setUp(self):
@@ -424,30 +527,72 @@ class TestKeypairValidation(base.SenlinTestCase):
 class TestNetworkValidation(base.SenlinTestCase):
 
     scenarios = [
-        ('validate:net-y:port-y:fixed_ip-n', dict(
+        ('validate:net-n:port-n:fixed_ip-n:sgroups-n', dict(
             reason=None,
             success=True,
-            inputs={'network': 'NET', 'port': 'PORT'},
-            net_result=[mock.Mock(id='NET_ID')],
+            inputs={'port': 'PORT'},
+            net_result=[],
             port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
-            result={'uuid': 'NET_ID', 'port': 'PORT_ID'},
+            sg_result=[],
+            floating_result=[],
+            result={'port': 'PORT_ID'},
             exception=None,
             message='')),
-        ('validate:net-y:port-n:fixed_ip-y', dict(
+        ('validate:net-y:port-n:fixed_ip-n:sgroups-y', dict(
+            reason=None,
+            success=True,
+            inputs={'network': 'NET', 'security_groups': ['default']},
+            net_result=[mock.Mock(id='NET_ID')],
+            port_result=[],
+            sg_result=[mock.Mock(id='SG_ID')],
+            floating_result=[],
+            result={'network': 'NET_ID', 'security_groups': ['SG_ID']},
+            exception=None,
+            message='')),
+        ('validate:net-y:port-n:fixed_ip-n:sgroups-n:floating_net-y', dict(
+            reason=None,
+            success=True,
+            inputs={'network': 'NET', 'floating_network': 'NET'},
+            net_result=[mock.Mock(id='NET_ID'), mock.Mock(id='NET_ID')],
+            port_result=[],
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID', 'floating_network': 'NET_ID'},
+            exception=None,
+            message='')),
+        ('validate:net-y:port-n:fixed_ip-n:floating_net-y:floating_ip-y', dict(
+            reason=None,
+            success=True,
+            inputs={'network': 'NET', 'floating_network': 'NET',
+                    'floating_ip': 'FLOATINGIP'},
+            net_result=[mock.Mock(id='NET_ID'), mock.Mock(id='NET_ID')],
+            port_result=[],
+            sg_result=[],
+            floating_result=[mock.Mock(id='FLOATINGIP_ID', status='INACTIVE')],
+            result={'network': 'NET_ID', 'floating_network': 'NET_ID',
+                    'floating_ip_id': 'FLOATINGIP_ID',
+                    'floating_ip': 'FLOATINGIP'},
+            exception=None,
+            message='')),
+        ('validate:net-y:port-n:fixed_ip-y:sgroups-n', dict(
             reason=None,
             success=True,
             inputs={'network': 'NET', 'fixed_ip': 'FIXED_IP'},
             net_result=[mock.Mock(id='NET_ID')],
             port_result=[],
-            result={'uuid': 'NET_ID', 'fixed_ip': 'FIXED_IP'},
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID', 'fixed_ip': 'FIXED_IP'},
             exception=None,
             message='')),
-        ('validate:net-f:port-y:fixed_ip-n', dict(
+        ('validate:net-f:port-y:fixed_ip-n:sgroups-n', dict(
             reason=None,
             success=False,
             inputs={'network': 'NET', 'port': 'PORT'},
             net_result=[exc.InternalError(message='NET Failure')],
             port_result=[],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.InvalidSpec,
             message='NET Failure')),
@@ -457,6 +602,8 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'port': 'PORT'},
             net_result=[],
             port_result=[exc.InternalError(message='PORT Failure')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.InvalidSpec,
             message='PORT Failure')),
@@ -466,24 +613,53 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'port': 'PORT'},
             net_result=[],
             port_result=[mock.Mock(id='PORT_ID', status='ACTIVE')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.InvalidSpec,
             message='The status of the port PORT must be DOWN')),
+        ('validate:net-n:port-y:fixed_ip-n:floating_net-n:floating_ip-y', dict(
+            reason=None,
+            success=False,
+            inputs={'port': 'PORT', 'floating_ip': 'FLOATINGIP'},
+            net_result=[],
+            port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
+            sg_result=[],
+            floating_result=[mock.Mock(id='FLOATINGIP_ID', status='INACTIVE')],
+            result={},
+            exception=exc.InvalidSpec,
+            message='Must specify a network to create floating IP')),
+        ('validate:net-n:port-y:fixed_ip-n:floating_ip-active', dict(
+            reason=None,
+            success=False,
+            inputs={'port': 'PORT', 'floating_network': 'NET',
+                    'floating_ip': 'FLOATINGIP'},
+            net_result=[mock.Mock(id='NET_ID')],
+            port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
+            sg_result=[],
+            floating_result=[mock.Mock(id='FLOATINGIP_ID', status='ACTIVE')],
+            result={},
+            exception=exc.InvalidSpec,
+            message='the floating IP FLOATINGIP has been used.')),
         ('validate:net-n:port-n:fixed_ip-n', dict(
             reason=None,
             success=False,
             inputs={'fixed_ip': 'FIXED_IP'},
             net_result=[],
             port_result=[],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.InvalidSpec,
-            message="'port' is required if 'network' is omitted")),
+            message="One of 'port' and 'network' must be provided")),
         ('validate:net-n:port-y:fixed_ip-y', dict(
             reason=None,
             success=False,
             inputs={'port': 'PORT', 'fixed_ip': 'FIXED_IP'},
             net_result=[],
             port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.InvalidSpec,
             message=("The 'port' property and the 'fixed_ip' property cannot "
@@ -494,7 +670,9 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'network': 'NET', 'port': 'PORT'},
             net_result=[mock.Mock(id='NET_ID')],
             port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
-            result={'uuid': 'NET_ID', 'port': 'PORT_ID'},
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID', 'port': 'PORT_ID'},
             exception=None,
             message='')),
         ('create:net-y:port-n:fixed_ip-y', dict(
@@ -503,7 +681,31 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'network': 'NET', 'fixed_ip': 'FIXED_IP'},
             net_result=[mock.Mock(id='NET_ID')],
             port_result=[],
-            result={'uuid': 'NET_ID', 'fixed_ip': 'FIXED_IP'},
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID', 'fixed_ip': 'FIXED_IP'},
+            exception=None,
+            message='')),
+        ('create:net-y:port-n:fixed_ip-n:sgroups-y', dict(
+            reason='create',
+            success=True,
+            inputs={'network': 'NET', 'security_groups': ['default']},
+            net_result=[mock.Mock(id='NET_ID')],
+            port_result=[],
+            sg_result=[mock.Mock(id='SG_ID')],
+            floating_result=[],
+            result={'network': 'NET_ID', 'security_groups': ['SG_ID']},
+            exception=None,
+            message='')),
+        ('create:net-y:port-n:fixed_ip-n:sgroups-n:floating_net-y', dict(
+            reason=None,
+            success=True,
+            inputs={'network': 'NET', 'floating_network': 'NET'},
+            net_result=[mock.Mock(id='NET_ID'), mock.Mock(id='NET_ID')],
+            port_result=[],
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID', 'floating_network': 'NET_ID'},
             exception=None,
             message='')),
         ('create:net-f:port-y:fixed_ip-n', dict(
@@ -512,6 +714,8 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'network': 'NET', 'port': 'PORT'},
             net_result=[exc.InternalError(message='NET Failure')],
             port_result=[],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceCreation,
             message='Failed in creating server: NET Failure.')),
@@ -521,6 +725,8 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'port': 'PORT'},
             net_result=[],
             port_result=[exc.InternalError(message='PORT Failure')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceCreation,
             message='Failed in creating server: PORT Failure.')),
@@ -530,6 +736,8 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'port': 'PORT'},
             net_result=[],
             port_result=[mock.Mock(id='PORT_ID', status='ACTIVE')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceCreation,
             message=('Failed in creating server: The status of the port PORT '
@@ -540,16 +748,20 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'fixed_ip': 'FIXED_IP'},
             net_result=[],
             port_result=[],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceCreation,
-            message=("Failed in creating server: 'port' is required if "
-                     "'network' is omitted."))),
+            message=("Failed in creating server: One of 'port' "
+                     "and 'network' must be provided."))),
         ('create:net-n:port-y:fixed_ip-y', dict(
             reason='create',
             success=False,
             inputs={'port': 'PORT', 'fixed_ip': 'FIXED_IP'},
             net_result=[],
             port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceCreation,
             message=("Failed in creating server: The 'port' property and the "
@@ -561,7 +773,9 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'network': 'NET', 'port': 'PORT'},
             net_result=[mock.Mock(id='NET_ID')],
             port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
-            result={'net_id': 'NET_ID', 'port_id': 'PORT_ID'},
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID', 'port': 'PORT_ID'},
             exception=None,
             message='')),
         ('update:net-y:port-n:fixed_ip-y', dict(
@@ -570,8 +784,32 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'network': 'NET', 'fixed_ip': 'FIXED_IP'},
             net_result=[mock.Mock(id='NET_ID')],
             port_result=[],
-            result={'net_id': 'NET_ID',
-                    'fixed_ips': [{'ip_address': 'FIXED_IP'}]},
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID',
+                    'fixed_ip': 'FIXED_IP'},
+            exception=None,
+            message='')),
+        ('update:net-y:port-n:fixed_ip-n:sgroups-y', dict(
+            reason='create',
+            success=True,
+            inputs={'network': 'NET', 'security_groups': ['default']},
+            net_result=[mock.Mock(id='NET_ID')],
+            port_result=[],
+            sg_result=[mock.Mock(id='SG_ID')],
+            floating_result=[],
+            result={'network': 'NET_ID', 'security_groups': ['SG_ID']},
+            exception=None,
+            message='')),
+        ('update:net-y:port-n:fixed_ip-n:sgroups-n:floating_net-y', dict(
+            reason=None,
+            success=True,
+            inputs={'network': 'NET', 'floating_network': 'NET'},
+            net_result=[mock.Mock(id='NET_ID'), mock.Mock(id='NET_ID')],
+            port_result=[],
+            sg_result=[],
+            floating_result=[],
+            result={'network': 'NET_ID', 'floating_network': 'NET_ID'},
             exception=None,
             message='')),
         ('update:net-f:port-y:fixed_ip-n', dict(
@@ -580,49 +818,59 @@ class TestNetworkValidation(base.SenlinTestCase):
             inputs={'network': 'NET', 'port': 'PORT'},
             net_result=[exc.InternalError(message='NET Failure')],
             port_result=[],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceUpdate,
-            message='Failed in updating server NOVA_ID: NET Failure.')),
+            message="Failed in updating server 'NOVA_ID': NET Failure.")),
         ('update:net-n:port-f:fixed_ip-n', dict(
             reason='update',
             success=False,
             inputs={'port': 'PORT'},
             net_result=[],
             port_result=[exc.InternalError(message='PORT Failure')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceUpdate,
-            message='Failed in updating server NOVA_ID: PORT Failure.')),
+            message="Failed in updating server 'NOVA_ID': PORT Failure.")),
         ('update:net-n:port-active:fixed_ip-n', dict(
             reason='update',
             success=False,
             inputs={'port': 'PORT'},
             net_result=[],
             port_result=[mock.Mock(id='PORT_ID', status='ACTIVE')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceUpdate,
-            message=('Failed in updating server NOVA_ID: The status of the '
-                     'port PORT must be DOWN.'))),
+            message=("Failed in updating server 'NOVA_ID': The status of the "
+                     "port PORT must be DOWN."))),
         ('update:net-n:port-n:fixed_ip-n', dict(
             reason='update',
             success=False,
             inputs={'fixed_ip': 'FIXED_IP'},
             net_result=[],
             port_result=[],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceUpdate,
-            message=("Failed in updating server NOVA_ID: 'port' is required "
-                     "if 'network' is omitted."))),
+            message=("Failed in updating server 'NOVA_ID': One of 'port' "
+                     "and 'network' must be provided."))),
         ('update:net-n:port-y:fixed_ip-y', dict(
             reason='update',
             success=False,
             inputs={'port': 'PORT', 'fixed_ip': 'FIXED_IP'},
             net_result=[],
             port_result=[mock.Mock(id='PORT_ID', status='DOWN')],
+            sg_result=[],
+            floating_result=[],
             result={},
             exception=exc.EResourceUpdate,
-            message=("Failed in updating server NOVA_ID: The 'port' property "
-                     "and the 'fixed_ip' property cannot be specified at the "
-                     "same time."))),
+            message=("Failed in updating server 'NOVA_ID': The 'port' "
+                     "property and the 'fixed_ip' property cannot be "
+                     "specified at the same time."))),
     ]
 
     def setUp(self):
@@ -635,6 +883,8 @@ class TestNetworkValidation(base.SenlinTestCase):
     def test_validation(self):
         self.nc.network_get.side_effect = self.net_result
         self.nc.port_find.side_effect = self.port_result
+        self.nc.security_group_find.side_effect = self.sg_result
+        self.nc.floatingip_find.side_effect = self.floating_result
         obj = mock.Mock(physical_id='NOVA_ID')
 
         if self.success:
@@ -647,9 +897,13 @@ class TestNetworkValidation(base.SenlinTestCase):
             self.assertEqual(self.message, six.text_type(ex))
 
         if self.net_result:
-            self.nc.network_get.assert_called_once_with('NET')
+            self.nc.network_get.assert_called_with('NET')
         if self.port_result:
             self.nc.port_find.assert_called_once_with('PORT')
+        if self.sg_result:
+            self.nc.security_group_find.assert_called_once_with('default')
+        if self.floating_result:
+            self.nc.floatingip_find.assert_called_once_with('FLOATINGIP')
 
 
 class TestNovaServerValidate(base.SenlinTestCase):
