@@ -11,7 +11,6 @@
 # under the License.
 
 import copy
-from types import GeneratorType
 
 import mock
 from oslo_context import context as oslo_ctx
@@ -21,10 +20,12 @@ from senlin.common import context as senlin_ctx
 from senlin.common import exception
 from senlin.common import schema
 from senlin.common import utils as common_utils
-from senlin.db import api as db_api
 from senlin.engine import environment
 from senlin.engine import parser
+from senlin.objects import credential as co
+from senlin.objects import profile as po
 from senlin.profiles import base as pb
+from senlin.profiles.os.nova import server as nova_server
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -61,6 +62,16 @@ class DummyProfile(pb.Profile):
             'third key',
         ),
     }
+    OPERATIONS = {
+        'op1': schema.Operation(
+            'Operation 1',
+            schema={
+                'param1': schema.StringParam(
+                    'description of param1',
+                )
+            }
+        )
+    }
 
     def __init__(self, name, spec, **kwargs):
         super(DummyProfile, self).__init__(name, spec, **kwargs)
@@ -77,9 +88,9 @@ class TestProfileBase(base.SenlinTestCase):
 
     def _create_profile(self, name, pid=None, context=None):
         profile = pb.Profile(name, self.spec,
-                             user=self.ctx.user,
-                             project=self.ctx.project,
-                             domain=self.ctx.domain,
+                             user=self.ctx.user_id,
+                             project=self.ctx.project_id,
+                             domain=self.ctx.domain_id,
                              context=context)
         if pid:
             profile.id = pid
@@ -87,18 +98,20 @@ class TestProfileBase(base.SenlinTestCase):
 
         return profile
 
-    @mock.patch.object(senlin_ctx, 'get_service_context')
-    def test_init(self, mock_ctx):
-        mock_ctx.return_value = {'foo': 'bar'}
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_init(self, mock_creds):
+        mock_creds.return_value = {'foo': 'bar'}
         profile = self._create_profile('test-profile')
 
         self.assertIsNone(profile.id)
         self.assertEqual('test-profile', profile.name)
         self.assertEqual(self.spec, profile.spec)
+        self.assertEqual('os.dummy', profile.type_name)
+        self.assertEqual('1.0', profile.version)
         self.assertEqual('os.dummy-1.0', profile.type)
-        self.assertEqual(self.ctx.user, profile.user)
-        self.assertEqual(self.ctx.project, profile.project)
-        self.assertEqual(self.ctx.domain, profile.domain)
+        self.assertEqual(self.ctx.user_id, profile.user)
+        self.assertEqual(self.ctx.project_id, profile.project)
+        self.assertEqual(self.ctx.domain_id, profile.domain)
         self.assertEqual({}, profile.metadata)
         self.assertIsNone(profile.created_at)
         self.assertIsNone(profile.updated_at)
@@ -112,9 +125,14 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertEqual(2, profile.properties['key2'])
         self.assertEqual({'foo': 'bar'}, profile.context)
 
-    @mock.patch.object(senlin_ctx, 'get_service_context')
-    def test_init_with_context(self, mock_ctx):
-        mock_ctx.return_value = {'foo': 'bar'}
+        self.assertIsNone(profile._computeclient)
+        self.assertIsNone(profile._networkclient)
+        self.assertIsNone(profile._orchestrationclient)
+        self.assertIsNone(profile._block_storageclient)
+
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_init_with_context(self, mock_creds):
+        mock_creds.return_value = {'foo': 'bar'}
         profile = self._create_profile('test-profile',
                                        pid='FAKE_ID', context={'bar': 'foo'})
         self.assertEqual({'bar': 'foo'}, profile.context)
@@ -126,7 +144,7 @@ class TestProfileBase(base.SenlinTestCase):
             'properties': '',
         }
 
-        self.assertRaises(exception.ProfileTypeNotFound,
+        self.assertRaises(exception.ResourceNotFound,
                           pb.Profile,
                           'test-profile', bad_spec)
 
@@ -134,17 +152,17 @@ class TestProfileBase(base.SenlinTestCase):
         bad_spec = copy.deepcopy(self.spec)
         del bad_spec['version']
 
-        ex = self.assertRaises(exception.SpecValidationFailed,
+        ex = self.assertRaises(exception.ESchema,
                                pb.Profile, 'test-profile', bad_spec)
         self.assertEqual("The 'version' key is missing from the provided "
                          "spec map.", six.text_type(ex))
 
-    def test_from_db_record(self):
+    def test_from_object(self):
         obj = self._create_profile('test_profile_for_record')
         obj.store(self.ctx)
-        profile = db_api.profile_get(self.ctx, obj.id)
+        profile = po.Profile.get(self.ctx, obj.id)
 
-        result = pb.Profile.from_db_record(profile)
+        result = pb.Profile._from_object(profile)
 
         self.assertEqual(profile.id, result.id)
         self.assertEqual(profile.name, result.name)
@@ -153,7 +171,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertEqual(profile.project, result.project)
         self.assertEqual(profile.domain, result.domain)
         self.assertEqual(profile.spec, result.spec)
-        self.assertEqual(profile.meta_data, result.metadata)
+        self.assertEqual(profile.metadata, result.metadata)
         self.assertEqual('value1', result.properties['key1'])
         self.assertEqual(2, result.properties['key2'])
 
@@ -161,10 +179,10 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertEqual(profile.updated_at, result.updated_at)
         self.assertEqual(profile.context, result.context)
 
-    def test_load_with_profile_object(self):
+    def test_load_with_poect(self):
         obj = self._create_profile('test-profile-bb')
         profile_id = obj.store(self.ctx)
-        profile = db_api.profile_get(self.ctx, profile_id)
+        profile = po.Profile.get(self.ctx, profile_id)
 
         result = pb.Profile.load(self.ctx, profile=profile)
 
@@ -181,91 +199,79 @@ class TestProfileBase(base.SenlinTestCase):
     def test_load_with_both(self):
         profile = self._create_profile('test1')
         profile.store(self.ctx)
-        db_profile = db_api.profile_get(self.ctx, profile.id)
+        db_profile = po.Profile.get(self.ctx, profile.id)
 
         res = pb.Profile.load(self.ctx, profile=db_profile,
                               profile_id=profile.id)
 
         self.assertEqual(profile.id, res.id)
 
-    @mock.patch.object(db_api, 'profile_get')
+    @mock.patch.object(po.Profile, 'get')
     def test_load_not_found(self, mock_get):
         mock_get.return_value = None
-        self.assertRaises(exception.ProfileNotFound,
+        self.assertRaises(exception.ResourceNotFound,
                           pb.Profile.load,
                           self.ctx, profile_id='FAKE_ID')
         mock_get.assert_called_once_with(self.ctx, 'FAKE_ID',
                                          project_safe=True)
 
-    @mock.patch.object(db_api, 'profile_get_all')
-    def test_load_all_empty(self, mock_get):
-        mock_get.return_value = []
-        res = pb.Profile.load_all(self.ctx)
-        self.assertIsInstance(res, GeneratorType)
-        value = [v for v in res]
-        self.assertEqual([], value)
-        mock_get.assert_called_once_with(self.ctx, limit=None, marker=None,
-                                         sort=None, filters=None,
-                                         project_safe=True)
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_create(self, mock_creds):
+        mock_creds.return_value = {}
+        res = pb.Profile.create(self.ctx, 'my_profile', self.spec)
 
-    @mock.patch.object(db_api, 'profile_get_all')
-    @mock.patch.object(pb.Profile, 'from_db_record')
-    def test_load_all_with_results(self, mock_from, mock_get):
-        dbobj = mock.Mock()
-        mock_get.return_value = [dbobj]
-        obj = mock.Mock()
-        mock_from.return_value = obj
-        res = pb.Profile.load_all(self.ctx)
-        self.assertIsInstance(res, GeneratorType)
-        value = [v for v in res]
-        self.assertEqual([obj], value)
-        mock_get.assert_called_once_with(self.ctx, limit=None, marker=None,
-                                         sort=None, filters=None,
-                                         project_safe=True)
-        mock_from.assert_called_once_with(dbobj)
+        self.assertIsInstance(res, pb.Profile)
 
-    @mock.patch.object(db_api, 'profile_get_all')
-    @mock.patch.object(pb.Profile, 'from_db_record')
-    def test_load_all_with_params(self, mock_from, mock_get):
-        dbobj = mock.Mock()
-        mock_get.return_value = [dbobj]
-        obj = mock.Mock()
-        mock_from.return_value = obj
-        res = pb.Profile.load_all(self.ctx, limit=123, marker='MARKER',
-                                  sort='FOOKEY', filters='BARDICT',
-                                  project_safe=False)
-        self.assertIsInstance(res, GeneratorType)
-        value = [v for v in res]
-        self.assertEqual([obj], value)
-        mock_get.assert_called_once_with(self.ctx, limit=123, marker='MARKER',
-                                         sort='FOOKEY', filters='BARDICT',
-                                         project_safe=False)
-        mock_from.assert_called_once_with(dbobj)
+        obj = po.Profile.get(self.ctx, res.id)
+        self.assertEqual('my_profile', obj.name)
 
-    @mock.patch.object(db_api, 'profile_delete')
+    def test_create_profile_type_not_found(self):
+        spec = copy.deepcopy(self.spec)
+        spec['type'] = "bogus"
+        ex = self.assertRaises(exception.InvalidSpec,
+                               pb.Profile.create,
+                               self.ctx, 'my_profile', spec)
+
+        self.assertEqual("Failed in creating profile my_profile: The "
+                         "profile_type 'bogus-1.0' could not be found.",
+                         six.text_type(ex))
+
+    @mock.patch.object(pb.Profile, 'validate')
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_create_failed_validation(self, mock_creds, mock_validate):
+        mock_creds.return_value = {}
+        mock_validate.side_effect = exception.ESchema(message="Boom")
+
+        ex = self.assertRaises(exception.InvalidSpec,
+                               pb.Profile.create,
+                               self.ctx, 'my_profile', self.spec)
+
+        self.assertEqual("Failed in creating profile my_profile: "
+                         "Boom", six.text_type(ex))
+
+    @mock.patch.object(po.Profile, 'delete')
     def test_delete(self, mock_delete):
         res = pb.Profile.delete(self.ctx, 'FAKE_ID')
         self.assertIsNone(res)
         mock_delete.assert_called_once_with(self.ctx, 'FAKE_ID')
 
-    @mock.patch.object(db_api, 'profile_delete')
+    @mock.patch.object(po.Profile, 'delete')
     def test_delete_busy(self, mock_delete):
-        err = exception.ResourceBusyError(resource_type='profile',
-                                          resource_id='FAKE_ID')
+        err = exception.EResourceBusy(type='profile', id='FAKE_ID')
         mock_delete.side_effect = err
-        self.assertRaises(exception.ResourceBusyError,
+        self.assertRaises(exception.EResourceBusy,
                           pb.Profile.delete,
                           self.ctx, 'FAKE_ID')
         mock_delete.assert_called_once_with(self.ctx, 'FAKE_ID')
 
-    @mock.patch.object(db_api, 'profile_delete')
+    @mock.patch.object(po.Profile, 'delete')
     def test_delete_not_found(self, mock_delete):
         mock_delete.return_value = None
         result = pb.Profile.delete(self.ctx, 'BOGUS')
         self.assertIsNone(result)
         mock_delete.assert_called_once_with(self.ctx, 'BOGUS')
 
-    @mock.patch.object(db_api, 'profile_create')
+    @mock.patch.object(po.Profile, 'create')
     def test_store_for_create(self, mock_create):
         profile = self._create_profile('test-profile')
         self.assertIsNone(profile.id)
@@ -292,7 +298,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertEqual('FAKE_ID', profile_id)
         self.assertIsNotNone(profile.created_at)
 
-    @mock.patch.object(db_api, 'profile_update')
+    @mock.patch.object(po.Profile, 'update')
     def test_store_for_update(self, mock_update):
         profile = self._create_profile('test-profile')
         self.assertIsNone(profile.id)
@@ -361,6 +367,18 @@ class TestProfileBase(base.SenlinTestCase):
         res_obj = profile.do_delete.return_value
         self.assertEqual(res_obj, res)
 
+    @mock.patch.object(pb.Profile, 'load')
+    def test_check_object_exception_return_value(self, mock_load):
+        profile = pb.Profile
+        profile.load(self.ctx).do_check = mock.Mock(
+            side_effect=exception.InternalError(code=400, message='BAD'))
+        obj = mock_load
+
+        res = profile.check_object(self.ctx, obj)
+
+        profile.load(self.ctx).do_check.assert_called_once_with(obj)
+        self.assertFalse(res)
+
     @mock.patch.object(pb.Profile, 'load')
     def test_update_object_with_profile(self, mock_load):
         old_profile = mock.Mock()
@@ -431,7 +449,6 @@ class TestProfileBase(base.SenlinTestCase):
         expected = {
             'context': {
                 'description': 'context data',
-                'readonly': False,
                 'required': False,
                 'updatable': False,
                 'type': 'Map'
@@ -439,21 +456,18 @@ class TestProfileBase(base.SenlinTestCase):
             'key1': {
                 'default': 'value1',
                 'description': 'first key',
-                'readonly': False,
                 'required': False,
                 'updatable': True,
                 'type': 'String',
             },
             'key2': {
                 'description': 'second key',
-                'readonly': False,
                 'required': True,
                 'updatable': True,
                 'type': 'Integer'
             },
             'key3': {
                 'description': 'third key',
-                'readonly': False,
                 'required': False,
                 'updatable': False,
                 'type': 'String'
@@ -463,6 +477,34 @@ class TestProfileBase(base.SenlinTestCase):
         actual = DummyProfile.get_schema()
         self.assertEqual(expected, actual)
 
+    def test_get_ops(self):
+        expected = {
+            'op1': {
+                'description': 'Operation 1',
+                'parameters': {
+                    'param1': {
+                        'type': 'String',
+                        'required': False,
+                        'description': 'description of param1',
+                    }
+                }
+            },
+        }
+
+        actual = DummyProfile.get_ops()
+        self.assertEqual(expected, actual)
+
+    @mock.patch.object(nova_server.ServerProfile, 'do_adopt')
+    def test_adopt_node(self, mock_adopt):
+        obj = mock.Mock()
+
+        res = pb.Profile.adopt_node(self.ctx, obj, "os.nova.server-1.0",
+                                    overrides=None, snapshot=False)
+
+        mock_adopt.assert_called_once_with(obj, overrides=None, snapshot=False)
+        res_obj = mock_adopt.return_value
+        self.assertEqual(res_obj, res)
+
     @mock.patch.object(pb.Profile, 'load')
     def test_join_cluster(self, mock_load):
         profile = mock.Mock()
@@ -491,21 +533,46 @@ class TestProfileBase(base.SenlinTestCase):
         res_obj = profile.do_leave.return_value
         self.assertEqual(res_obj, res)
 
-    def test_validate(self):
+    def test_validate_without_properties(self):
         profile = self._create_profile('test_profile')
 
-        profile.spec_data = mock.Mock()
-        profile.properties = mock.Mock()
+        profile.do_validate = mock.Mock()
 
         profile.validate()
 
-        profile.spec_data.validate.assert_called_once_with()
-        profile.properties.validate.assert_called_once_with()
+        profile.do_validate.assert_not_called()
 
-    @mock.patch.object(senlin_ctx, 'get_service_context')
-    def test__init_context(self, mock_get):
+    def test_validate_with_properties(self):
+        profile = self._create_profile('test_profile')
+
+        profile.do_validate = mock.Mock()
+
+        profile.validate(validate_props=True)
+
+        profile.do_validate.assert_called_once_with(obj=profile)
+
+    def test_validate_bad_context(self):
+        spec = {
+            "type": "os.dummy",
+            "version": "1.0",
+            "properties": {
+                "context": {
+                    "foo": "bar"
+                },
+                "key1": "value1",
+                "key2": 2,
+            }
+        }
+        profile = DummyProfile("p-bad-ctx", spec, user=self.ctx.user_id,
+                               project=self.ctx.project_id,
+                               domain=self.ctx.domain_id)
+
+        self.assertRaises(exception.ESchema, profile.validate)
+
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test__init_context(self, mock_creds):
         fake_ctx = mock.Mock()
-        mock_get.return_value = fake_ctx
+        mock_creds.return_value = fake_ctx
 
         # _init_context() is called from __init__
         self._create_profile('test-profile')
@@ -515,10 +582,10 @@ class TestProfileBase(base.SenlinTestCase):
             mock.call('project_name', None),
             mock.call('project_domain_name', None),
         ])
-        mock_get.assert_called_once_with()
+        mock_creds.assert_called_once_with()
 
-    @mock.patch.object(senlin_ctx, 'get_service_context')
-    def test__init_context_for_real(self, mock_get):
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test__init_context_for_real(self, mock_creds):
         fake_ctx = {
             'project_name': 'this project',
             'project_domain_name': 'this domain',
@@ -526,12 +593,12 @@ class TestProfileBase(base.SenlinTestCase):
             'user_id': 'fake_user',
             'foo': 'bar',
         }
-        mock_get.return_value = fake_ctx
+        mock_creds.return_value = fake_ctx
 
         # _init_context() is called from __init__
         profile = self._create_profile('test-profile')
 
-        mock_get.assert_called_once_with()
+        mock_creds.assert_called_once_with()
         expected = {
             'auth_url': 'some url',
             'user_id': 'fake_user',
@@ -539,8 +606,8 @@ class TestProfileBase(base.SenlinTestCase):
         }
         self.assertEqual(expected, profile.context)
 
-    @mock.patch.object(senlin_ctx, 'get_service_context')
-    def test__init_context_for_real_with_data(self, mock_get):
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test__init_context_for_real_with_data(self, mock_creds):
         fake_ctx = {
             'project_name': 'this project',
             'project_domain_name': 'this domain',
@@ -548,7 +615,7 @@ class TestProfileBase(base.SenlinTestCase):
             'user_id': 'fake_user',
             'foo': 'bar',
         }
-        mock_get.return_value = fake_ctx
+        mock_creds.return_value = fake_ctx
         self.spec['properties']['context'] = {
             'region_name': 'region_dist'
         }
@@ -556,7 +623,7 @@ class TestProfileBase(base.SenlinTestCase):
         # _init_context() is called from __init__
         profile = self._create_profile('test-profile')
 
-        mock_get.assert_called_once_with(region_name='region_dist')
+        mock_creds.assert_called_once_with(region_name='region_dist')
         expected = {
             'auth_url': 'some url',
             'user_id': 'fake_user',
@@ -564,7 +631,7 @@ class TestProfileBase(base.SenlinTestCase):
         }
         self.assertEqual(expected, profile.context)
 
-    @mock.patch.object(db_api, 'cred_get')
+    @mock.patch.object(co.Credential, 'get')
     @mock.patch.object(oslo_ctx, 'get_current')
     def test__build_conn_params(self, mock_current, mock_get):
         profile = self._create_profile('test-profile')
@@ -586,7 +653,7 @@ class TestProfileBase(base.SenlinTestCase):
         mock_current.assert_called_once_with()
         mock_get.assert_called_once_with(fake_ctx, 'FAKE_USER', 'FAKE_PROJECT')
 
-    @mock.patch.object(db_api, 'cred_get')
+    @mock.patch.object(co.Credential, 'get')
     @mock.patch.object(oslo_ctx, 'get_current')
     def test__build_conn_params_trust_not_found(self, mock_current, mock_get):
         profile = self._create_profile('test-profile')
@@ -601,66 +668,206 @@ class TestProfileBase(base.SenlinTestCase):
         mock_current.assert_called_once_with()
         mock_get.assert_called_once_with(fake_ctx, 'FAKE_USER', 'FAKE_PROJECT')
 
+    @mock.patch.object(pb.Profile, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_compute(self, mock_senlindriver, mock_params):
+        obj = mock.Mock()
+        sd = mock.Mock()
+        cc = mock.Mock()
+        sd.compute.return_value = cc
+        mock_senlindriver.return_value = sd
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        profile = self._create_profile('test-profile')
+
+        res = profile.compute(obj)
+
+        self.assertEqual(cc, res)
+        self.assertEqual(cc, profile._computeclient)
+        mock_params.assert_called_once_with(obj.user, obj.project)
+        sd.compute.assert_called_once_with(fake_params)
+
+    def test_compute_with_cache(self):
+        cc = mock.Mock()
+        profile = self._create_profile('test-profile')
+        profile._computeclient = cc
+
+        res = profile.compute(mock.Mock())
+
+        self.assertEqual(cc, res)
+
+    @mock.patch.object(pb.Profile, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_neutron_client(self, mock_senlindriver, mock_params):
+        obj = mock.Mock()
+        sd = mock.Mock()
+        nc = mock.Mock()
+        sd.network.return_value = nc
+        mock_senlindriver.return_value = sd
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        profile = self._create_profile('test-profile')
+
+        res = profile.network(obj)
+
+        self.assertEqual(nc, res)
+        self.assertEqual(nc, profile._networkclient)
+        mock_params.assert_called_once_with(obj.user, obj.project)
+        sd.network.assert_called_once_with(fake_params)
+
+    @mock.patch.object(pb.Profile, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_cinder_client(self, mock_senlindriver, mock_params):
+        obj = mock.Mock()
+        sd = mock.Mock()
+        nc = mock.Mock()
+        sd.block_storage.return_value = nc
+        mock_senlindriver.return_value = sd
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        profile = self._create_profile('test-profile')
+
+        res = profile.block_storage(obj)
+
+        self.assertEqual(nc, res)
+        self.assertEqual(nc, profile._block_storageclient)
+        mock_params.assert_called_once_with(obj.user, obj.project)
+        sd.block_storage.assert_called_once_with(fake_params)
+
     def test_interface_methods(self):
         profile = self._create_profile('test-profile')
 
-        self.assertEqual(NotImplemented, profile.do_create(mock.Mock()))
-        self.assertEqual(NotImplemented, profile.do_delete(mock.Mock()))
-        self.assertEqual(NotImplemented,
-                         profile.do_update(mock.Mock(), mock.Mock()))
-        self.assertEqual(NotImplemented, profile.do_check(mock.Mock()))
-        self.assertEqual(NotImplemented, profile.do_get_details(mock.Mock()))
-        self.assertEqual(True,
-                         profile.do_join(mock.Mock(), mock.Mock()))
-        self.assertEqual(True, profile.do_leave(mock.Mock()))
-        self.assertEqual(NotImplemented, profile.do_rebuild(mock.Mock()))
+        self.assertRaises(NotImplementedError, profile.do_create, mock.Mock())
+        self.assertRaises(NotImplementedError, profile.do_delete, mock.Mock())
+        self.assertTrue(profile.do_update(mock.Mock(), mock.Mock()))
+        self.assertTrue(profile.do_check(mock.Mock()))
+        self.assertEqual({}, profile.do_get_details(mock.Mock()))
+        self.assertTrue(profile.do_join(mock.Mock(), mock.Mock()))
+        self.assertTrue(profile.do_leave(mock.Mock()))
+        self.assertTrue(profile.do_validate(mock.Mock()))
 
     def test_do_recover_default(self):
         profile = self._create_profile('test-profile')
-        self.assertEqual(NotImplemented, profile.do_recover(mock.Mock()))
-        self.assertEqual(NotImplemented,
-                         profile.do_recover(mock.Mock(), foo='bar'))
-        self.assertEqual(NotImplemented,
-                         profile.do_recover(mock.Mock(), operation='bar'))
+        self.patchobject(profile, 'do_create', return_value=True)
+        self.patchobject(profile, 'do_delete', return_value=True)
+
+        res, status = profile.do_recover(mock.Mock())
+        self.assertTrue(status)
+
+        res, status = profile.do_recover(
+            mock.Mock(), operation=[{'name': 'bar'}])
+        self.assertFalse(status)
+
+    def test_do_recover_with_fencing(self):
+        profile = self._create_profile('test-profile')
+        self.patchobject(profile, 'do_create', return_value=True)
+        self.patchobject(profile, 'do_delete', return_value=True)
+        obj = mock.Mock()
+
+        res = profile.do_recover(obj, ignore_missing=True,
+                                 params={"fence_compute": True})
+
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=True,
+                                                  timeout=None,
+                                                  delete_ports_on_failure=None)
+        profile.do_create.assert_called_once_with(obj)
+
+    def test_do_recover_with_delete_timeout(self):
+        profile = self._create_profile('test-profile')
+        self.patchobject(profile, 'do_create', return_value=True)
+        self.patchobject(profile, 'do_delete', return_value=True)
+        obj = mock.Mock()
+
+        res = profile.do_recover(obj, ignore_missing=True, delete_timeout=5)
+
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=5,
+                                                  delete_ports_on_failure=None)
+        profile.do_create.assert_called_once_with(obj)
+
+    def test_do_recover_with_force_recreate(self):
+        profile = self._create_profile('test-profile')
+        self.patchobject(profile, 'do_create', return_value=True)
+        self.patchobject(profile, 'do_delete', return_value=True)
+        obj = mock.Mock()
+
+        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True)
+
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=None,
+                                                  delete_ports_on_failure=True)
+        profile.do_create.assert_called_once_with(obj)
+
+    def test_do_recover_with_force_recreate_failed_delete(self):
+        profile = self._create_profile('test-profile')
+        self.patchobject(profile, 'do_create', return_value=True)
+        err = exception.EResourceDeletion(type='STACK', id='ID',
+                                          message='BANG')
+        self.patchobject(profile, 'do_delete', side_effect=err)
+        obj = mock.Mock()
+
+        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True)
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=None,
+                                                  delete_ports_on_failure=True)
+        profile.do_create.assert_called_once_with(obj)
+
+    def test_do_recover_with_false_force_recreate_failed_delete(self):
+        profile = self._create_profile('test-profile')
+        err = exception.EResourceDeletion(type='STACK', id='ID',
+                                          message='BANG')
+        self.patchobject(profile, 'do_delete', side_effect=err)
+        operation = [{"name": "RECREATE"}]
+
+        ex = self.assertRaises(exception.EResourceOperation,
+                               profile.do_recover,
+                               mock.Mock(id='NODE_ID'), operation=operation,
+                               force_recreate=False)
+        self.assertEqual("Failed in recovering node 'NODE_ID': "
+                         "Failed in deleting STACK 'ID': BANG.",
+                         six.text_type(ex))
 
     def test_do_recover_with_recreate_succeeded(self):
         profile = self._create_profile('test-profile')
-
         self.patchobject(profile, 'do_delete', return_value=True)
         self.patchobject(profile, 'do_create', return_value=True)
-        res = profile.do_recover(mock.Mock(), operation='RECREATE')
+        operation = [{"name": "RECREATE"}]
+        res = profile.do_recover(mock.Mock(), operation=operation)
+
         self.assertTrue(res)
 
     def test_do_recover_with_recreate_failed_delete(self):
         profile = self._create_profile('test-profile')
+        err = exception.EResourceDeletion(type='STACK', id='ID',
+                                          message='BANG')
+        self.patchobject(profile, 'do_delete', side_effect=err)
+        operation = [{"name": "RECREATE"}]
 
-        # Fail because do_delete not implemented
-        res = profile.do_recover(mock.Mock(), operation='RECREATE')
-        self.assertEqual(NotImplemented, res)
-
-        # Failed in do_delete
-        self.patchobject(profile, 'do_delete', return_value=False)
-        res = profile.do_recover(mock.Mock(), operation='RECREATE')
-        self.assertFalse(res)
+        ex = self.assertRaises(exception.EResourceOperation,
+                               profile.do_recover,
+                               mock.Mock(id='NODE_ID'), operation=operation)
+        self.assertEqual("Failed in recovering node 'NODE_ID': "
+                         "Failed in deleting STACK 'ID': BANG.",
+                         six.text_type(ex))
 
     def test_do_recover_with_recreate_failed_create(self):
         profile = self._create_profile('test-profile')
-
-        # Failed because do_create is not implemented (weird case)
         self.patchobject(profile, 'do_delete', return_value=True)
-        res = profile.do_recover(mock.Mock(), operation='RECREATE')
-        self.assertEqual(NotImplemented, res)
+        err = exception.EResourceCreation(type='STACK', message='BANNG')
+        self.patchobject(profile, 'do_create', side_effect=err)
+        operation = [{"name": "RECREATE"}]
 
-        # Failed in do_create
-        self.patchobject(profile, 'do_delete', return_value=True)
-        self.patchobject(profile, 'do_create', return_value=False)
-        res = profile.do_recover(mock.Mock(), operation='RECREATE')
-        self.assertFalse(res)
+        ex = self.assertRaises(exception.EResourceOperation,
+                               profile.do_recover,
+                               mock.Mock(id='NODE_ID'), operation=operation)
 
-        # Failed in do_create with exception
-        self.patchobject(profile, 'do_create', side_effect=Exception)
-        res = profile.do_recover(mock.Mock(), operation='RECREATE')
-        self.assertFalse(res)
+        msg = ("Failed in recovering node 'NODE_ID': Failed in creating "
+               "STACK: BANNG.")
+        self.assertEqual(msg, six.text_type(ex))
 
     def test_to_dict(self):
         profile = self._create_profile('test-profile')
@@ -675,7 +882,7 @@ class TestProfileBase(base.SenlinTestCase):
             'domain': profile.domain,
             'spec': profile.spec,
             'metadata': profile.metadata,
-            'created_at': common_utils.format_time(profile.created_at),
+            'created_at': common_utils.isotime(profile.created_at),
             'updated_at': None,
         }
 
@@ -690,9 +897,9 @@ class TestProfileBase(base.SenlinTestCase):
         new_spec['properties']['key1'] = 'new_v1'
         new_spec['properties']['key2'] = 3
         new_profile = pb.Profile('new-profile', new_spec,
-                                 user=self.ctx.user,
-                                 project=self.ctx.project,
-                                 domain=self.ctx.domain,
+                                 user=self.ctx.user_id,
+                                 project=self.ctx.project_id,
+                                 domain=self.ctx.domain_id,
                                  context=None)
         res = profile.validate_for_update(new_profile)
         self.assertTrue(res)
@@ -704,9 +911,9 @@ class TestProfileBase(base.SenlinTestCase):
         new_spec = copy.deepcopy(self.spec)
         new_spec['properties']['key3'] = 'new_v3'
         new_profile = pb.Profile('new-profile', new_spec,
-                                 user=self.ctx.user,
-                                 project=self.ctx.project,
-                                 domain=self.ctx.domain,
+                                 user=self.ctx.user_id,
+                                 project=self.ctx.project_id,
+                                 domain=self.ctx.domain_id,
                                  context=None)
 
         res = profile.validate_for_update(new_profile)
